@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ class CControllerMenuPopup extends CController {
 
 	protected function checkInput() {
 		$fields = [
-			'type' => 'required|in dashboard,history,host,item,item_prototype,map_element,refresh,trigger,trigger_macro',
+			'type' => 'required|in dashboard,history,host,item,item_prototype,map_element,refresh,trigger,trigger_macro,widget_actions',
 			'data' => 'array'
 		];
 
@@ -35,7 +35,7 @@ class CControllerMenuPopup extends CController {
 				$output['errors'] = $messages->toString();
 			}
 
-			$this->setResponse(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]));
+			$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
 		}
 
 		return $ret;
@@ -128,18 +128,28 @@ class CControllerMenuPopup extends CController {
 
 		$db_hosts = $has_goto
 			? API::Host()->get([
-				'output' => ['status'],
+				'output' => ['hostid', 'status'],
 				'selectGraphs' => API_OUTPUT_COUNT,
 				'selectScreens' => API_OUTPUT_COUNT,
+				'selectHttpTests' => API_OUTPUT_COUNT,
 				'hostids' => $data['hostid']
 			])
 			: API::Host()->get([
-				'output' => [],
+				'output' => ['hostid'],
 				'hostids' => $data['hostid']
 			]);
 
 		if ($db_hosts) {
 			$db_host = $db_hosts[0];
+			$rw_hosts = false;
+
+			if ($has_goto && CWebUser::getType() > USER_TYPE_ZABBIX_USER) {
+				$rw_hosts = (bool) API::Host()->get([
+					'output' => [],
+					'hostids' => $db_host['hostid'],
+					'editable' => true
+				]);
+			}
 
 			$scripts = API::Script()->getScriptsByHosts([$data['hostid']])[$data['hostid']];
 
@@ -159,9 +169,12 @@ class CControllerMenuPopup extends CController {
 			if ($has_goto) {
 				$menu_data['showGraphs'] = (bool) $db_host['graphs'];
 				$menu_data['showScreens'] = (bool) $db_host['screens'];
+				$menu_data['showWeb'] = (bool) $db_host['httpTests'];
+				$menu_data['showConfig'] = (CWebUser::getType() > USER_TYPE_ZABBIX_USER);
+				$menu_data['isWriteable'] = $rw_hosts;
 				$menu_data['showTriggers'] = ($db_host['status'] == HOST_STATUS_MONITORED);
 				if (array_key_exists('severity_min', $data)) {
-					$menu_data['severity_min'] = $data['severity_min'];
+					$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 				}
 				if (array_key_exists('show_suppressed', $data)) {
 					$menu_data['show_suppressed'] = $data['show_suppressed'];
@@ -202,7 +215,7 @@ class CControllerMenuPopup extends CController {
 	 */
 	private static function getMenuDataItem(array $data) {
 		$db_items = API::Item()->get([
-			'output' => ['hostid', 'name', 'value_type'],
+			'output' => ['hostid', 'name', 'value_type', 'flags'],
 			'itemids' => $data['itemid'],
 			'webitems' => true
 		]);
@@ -213,7 +226,9 @@ class CControllerMenuPopup extends CController {
 				'type' => 'item',
 				'itemid' => $data['itemid'],
 				'hostid' => $db_item['hostid'],
-				'name' => $db_item['name']
+				'name' => $db_item['name'],
+				'create_dependent_item' => ($db_item['flags'] != ZBX_FLAG_DISCOVERY_CREATED),
+				'create_dependent_discovery' => ($db_item['flags'] != ZBX_FLAG_DISCOVERY_CREATED)
 			];
 
 			if (in_array($db_item['value_type'], [ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_TEXT])) {
@@ -283,6 +298,49 @@ class CControllerMenuPopup extends CController {
 		return null;
 	}
 
+	/**
+	 * Combines map URLs with element's URLs and performs other modifications with the URLs.
+	 *
+	 * @param array  $selement
+	 * @param array  $map_urls
+	 *
+	 * @return array
+	 */
+	private static function prepareMapElementUrls(array $selement, array $map_urls) {
+		// Remove unused selement url data.
+		foreach ($selement['urls'] as &$url) {
+			unset($url['sysmapelementurlid'], $url['selementid']);
+		}
+		unset($url);
+
+		// Add map urls to element's urls based on their type.
+		foreach ($map_urls as $map_url) {
+			if ($selement['elementtype'] == $map_url['elementtype']) {
+				unset($map_url['elementtype']);
+				$selement['urls'][] = $map_url;
+			}
+		}
+
+		$selement = CMacrosResolverHelper::resolveMacrosInMapElements([$selement], ['resolve_element_urls' => true])[0];
+
+		// Unset URLs with empty name or value.
+		foreach ($selement['urls'] as $url_nr => $url) {
+			if ($url['name'] === '' || $url['url'] === '') {
+				unset($selement['urls'][$url_nr]);
+			}
+			elseif (CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false]) === false) {
+				$selement['urls'][$url_nr]['url'] = 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.',
+					zbx_jsvalue($url['url'], false, false)).'\');';
+			}
+		}
+
+		CArrayHelper::sort($selement['urls'], ['name']);
+		// Prepare urls for processing in menupopup.js.
+		$selement['urls'] = CArrayHelper::renameObjectsKeys($selement['urls'], ['name' => 'label']);
+
+		return $selement;
+	}
+
 	/**
 	 * Prepare data for map element context menu popup.
 	 *
@@ -300,8 +358,8 @@ class CControllerMenuPopup extends CController {
 		$db_maps = API::Map()->get([
 			'output' => ['show_suppressed'],
 			'selectSelements' => ['selementid', 'elementtype', 'elementsubtype', 'elements', 'urls', 'application'],
-			'sysmapids' => $data['sysmapid'],
-			'expandUrls' => true
+			'selectUrls' => ['name', 'url', 'elementtype'],
+			'sysmapids' => $data['sysmapid']
 		]);
 
 		if ($db_maps) {
@@ -316,26 +374,16 @@ class CControllerMenuPopup extends CController {
 			}
 
 			if ($selement !== null) {
-				CArrayHelper::sort($selement['urls'], ['name']);
-				$selement['urls'] = CArrayHelper::renameObjectsKeys($selement['urls'], ['name' => 'label']);
-
 				if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST_GROUP
 						&& $selement['elementsubtype'] == SYSMAP_ELEMENT_SUBTYPE_HOST_GROUP_ELEMENTS
 						&& array_key_exists('hostid', $data) && $data['hostid'] != 0) {
 					$selement['elementtype'] = SYSMAP_ELEMENT_TYPE_HOST;
 					$selement['elementsubtype'] = SYSMAP_ELEMENT_SUBTYPE_HOST_GROUP;
 					$selement['elements'][0]['hostid'] = $data['hostid'];
-
-					/*
-					 * Expanding host URL macros again as some hosts were added from hostgroup areas and automatic
-					 * expanding only happens for elements that are defined for map in DB.
-					 */
-					foreach ($selement['urls'] as &$url) {
-						$url['url'] = str_replace('{HOST.ID}', $data['hostid'], $url['url']);
-					}
-					unset($url);
 				}
 
+				$selement = self::prepareMapElementUrls($selement, $db_map['urls']);
+
 				switch ($selement['elementtype']) {
 					case SYSMAP_ELEMENT_TYPE_MAP:
 						$menu_data = [
@@ -359,7 +407,7 @@ class CControllerMenuPopup extends CController {
 							'groupid' => $selement['elements'][0]['groupid']
 						];
 						if (array_key_exists('severity_min', $data)) {
-							$menu_data['severity_min'] = $data['severity_min'];
+							$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 						}
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
@@ -396,7 +444,7 @@ class CControllerMenuPopup extends CController {
 							'triggerids' => zbx_objectValues($selement['elements'], 'triggerid')
 						];
 						if (array_key_exists('severity_min', $data)) {
-							$menu_data['severity_min'] = $data['severity_min'];
+							$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 						}
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
@@ -454,15 +502,13 @@ class CControllerMenuPopup extends CController {
 	 *
 	 * @param array  $data
 	 * @param string $data['triggerid']
-	 * @param string $data['eventid']                 (optional) Mandatory for Acknowledge and Description menus.
-	 * @param array  $data['acknowledge']             (optional) Acknowledge link parameters.
-	 * @param string $data['acknowledge']['backurl']
+	 * @param string $data['eventid']                 (optional) Mandatory for Acknowledge menu.
+	 * @param bool   $data['acknowledge']             (optional) Whether to show Acknowledge menu.
 	 * @param int    $data['severity_min']            (optional)
 	 * @param bool   $data['show_suppressed']         (optional)
 	 * @param array  $data['urls']                    (optional)
 	 * @param string $data['urls']['name']
 	 * @param string $data['urls']['url']
-	 * @param bool   $data['show_description']        (optional) default: true
 	 *
 	 * @return mixed
 	 */
@@ -476,11 +522,15 @@ class CControllerMenuPopup extends CController {
 		]);
 
 		if ($db_triggers) {
-			$db_triggers = CMacrosResolverHelper::resolveTriggerUrls($db_triggers);
-
 			$db_trigger = reset($db_triggers);
 			$db_trigger['items'] = CMacrosResolverHelper::resolveItemNames($db_trigger['items']);
 
+			if (array_key_exists('eventid', $data)) {
+				$db_trigger['eventid'] = $data['eventid'];
+			}
+
+			$db_trigger['url'] = CMacrosResolverHelper::resolveTriggerUrl($db_trigger, $url) ? $url : '';
+
 			$hosts = [];
 			$show_events = true;
 
@@ -517,50 +567,54 @@ class CControllerMenuPopup extends CController {
 				];
 			}
 
-			$options = [
-				'show_description' => !array_key_exists('show_description', $data) || $data['show_description']
-			];
-
-			if ($options['show_description']) {
-				$rw_triggers = API::Trigger()->get([
-					'output' => ['flags'],
-					'triggerids' => $data['triggerid'],
-					'editable' => true
-				]);
-
-				$options['description_enabled'] = ($db_trigger['comments'] !== ''
-					|| ($rw_triggers && $rw_triggers[0]['flags'] == ZBX_FLAG_DISCOVERY_NORMAL));
-			}
-
 			$menu_data = [
 				'type' => 'trigger',
 				'triggerid' => $data['triggerid'],
 				'items' => $items,
 				'showEvents' => $show_events,
-				'configuration' =>
-					in_array(CWebUser::$data['type'], [USER_TYPE_ZABBIX_ADMIN, USER_TYPE_SUPER_ADMIN])
+				'configuration' => in_array(CWebUser::$data['type'], [USER_TYPE_ZABBIX_ADMIN, USER_TYPE_SUPER_ADMIN])
 			];
 
-			if (!$options['show_description']) {
-				$menu_data['show_description'] = false;
-			}
-			else if (!$options['description_enabled']) {
-				$menu_data['description_enabled'] = false;
+			if ($db_trigger['url'] !== '') {
+				$menu_data['urls'][] = [
+					'label' => _('Trigger URL'),
+					'url' => $db_trigger['url']
+				];
 			}
 
 			if (array_key_exists('eventid', $data)) {
 				$menu_data['eventid'] = $data['eventid'];
+
+				$events = API::Event()->get([
+					'output' => ['urls'],
+					'eventids' => $data['eventid']
+				]);
+
+				if ($events) {
+					foreach ($events[0]['urls'] as $url) {
+						$menu_data['urls'][] = [
+							'label' => $url['name'],
+							'url' => $url['url'],
+							'target' => '_blank'
+						];
+					}
+				}
+			}
+
+			if (array_key_exists('urls', $menu_data)) {
+				foreach ($menu_data['urls'] as &$url) {
+					if (!CHtmlUrlValidator::validate($url['url'], ['allow_user_macro' => false])) {
+						$url['url'] = 'javascript: alert(\''.
+							_s('Provided URL "%1$s" is invalid.', zbx_jsvalue($url['url'], false, false)).
+						'\');';
+						unset($url['target']);
+					}
+				}
+				unset($url);
 			}
 
 			if (array_key_exists('acknowledge', $data)) {
-				$menu_data['acknowledge'] = $data['acknowledge'];
-			}
-
-			if ($db_trigger['url'] !== '') {
-				$menu_data['url'] = CHtmlUrlValidator::validate($db_trigger['url'])
-					? $db_trigger['url']
-					: 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.', zbx_jsvalue($db_trigger['url'],
-							false, false)).'\');';
+				$menu_data['acknowledge'] = (bool) $data['acknowledge'];
 			}
 
 			return $menu_data;
@@ -580,6 +634,49 @@ class CControllerMenuPopup extends CController {
 		return ['type' => 'trigger_macro'];
 	}
 
+	/**
+	 * Prepare data for widget actions menu popup.
+	 *
+	 * @param array  $data
+	 * @param string $data['widget_uniqueid']  Widget instance unique id.
+	 * @param string $data['currentRate']      Refresh rate for widget.
+	 * @param bool   $data['multiplier']       Multiplier or time mode.
+	 *
+	 * @return mixed
+	 */
+	private static function getMenuDataWidgetActions(array $data) {
+		$menu_data = [
+			'type' => 'widget_actions',
+			'widget_uniqueid' => $data['widget_uniqueid'],
+			'currentRate' => $data['currentRate'],
+			'multiplier' => (bool) $data['multiplier']
+		];
+
+		if ($data['widgetType'] == WIDGET_SVG_GRAPH) {
+			$menu_data['download'] = true;
+		}
+
+		if (array_key_exists('params', $data)) {
+			$menu_data['params'] = $data['params'];
+		}
+
+		if ($data['widgetType'] == WIDGET_GRAPH && array_key_exists('graphid', $data) && $data['graphid']) {
+			$menu_data['download'] = (bool) API::Graph()->get([
+				'output' => ['graphid'],
+				'graphids' => $data['graphid']
+			]);
+		}
+		elseif ($data['widgetType'] == WIDGET_GRAPH && array_key_exists('itemid', $data) && $data['itemid']) {
+			$menu_data['download'] = (bool) API::Item()->get([
+				'output' => ['itemid'],
+				'itemids' => $data['itemid'],
+				'webitems' => true
+			]);
+		}
+
+		return $menu_data;
+	}
+
 	protected function doAction() {
 		$data = $this->hasInput('data') ? $this->getInput('data') : [];
 
@@ -619,6 +716,10 @@ class CControllerMenuPopup extends CController {
 			case 'trigger_macro':
 				$menu_data = self::getMenuDataTriggerMacro();
 				break;
+
+			case 'widget_actions':
+				$menu_data = self::getMenuDataWidgetActions($data);
+				break;
 		}
 
 		$output = [];
@@ -631,6 +732,6 @@ class CControllerMenuPopup extends CController {
 			$output['errors'] = $messages->toString();
 		}
 
-		$this->setResponse(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]));
+		$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
 	}
 }
