@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,12 +28,13 @@ class CControllerPopupHttpStep extends CController {
 	protected function checkInput() {
 
 		$fields = [
-			'httpstepid' =>			'int32',
+			'no' =>					'int32',
+			'httpstepid' =>			'db httpstep.httpstepid',
 			'name' =>				'string|not_empty',
 			'url' =>				'string|not_empty',
 			'post_type' =>			'in '.implode(',', [ZBX_POSTTYPE_RAW, ZBX_POSTTYPE_FORM]),
 			'posts' =>				'string',
-			'retrieve_mode' =>		'in '.implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS]),
+			'retrieve_mode' =>		'in '.implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS, HTTPTEST_STEP_RETRIEVE_MODE_BOTH]),
 			'follow_redirects' =>	'in '.implode(',', [HTTPTEST_STEP_FOLLOW_REDIRECTS_ON, HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF]),
 			'timeout' =>			'string|not_empty',
 			'required' =>			'string',
@@ -53,7 +54,7 @@ class CControllerPopupHttpStep extends CController {
 			}
 
 			$this->setResponse(
-				(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]))->disableView()
+				(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 			);
 		}
 
@@ -75,7 +76,8 @@ class CControllerPopupHttpStep extends CController {
 			'required' => $this->getInput('required', ''),
 			'status_codes' => $this->getInput('status_codes', ''),
 			'old_name' => $this->getInput('old_name', ''),
-			'httpstepid' => $this->getInput('httpstepid', -1),
+			'httpstepid' => $this->getInput('httpstepid', 0),
+			'no' => $this->getInput('no', -1),
 			'steps_names' => $this->getInput('steps_names', [])
 		];
 
@@ -94,8 +96,10 @@ class CControllerPopupHttpStep extends CController {
 			elseif ($page_options['timeout'][0] !== '{') {
 				$seconds = timeUnitToSeconds($page_options['timeout']);
 
-				if (bccomp($seconds, SEC_PER_HOUR) > 0) {
-					error(_s('Incorrect value for field "%1$s": %2$s.', 'timeout', _('a number is too large')));
+				if ($seconds < 1 || $seconds > SEC_PER_HOUR) {
+					error(_s('Invalid parameter "%1$s": %2$s.', 'timeout',
+						_s('value must be one of %1$s', '1-'.SEC_PER_HOUR)
+					));
 				}
 			}
 
@@ -123,18 +127,17 @@ class CControllerPopupHttpStep extends CController {
 					'required' => $page_options['required'],
 					'status_codes' => $page_options['status_codes'],
 					'follow_redirects' => $page_options['follow_redirects'],
-					'retrieve_mode' => $page_options['retrieve_mode']
+					'retrieve_mode' => $page_options['retrieve_mode'],
+					'no' => $page_options['no']
 				];
 
-				$params['httpstepid'] = $page_options['httpstepid'];
-
 				$output = [
 					'params' => $params
 				];
 			}
 
 			$this->setResponse(
-				(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]))->disableView()
+				(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 			);
 		}
 		else {
