@@ -67,9 +67,13 @@ class CControllerProxyHostEnable extends CController {
 
 		$updated = count($hosts);
 
-		$response = new CControllerResponseRedirect('zabbix.php?action=proxy.list&uncheck=1');
+		$response = new CControllerResponseRedirect((new CUrl('zabbix.php'))
+			->setArgument('action', 'proxy.list')
+			->setArgument('page', CPagerHelper::loadPage('proxy.list', null))
+		);
 
 		if ($result) {
+			$response->setFormData(['uncheck' => '1']);
 			$response->setMessageOk(_n('Host enabled', 'Hosts enabled', $updated));
 		}
 		else {
