@@ -27,7 +27,8 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 		$this->setType(WIDGET_PROBLEMS);
 		$this->setValidationRules([
 			'name' => 'string',
-			'fields' => 'json'
+			'fields' => 'json',
+			'initial_load' => 'in 0,1'
 		]);
 	}
 
@@ -47,7 +48,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 			'tags' => $fields['tags'],
 			'show_suppressed' => $fields['show_suppressed'],
 			'unacknowledged' => $fields['unacknowledged'],
-			'show_latest_values' => $fields['show_latest_values']
+			'show_opdata' => $fields['show_opdata']
 		], $config, true, true);
 		list($sortfield, $sortorder) = self::getSorting($fields['sort_triggers']);
 		$data = CScreenProblem::sortData($data, $config, $sortfield, $sortorder);
@@ -67,7 +68,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 		$data = CScreenProblem::makeData($data, [
 			'show' => $fields['show'],
 			'details' => 0,
-			'show_latest_values' => $fields['show_latest_values']
+			'show_opdata' => $fields['show_opdata']
 		], true, true);
 
 		if ($fields['show_tags']) {
@@ -82,6 +83,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 
 		$this->setResponse(new CControllerResponseData([
 			'name' => $this->getInput('name', $this->getDefaultHeader()),
+			'initial_load' => (bool) $this->getInput('initial_load', 0),
 			'fields' => [
 				'show' => $fields['show'],
 				'show_lines' => $fields['show_lines'],
@@ -90,7 +92,7 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 				'tags' => $fields['tags'],
 				'tag_name_format' => $fields['tag_name_format'],
 				'tag_priority' => $fields['tag_priority'],
-				'show_latest_values' => $fields['show_latest_values']
+				'show_opdata' => $fields['show_opdata']
 			],
 			'config' => [
 				'problem_unack_style' => $config['problem_unack_style'],
