@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,47 +19,63 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 if ($data['uncheck']) {
 	uncheckTableRows('mediatype');
 }
 
 $widget = (new CWidget())
 	->setTitle(_('Media types'))
-	->setControls((new CForm())
-		->cleanItems()
-		->addItem((new CList())->addItem(new CRedirectButton(_('Create media type'), 'zabbix.php?action=mediatype.edit')))
+	->setControls((new CTag('nav', true,
+		(new CList())
+			->addItem(new CRedirectButton(_('Create media type'), 'zabbix.php?action=mediatype.edit'))
+			->addItem(new CRedirectButton(_('Import'), 'conf.import.php?rules_preset=mediatype'))
+		))
+			->setAttribute('aria-label', _('Content controls'))
 	)
-	->addItem((new CFilter('web.media_types.filter.state'))
+	->addItem((new CFilter((new CUrl('zabbix.php'))->setArgument('action', 'mediatype.list')))
+		->setProfile($data['profileIdx'])
+		->setActiveTab($data['active_tab'])
+		->addFilterTab(_('Filter'), [
+			(new CFormList())->addRow(_('Name'),
+				(new CTextBox('filter_name', $data['filter']['name']))
+					->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+					->setAttribute('autofocus', 'autofocus')
+			),
+			(new CFormList())->addRow(_('Status'),
+				(new CRadioButtonList('filter_status', (int) $data['filter']['status']))
+					->addValue(_('Any'), -1)
+					->addValue(_('Enabled'), MEDIA_TYPE_STATUS_ACTIVE)
+					->addValue(_('Disabled'), MEDIA_TYPE_STATUS_DISABLED)
+					->setModern(true)
+			)
+		])
 		->addVar('action', 'mediatype.list')
-		->addColumn((new CFormList())->addRow(_('Name'),
-			(new CTextBox('filter_name', $data['filter']['name']))
-				->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
-				->setAttribute('autofocus', 'autofocus')
-		))
-		->addColumn((new CFormList())->addRow(_('Status'),
-			(new CRadioButtonList('filter_status', (int) $data['filter']['status']))
-				->addValue(_('Any'), -1)
-				->addValue(_('Enabled'), MEDIA_TYPE_STATUS_ACTIVE)
-				->addValue(_('Disabled'), MEDIA_TYPE_STATUS_DISABLED)
-				->setModern(true)
-		))
 	);
 
 // create form
 $mediaTypeForm = (new CForm())->setName('mediaTypesForm');
 
 // create table
+$url = (new CUrl('zabbix.php'))
+	->setArgument('action', 'mediatype.list')
+	->getUrl();
+
 $mediaTypeTable = (new CTableInfo())
 	->setHeader([
 		(new CColHeader(
 			(new CCheckBox('all_media_types'))
 				->onClick("checkAll('".$mediaTypeForm->getName()."', 'all_media_types', 'mediatypeids');")
 		))->addClass(ZBX_STYLE_CELL_WIDTH),
-		make_sorting_header(_('Name'), 'description', $data['sort'], $data['sortorder']),
-		make_sorting_header(_('Type'), 'type', $data['sort'], $data['sortorder']),
+		make_sorting_header(_('Name'), 'name', $data['sort'], $data['sortorder'], $url),
+		make_sorting_header(_('Type'), 'type', $data['sort'], $data['sortorder'], $url),
 		_('Status'),
 		_('Used in actions'),
-		_('Details')
+		_('Details'),
+		_('Action')
 	]);
 
 foreach ($data['mediatypes'] as $mediaType) {
@@ -79,14 +95,6 @@ foreach ($data['mediatypes'] as $mediaType) {
 			$details = _('GSM modem').NAME_DELIMITER.'"'.$mediaType['gsm_modem'].'"';
 			break;
 
-		case MEDIA_TYPE_JABBER:
-			$details = _('Jabber identifier').NAME_DELIMITER.'"'.$mediaType['username'].'"';
-			break;
-
-		case MEDIA_TYPE_EZ_TEXTING:
-			$details = _('Username').NAME_DELIMITER.'"'.$mediaType['username'].'"';
-			break;
-
 		default:
 			$details = '';
 			break;
@@ -124,7 +132,15 @@ foreach ($data['mediatypes'] as $mediaType) {
 			->addClass(ZBX_STYLE_RED)
 			->addSID();
 
-	$name = new CLink($mediaType['description'], '?action=mediatype.edit&mediatypeid='.$mediaType['mediatypeid']);
+	$test_link = (new CButton('mediatypetest_edit', _('Test')))
+		->addClass(ZBX_STYLE_BTN_LINK)
+		->removeId()
+		->setEnabled(MEDIA_TYPE_STATUS_ACTIVE == $mediaType['status'])
+		->onClick('return PopUp("popup.mediatypetest.edit",'.json_encode([
+			'mediatypeid' => $mediaType['mediatypeid']
+		]).', "mediatypetest_edit", this);');
+
+	$name = new CLink($mediaType['name'], '?action=mediatype.edit&mediatypeid='.$mediaType['mediatypeid']);
 
 	// append row
 	$mediaTypeTable->addRow([
@@ -133,7 +149,8 @@ foreach ($data['mediatypes'] as $mediaType) {
 		media_type2str($mediaType['typeid']),
 		$status,
 		$actionColumn,
-		$details
+		$details,
+		$test_link
 	]);
 }
 
@@ -144,6 +161,15 @@ $mediaTypeForm->addItem([
 	new CActionButtonList('action', 'mediatypeids', [
 		'mediatype.enable' => ['name' => _('Enable'), 'confirm' => _('Enable selected media types?')],
 		'mediatype.disable' => ['name' => _('Disable'), 'confirm' => _('Disable selected media types?')],
+		'mediatype.export' => ['name' => _('Export'), 'redirect' =>
+			(new CUrl('zabbix.php'))
+				->setArgument('action', 'export.mediatypes.xml')
+				->setArgument('backurl', (new CUrl('zabbix.php'))
+					->setArgument('action', 'mediatype.list')
+					->setArgument('page', $data['page'] == 1 ? null : $data['page'])
+					->getUrl())
+				->getUrl()
+		],
 		'mediatype.delete' => ['name' => _('Delete'), 'confirm' => _('Delete selected media types?')]
 	], 'mediatype')
 ]);
