@@ -19,13 +19,13 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 // indicator of sort field
 $sort_div = (new CSpan())->addClass(($data['sortorder'] === ZBX_SORT_DOWN) ? ZBX_STYLE_ARROW_DOWN : ZBX_STYLE_ARROW_UP);
 
-$backurl = (new CUrl('zabbix.php'))
-	->setArgument('action', 'dashboard.view')
-	->getUrl();
-
 $url_details = (new CUrl('tr_events.php'))
 	->setArgument('triggerid', '')
 	->setArgument('eventid', '');
@@ -46,6 +46,8 @@ else {
 	$header = [$header_time];
 }
 
+$show_opdata = $data['fields']['show_opdata'];
+
 $table = (new CTableInfo())
 	->setHeader(array_merge($header, [
 		$show_recovery_data ? _('Recovery time') : null,
@@ -57,7 +59,7 @@ $table = (new CTableInfo())
 			' &bullet; ',
 			($data['sortfield'] === 'severity') ? [_('Severity'), $sort_div] : _('Severity')
 		],
-		$data['fields']['show_latest_values'] ? _('Latest values') : null,
+		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? _('Operational data') : null,
 		_('Duration'),
 		_('Ack'),
 		_('Actions'),
@@ -103,7 +105,7 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
 	$cell_clock = new CCol(new CLink($cell_clock, $url_details));
 
-	$is_acknowledged = $problem['acknowledged'] == EVENT_ACKNOWLEDGED;
+	$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 	if ($show_recovery_data) {
 		if ($problem['r_eventid'] != 0) {
@@ -149,18 +151,55 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$info_icons[] = makeSuppressedProblemIcon($problem['suppression_data']);
 	}
 
-	$description = (new CCol([
+	$opdata = null;
+	if ($show_opdata != OPERATIONAL_DATA_SHOW_NONE) {
+
+		// operational data
+		if ($trigger['opdata'] === '') {
+			if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
+				$opdata = (new CCol(CScreenProblem::getLatestValues($trigger['items'])))->addClass('latest-values');
+			}
+		} else {
+			$opdata = CMacrosResolverHelper::resolveTriggerOpdata(
+				[
+					'triggerid' => $trigger['triggerid'],
+					'expression' => $trigger['expression'],
+					'opdata' => $trigger['opdata'],
+					'clock' => ($problem['r_eventid'] != 0) ? $problem['r_clock'] : $problem['clock'],
+					'ns' => ($problem['r_eventid'] != 0) ? $problem['r_ns'] : $problem['ns']
+				],
+				[
+					'events' => true,
+					'html' => true
+				]
+			);
+
+			if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
+				$opdata = (new CCol($opdata))
+					->addClass('opdata')
+					->addClass(ZBX_STYLE_WORDWRAP);
+			}
+		}
+	}
+
+	$problem_link = [
 		(new CLinkAction($problem['name']))
 			->setHint(
-				make_popup_eventlist(['comments' => $problem['comments']] + $trigger, $eventid, $backurl,
-					$show_timeline, $data['fields']['show_tags'], $data['fields']['tags'],
-					$data['fields']['tag_name_format'], $data['fields']['tag_priority']
+				make_popup_eventlist(['comments' => $problem['comments'], 'url' => $problem['url'],
+					'triggerid' => $trigger['triggerid']], $eventid, $show_timeline, $data['fields']['show_tags'],
+					$data['fields']['tags'], $data['fields']['tag_name_format'], $data['fields']['tag_priority']
 				)
 			)
 			->setAttribute('aria-label', _xs('%1$s, Severity, %2$s', 'screen reader',
 				$problem['name'], getSeverityName($problem['severity'], $data['config'])
 			))
-	]));
+	];
+
+	if ($show_opdata == OPERATIONAL_DATA_SHOW_WITH_PROBLEM && $opdata) {
+		$problem_link = array_merge($problem_link, [' (', $opdata, ')']);
+	}
+
+	$description = (new CCol($problem_link));
 
 	$description_style = getSeverityStyle($problem['severity']);
 
@@ -202,12 +241,11 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		];
 	}
 
-	// Create acknowledge url.
-	$problem_update_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'acknowledge.edit')
-		->setArgument('eventids', [$problem['eventid']])
-		->setArgument('backurl', $backurl)
-		->getUrl();
+	// Create acknowledge link.
+	$problem_update_link = (new CLink($is_acknowledged ? _('Yes') : _('No')))
+		->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+		->addClass(ZBX_STYLE_LINK_ALT)
+		->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);');
 
 	$table->addRow(array_merge($row, [
 		$show_recovery_data ? $cell_r_clock : null,
@@ -215,12 +253,10 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		makeInformationList($info_icons),
 		$triggers_hosts[$trigger['triggerid']],
 		$description,
-		$data['fields']['show_latest_values'] ? CScreenProblem::getLatestValues($trigger['items']) : null,
+		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY ) ? $opdata : null,
 		(new CCol(zbx_date2age($problem['clock'], ($problem['r_eventid'] != 0) ? $problem['r_clock'] : 0)))
 			->addClass(ZBX_STYLE_NOWRAP),
-		(new CLink($problem['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-			->addClass($problem['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-			->addClass(ZBX_STYLE_LINK_ALT),
+		$problem_update_link,
 		makeEventActionsIcons($problem['eventid'], $data['data']['actions'], $data['data']['mediatypes'],
 			$data['data']['users'], $data['config']
 		),
@@ -242,6 +278,17 @@ $output = [
 	'body' => $table->toString()
 ];
 
+if ($data['initial_load']) {
+	$output['script_inline'] =
+		'if (typeof refreshProblemsWidget !== typeof(Function)) {'.
+			'function refreshProblemsWidget(event, response, overlay) {'.
+				'refreshWidgetOnAcknowledgeCreate("problems", response, overlay);'.
+			'}'.
+
+			'$.subscribe("acknowledge.create", refreshProblemsWidget);'.
+		'}';
+}
+
 if (($messages = getMessages()) !== null) {
 	$output['messages'] = $messages->toString();
 }
@@ -251,4 +298,4 @@ if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 	$output['debug'] = CProfiler::getInstance()->make()->toString();
 }
 
-echo (new CJson())->encode($output);
+echo json_encode($output);
