@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,36 +28,38 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'period' =>			[T_ZBX_INT, O_OPT, P_NZERO,	BETWEEN(ZBX_MIN_PERIOD, ZBX_MAX_PERIOD), null],
-	'stime' =>			[T_ZBX_INT, O_OPT, P_NZERO,	null,				null],
-	'isNow' =>			[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'profileIdx' =>		[T_ZBX_STR, O_OPT, null,		null,				null],
-	'profileIdx2' =>	[T_ZBX_STR, O_OPT, null,		null,				null],
-	'httptestid' =>		[T_ZBX_INT, O_OPT, P_NZERO,	null,				null],
-	'http_item_type' =>	[T_ZBX_INT, O_OPT, null,		null,				null],
-	'name' =>			[T_ZBX_STR, O_OPT, null,		null,				null],
-	'width' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_WIDTH_MIN, 65535),	null],
-	'height' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_HEIGHT_MIN, 65535),	null],
-	'ymin_type' =>		[T_ZBX_INT, O_OPT, null,		IN('0,1,2'),		null],
-	'ymax_type' =>		[T_ZBX_INT, O_OPT, null,		IN('0,1,2'),		null],
-	'ymin_itemid' =>	[T_ZBX_INT, O_OPT, null,		DB_ID,				null],
-	'ymax_itemid' =>	[T_ZBX_INT, O_OPT, null,		DB_ID,				null],
-	'legend' =>			[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'showworkperiod' =>	[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'showtriggers' =>	[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'graphtype' =>		[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'yaxismin' =>		[T_ZBX_DBL, O_OPT, null,		null,				null],
-	'yaxismax' =>		[T_ZBX_DBL, O_OPT, null,		null,				null],
-	'percent_left' =>	[T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100),	null],
-	'percent_right' =>	[T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100),	null],
-	'outer' =>			[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'items' =>			[T_ZBX_STR, O_OPT, null,		null,				null],
-	'onlyHeight' =>		[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null],
-	'widget_view' =>	[T_ZBX_INT, O_OPT, null,		IN('0,1'),			null]
+	'from' =>			[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,		null,				null],
+	'to' =>				[T_ZBX_RANGE_TIME,	O_OPT, P_SYS,		null,				null],
+	'profileIdx' =>		[T_ZBX_STR,			O_OPT, null,		null,				null],
+	'profileIdx2' =>	[T_ZBX_STR,			O_OPT, null,		null,				null],
+	'httptestid' =>		[T_ZBX_INT,			O_OPT, P_NZERO,	null,				null],
+	'http_item_type' =>	[T_ZBX_INT,			O_OPT, null,		null,				null],
+	'name' =>			[T_ZBX_STR,			O_OPT, null,		null,				null],
+	'width' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_WIDTH_MIN, 65535),	null],
+	'height' =>			[T_ZBX_INT,			O_OPT, null,	BETWEEN(CLineGraphDraw::GRAPH_HEIGHT_MIN, 65535),	null],
+	'ymin_type' =>		[T_ZBX_INT,			O_OPT, null,		IN('0,1,2'),		null],
+	'ymax_type' =>		[T_ZBX_INT,			O_OPT, null,		IN('0,1,2'),		null],
+	'ymin_itemid' =>	[T_ZBX_INT,			O_OPT, null,		DB_ID,				null],
+	'ymax_itemid' =>	[T_ZBX_INT,			O_OPT, null,		DB_ID,				null],
+	'legend' =>			[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
+	'showworkperiod' =>	[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
+	'showtriggers' =>	[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
+	'graphtype' =>		[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
+	'yaxismin' =>		[T_ZBX_DBL,			O_OPT, null,		null,				null],
+	'yaxismax' =>		[T_ZBX_DBL,			O_OPT, null,		null,				null],
+	'percent_left' =>	[T_ZBX_DBL,			O_OPT, null,		BETWEEN_DBL(0, 100, 4),	null],
+	'percent_right' =>	[T_ZBX_DBL,			O_OPT, null,		BETWEEN_DBL(0, 100, 4),	null],
+	'outer' =>			[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
+	'items' =>			[T_ZBX_STR,			O_OPT, null,		null,				null],
+	'i' =>				[T_ZBX_STR,			O_OPT, null,		null,				null],
+	'onlyHeight' =>		[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null],
+	'widget_view' =>	[T_ZBX_INT,			O_OPT, null,		IN('0,1'),			null]
 ];
 if (!check_fields($fields)) {
 	exit();
 }
+validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
+
 $graph_items = [];
 
 if ($httptestid = getRequest('httptestid', false)) {
@@ -98,7 +100,14 @@ if ($httptestid = getRequest('httptestid', false)) {
 
 	$name = getRequest('name', '');
 }
-elseif ($items = getRequest('items', [])) {
+elseif (hasRequest('i') || hasRequest('items')) {
+	if (hasRequest('i')) {
+		$items = array_map('expandShortGraphItem', getRequest('i', []));
+	}
+	else {
+		$items = getRequest('items', []);
+	}
+
 	CArrayHelper::sort($items, ['sortorder']);
 
 	$dbItems = API::Item()->get([
@@ -119,12 +128,17 @@ elseif ($items = getRequest('items', [])) {
 			access_deny();
 		}
 		$host = reset($dbItems[$item['itemid']]['hosts']);
-		unset($dbItems[$item['itemid']]['hosts']);
 		$graph_items[] = $dbItems[$item['itemid']] + $item + [
 			'host' => $host['host'],
 			'hostname' => $host['name']
 		];
 	}
+
+	foreach ($graph_items as &$graph_item) {
+		unset($graph_item['hosts']);
+	}
+	unset($graph_item);
+
 	$name = getRequest('name', '');
 }
 else {
@@ -135,24 +149,19 @@ else {
 /*
  * Display
  */
-$profileIdx = getRequest('profileIdx', 'web.httptest');
-$profileIdx2 = getRequest('httptestid', getRequest('profileIdx2'));
-
-$timeline = calculateTime([
-	'profileIdx' => $profileIdx,
-	'profileIdx2' => $profileIdx2,
-	'updateProfile' => false,
-	'period' => getRequest('period'),
-	'stime' => getRequest('stime'),
-	'isNow' => getRequest('isNow')
+$timeline = getTimeSelectorPeriod([
+	'profileIdx' => getRequest('profileIdx', 'web.httpdetails.filter'),
+	'profileIdx2' => getRequest('httptestid', getRequest('profileIdx2')),
+	'from' => getRequest('from'),
+	'to' => getRequest('to')
 ]);
 
-CProfile::update($profileIdx.'.httptestid', $profileIdx2, PROFILE_TYPE_ID);
+CProfile::update($timeline['profileIdx'].'.httptestid', $timeline['profileIdx2'], PROFILE_TYPE_ID);
 
 $graph = new CLineGraphDraw(getRequest('graphtype', GRAPH_TYPE_NORMAL));
 $graph->setHeader($name);
-$graph->setPeriod($timeline['period']);
-$graph->setSTime($timeline['stime']);
+$graph->setPeriod($timeline['to_ts'] - $timeline['from_ts']);
+$graph->setSTime($timeline['from_ts']);
 $graph->setWidth(getRequest('width', 900));
 $graph->setHeight(getRequest('height', 200));
 $graph->showLegend(getRequest('legend', 1));
@@ -179,12 +188,7 @@ foreach ($graph_items as $graph_item) {
 
 if (getRequest('onlyHeight', '0') === '1') {
 	$graph->drawDimensions();
-	$height = $graph->getHeight();
-
-	if (getRequest('widget_view') === '1') {
-		$height = $height - CLineGraphDraw::DEFAULT_TOP_BOTTOM_PADDING;
-	}
-	header('X-ZBX-SBOX-HEIGHT: '.$height);
+	header('X-ZBX-SBOX-HEIGHT: '.($graph->getHeight() + 1));
 }
 else {
 	$graph->draw();
