@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,23 +20,24 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
-require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
 require_once dirname(__FILE__).'/include/graphs.inc.php';
 require_once dirname(__FILE__).'/include/forms.inc.php';
 
-$page['title'] = isset($_REQUEST['parent_discoveryid']) ? _('Configuration of graph prototypes') : _('Configuration of graphs');
+$page['title'] = hasRequest('parent_discoveryid') ? _('Configuration of graph prototypes') : _('Configuration of graphs');
 $page['file'] = 'graphs.php';
-$page['scripts'] = ['colorpicker.js'];
+$page['scripts'] = ['colorpicker.js', 'multiselect.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
 	'parent_discoveryid' =>	[T_ZBX_INT, O_OPT, P_SYS,		DB_ID,			null],
-	'groupid' =>			[T_ZBX_INT, O_OPT, P_SYS,		DB_ID,			null],
 	'hostid' =>				[T_ZBX_INT, O_OPT, P_SYS,		DB_ID,			null],
-	'copy_type' => [T_ZBX_INT, O_OPT, P_SYS, IN([COPY_TYPE_TO_HOST, COPY_TYPE_TO_HOST_GROUP, COPY_TYPE_TO_TEMPLATE]), 'isset({copy})'],
+	'copy_type' =>			[T_ZBX_INT, O_OPT, P_SYS,
+								IN([COPY_TYPE_TO_HOST_GROUP, COPY_TYPE_TO_HOST, COPY_TYPE_TO_TEMPLATE]),
+								'isset({copy})'
+							],
 	'copy_mode' =>			[T_ZBX_INT, O_OPT, P_SYS,		IN('0'),		null],
 	'graphid' =>			[T_ZBX_INT, O_OPT, P_SYS,		DB_ID,			'isset({form}) && {form} == "update"'],
 	'name' =>				[T_ZBX_STR, O_OPT, null,		NOT_EMPTY,		'isset({add}) || isset({update})', _('Name')],
@@ -51,17 +52,17 @@ $fields = [
 	'yaxismax' =>			[T_ZBX_DBL, O_OPT, null,		null,			'(isset({add}) || isset({update})) && isset({graphtype}) && ({graphtype} == '.GRAPH_TYPE_NORMAL.' || {graphtype} == '.GRAPH_TYPE_STACKED.')'],
 	'ymin_itemid' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,			'(isset({add}) || isset({update})) && isset({ymin_type}) && {ymin_type} == '.GRAPH_YAXIS_TYPE_ITEM_VALUE],
 	'ymax_itemid' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,			'(isset({add}) || isset({update})) && isset({ymax_type}) && {ymax_type} == '.GRAPH_YAXIS_TYPE_ITEM_VALUE],
-	'percent_left' =>		[T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100), null, _('Percentile line (left)')],
-	'percent_right' =>		[T_ZBX_DBL, O_OPT, null,		BETWEEN(0, 100), null, _('Percentile line (right)')],
+	'percent_left' =>		[T_ZBX_DBL, O_OPT, null,		BETWEEN_DBL(0, 100, 4), null, _('Percentile line (left)')],
+	'percent_right' =>		[T_ZBX_DBL, O_OPT, null,		BETWEEN_DBL(0, 100, 4), null, _('Percentile line (right)')],
 	'visible' =>			[T_ZBX_INT, O_OPT, null,		BETWEEN(0, 1),	null],
 	'items' =>				[T_ZBX_STR, O_OPT, null,		null,			null],
+	'discover' =>			[T_ZBX_INT, O_OPT, null,		IN([ZBX_PROTOTYPE_DISCOVER, ZBX_PROTOTYPE_NO_DISCOVER]), null],
 	'show_work_period' =>	[T_ZBX_INT, O_OPT, null,		IN('1'),		null],
 	'show_triggers' =>		[T_ZBX_INT, O_OPT, null,		IN('1'),		null],
 	'group_graphid' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,			null],
-	'copy_targetid' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,			null],
-	'copy_groupid' =>		[T_ZBX_INT, O_OPT, P_SYS,		DB_ID,			'isset({copy}) && isset({copy_type}) && {copy_type} == 0'],
+	'copy_targetids' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,			null],
 	// actions
-	'action' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, IN('"graph.masscopyto","graph.massdelete"'),	null],
+	'action' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, IN('"graph.masscopyto","graph.massdelete","graph.updatediscover"'),	null],
 	'add' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
 	'update' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
 	'clone' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,			null],
@@ -70,8 +71,13 @@ $fields = [
 	'cancel' =>				[T_ZBX_STR, O_OPT, P_SYS,		null,			null],
 	'form' =>				[T_ZBX_STR, O_OPT, P_SYS,		null,			null],
 	'form_refresh' =>		[T_ZBX_INT, O_OPT, null,		null,			null],
+	// filter
+	'filter_set' =>			[T_ZBX_STR, O_OPT, P_SYS,		null,	null],
+	'filter_rst' =>			[T_ZBX_STR, O_OPT, P_SYS,		null,	null],
+	'filter_groups' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,	null],
+	'filter_hostids' =>		[T_ZBX_INT, O_OPT, null,		DB_ID,	null],
 	// sort and sortorder
-	'sort' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"graphtype","name"'),					null],
+	'sort' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"graphtype","name","discover"'),					null],
 	'sortorder' =>			[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
 $percentVisible = getRequest('visible');
@@ -89,19 +95,19 @@ if (isset($_REQUEST['yaxismax']) && zbx_empty($_REQUEST['yaxismax'])) {
 }
 check_fields($fields);
 
-$_REQUEST['items'] = getRequest('items', []);
+$gitems = [];
+foreach (getRequest('items', []) as $gitem) {
+	$gitems[] = json_decode($gitem, true);
+}
+
+$_REQUEST['items'] = $gitems;
 $_REQUEST['show_3d'] = getRequest('show_3d', 0);
 $_REQUEST['show_legend'] = getRequest('show_legend', 0);
 
 /*
  * Permissions
  */
-$groupId = getRequest('groupid');
-if ($groupId && !isWritableHostGroups([$groupId])) {
-	access_deny();
-}
-
-$hostId = getRequest('hostid', 0);
+$hostid = getRequest('hostid', 0);
 
 if (hasRequest('parent_discoveryid')) {
 	// check whether discovery rule is editable by user
@@ -115,7 +121,7 @@ if (hasRequest('parent_discoveryid')) {
 		access_deny();
 	}
 
-	$hostId = $discoveryRule['hostid'];
+	$hostid = $discoveryRule['hostid'];
 
 	// check whether graph prototype is editable by user
 	if (hasRequest('graphid')) {
@@ -140,7 +146,7 @@ elseif (hasRequest('graphid')) {
 		access_deny();
 	}
 }
-elseif ($hostId && !isWritableHostTemplates([$hostId])) {
+elseif ($hostid && !isWritableHostTemplates([$hostid])) {
 	access_deny();
 }
 
@@ -176,11 +182,9 @@ if (isset($_REQUEST['clone']) && isset($_REQUEST['graphid'])) {
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	$items = getRequest('items', []);
-
 	// remove passing "gitemid" to API if new items added via pop-up
-	foreach ($items as &$item) {
-		if (!$item['gitemid']) {
+	foreach ($gitems as &$item) {
+		if (array_key_exists('gitemid', $item) && !$item['gitemid']) {
 			unset($item['gitemid']);
 		}
 	}
@@ -203,13 +207,15 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		'show_3d' => getRequest('show_3d', 0),
 		'percent_left' => getRequest('percent_left', 0),
 		'percent_right' => getRequest('percent_right', 0),
-		'gitems' => $items
+		'gitems' => $gitems
 	];
 
 	DBstart();
 
 	// create and update graph prototypes
 	if (hasRequest('parent_discoveryid')) {
+		$graph['discover'] = getRequest('discover', DB::getDefault('graphs', 'discover'));
+
 		if (hasRequest('graphid')) {
 			$graph['graphid'] = getRequest('graphid');
 			$result = API::GraphPrototype()->update($graph);
@@ -242,7 +248,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$messageFailed = _('Cannot add graph');
 		}
 
-		$cookieId = $hostId;
+		$cookieId = $hostid;
 	}
 
 	if ($result) {
@@ -280,7 +286,7 @@ elseif (hasRequest('delete') && hasRequest('graphid')) {
 		$result = API::Graph()->delete([$graphId]);
 
 		if ($result) {
-			uncheckTableRows($hostId);
+			uncheckTableRows($hostid);
 		}
 		show_messages($result, _('Graph deleted'), _('Cannot delete graph'));
 	}
@@ -289,6 +295,14 @@ elseif (hasRequest('delete') && hasRequest('graphid')) {
 		unset($_REQUEST['form']);
 	}
 }
+elseif (getRequest('graphid', '') && getRequest('action', '') === 'graph.updatediscover') {
+	$result = API::GraphPrototype()->update([
+		'graphid' => getRequest('graphid'),
+		'discover' => getRequest('discover', DB::getDefault('graphs', 'discover'))
+	]);
+
+	show_messages($result, _('Graph updated'), _('Cannot update graph'));
+}
 elseif (hasRequest('action') && getRequest('action') === 'graph.massdelete' && hasRequest('group_graphid')) {
 	$graphIds = getRequest('group_graphid');
 
@@ -298,20 +312,38 @@ elseif (hasRequest('action') && getRequest('action') === 'graph.massdelete' && h
 		if ($result) {
 			uncheckTableRows(getRequest('parent_discoveryid'));
 		}
+		else {
+			$graphs = API::GraphPrototype()->get([
+				'graphids' => $graphIds,
+				'output' => [],
+				'editable' => true
+			]);
+
+			uncheckTableRows(getRequest('parent_discoveryid'), zbx_objectValues($graphs, 'graphid'));
+		}
 		show_messages($result, _('Graph prototypes deleted'), _('Cannot delete graph prototypes'));
 	}
 	else {
 		$result = API::Graph()->delete($graphIds);
 
 		if ($result) {
-			uncheckTableRows($hostId);
+			uncheckTableRows($hostid);
+		}
+		else {
+			$graphs = API::Graph()->get([
+				'graphids' => $graphIds,
+				'output' => [],
+				'editable' => true
+			]);
+
+			uncheckTableRows($hostid, zbx_objectValues($graphs, 'graphid'));
 		}
 		show_messages($result, _('Graphs deleted'), _('Cannot delete graphs'));
 	}
 }
 elseif (hasRequest('action') && getRequest('action') === 'graph.masscopyto' && hasRequest('copy')
 		&& hasRequest('group_graphid')) {
-	if (getRequest('copy_targetid') != 0 && hasRequest('copy_type')) {
+	if (getRequest('copy_targetids', []) && hasRequest('copy_type')) {
 		$result = true;
 
 		$options = [
@@ -322,11 +354,11 @@ elseif (hasRequest('action') && getRequest('action') === 'graph.masscopyto' && h
 
 		// hosts or templates
 		if (getRequest('copy_type') == COPY_TYPE_TO_HOST || getRequest('copy_type') == COPY_TYPE_TO_TEMPLATE) {
-			$options['hostids'] = getRequest('copy_targetid');
+			$options['hostids'] = getRequest('copy_targetids');
 		}
 		// host groups
 		else {
-			$groupids = getRequest('copy_targetid');
+			$groupids = getRequest('copy_targetids');
 			zbx_value2array($groupids);
 
 			$dbGroups = API::HostGroup()->get([
@@ -359,7 +391,7 @@ elseif (hasRequest('action') && getRequest('action') === 'graph.masscopyto' && h
 
 		if ($result) {
 			uncheckTableRows(
-				(getRequest('parent_discoveryid') == 0) ? $hostId : getRequest('parent_discoveryid')
+				(getRequest('parent_discoveryid') == 0) ? $hostid : getRequest('parent_discoveryid')
 			);
 			unset($_REQUEST['group_graphid']);
 		}
@@ -374,35 +406,82 @@ elseif (hasRequest('action') && getRequest('action') === 'graph.masscopyto' && h
 	show_messages();
 }
 
-/*
- * Display
+/**
+ * Update profile keys.
  */
-$pageFilter = new CPageFilter([
-	'groups' => [
-		'with_hosts_and_templates' => true,
-		'editable' => true
-	],
-	'hosts' => [
-		'editable' => true,
-		'templated_hosts' => true
-	],
-	'groupid' => $groupId,
-	'hostid' => $hostId
-]);
+$sort_field = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
+$sort_order = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_UP));
 
-if (empty($_REQUEST['parent_discoveryid'])) {
-	if ($pageFilter->hostid > 0) {
-		$hostId = $pageFilter->hostid;
+CProfile::update('web.'.$page['file'].'.sort', $sort_field, PROFILE_TYPE_STR);
+CProfile::update('web.'.$page['file'].'.sortorder', $sort_order, PROFILE_TYPE_STR);
+
+if (hasRequest('filter_set')) {
+	CProfile::updateArray('web.graphs.filter_groups', getRequest('filter_groups', []), PROFILE_TYPE_ID);
+	CProfile::updateArray('web.graphs.filter_hostids', getRequest('filter_hostids', []), PROFILE_TYPE_ID);
+}
+elseif (hasRequest('filter_rst')) {
+	CProfile::deleteIdx('web.graphs.filter_groups');
+
+	$filter_hostids = getRequest('filter_hostids', CProfile::getArray('web.graphs.filter_hostids', []));
+	if (count($filter_hostids) != 1) {
+		CProfile::deleteIdx('web.graphs.filter_hostids');
 	}
 }
 
-if (hasRequest('action') && getRequest('action') == 'graph.masscopyto' && hasRequest('group_graphid')) {
-	// render view
+/*
+ * Display
+ */
+if (hasRequest('parent_discoveryid')) {
+	// Argument parent_discoveryid is considered as alternative filter.
+	$filter = [
+		'groups' => null,
+		'hosts' => null
+	];
+}
+else {
+	$filter = [
+		'groups' => CProfile::getArray('web.graphs.filter_groups', null),
+		'hosts' => CProfile::getArray('web.graphs.filter_hostids', null)
+	];
+}
+
+// Get host groups.
+$filter['groups'] = $filter['groups']
+	? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+		'output' => ['groupid', 'name'],
+		'groupids' => $filter['groups'],
+		'editable' => true,
+		'preservekeys' => true
+	]), ['groupid' => 'id'])
+	: [];
+
+$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
+if ($filter_groupids) {
+	$filter_groupids = getSubGroups($filter_groupids);
+}
+
+// Get hosts.
+$filter['hosts'] = $filter['hosts']
+	? CArrayHelper::renameObjectsKeys(API::Host()->get([
+		'output' => ['hostid', 'name'],
+		'hostids' => $filter['hosts'],
+		'templated_hosts' => true,
+		'editable' => true,
+		'preservekeys' => true
+	]), ['hostid' => 'id'])
+	: [];
+
+// Get hostid.
+if ($hostid == 0 && count($filter['hosts']) == 1) {
+	$hostid = reset($filter['hosts'])['id'];
+}
+
+if (hasRequest('action') && getRequest('action') === 'graph.masscopyto' && hasRequest('group_graphid')) {
 	$data = getCopyElementsFormData('group_graphid', _('Graphs'));
 	$data['action'] = 'graph.masscopyto';
-	$graphView = new CView('configuration.copy.elements', $data);
-	$graphView->render();
-	$graphView->show();
+
+	// render view
+	echo (new CView('configuration.copy.elements', $data))->getOutput();
 }
 elseif (isset($_REQUEST['form'])) {
 	$data = [
@@ -411,8 +490,7 @@ elseif (isset($_REQUEST['form'])) {
 		'graphid' => getRequest('graphid', 0),
 		'parent_discoveryid' => getRequest('parent_discoveryid'),
 		'group_gid' => getRequest('group_gid', []),
-		'hostid' => $hostId,
-		'groupid' => $groupId,
+		'hostid' => $hostid,
 		'normal_only' => getRequest('normal_only')
 	];
 
@@ -424,7 +502,10 @@ elseif (isset($_REQUEST['form'])) {
 		];
 
 		if ($data['parent_discoveryid'] === null) {
-			$options['selectDiscoveryRule'] = ['itemid', 'name'];
+			$options += [
+				'selectDiscoveryRule'	=> ['itemid', 'name'],
+				'selectGraphDiscovery'	=> ['parent_graphid']
+			];
 			$graph = API::Graph()->get($options);
 		}
 		else {
@@ -438,8 +519,8 @@ elseif (isset($_REQUEST['form'])) {
 		$data['height'] = $graph['height'];
 		$data['ymin_type'] = $graph['ymin_type'];
 		$data['ymax_type'] = $graph['ymax_type'];
-		$data['yaxismin'] = $graph['yaxismin'];
-		$data['yaxismax'] = $graph['yaxismax'];
+		$data['yaxismin'] = sprintf('%.'.ZBX_FLOAT_DIG.'G', $graph['yaxismin']);
+		$data['yaxismax'] = sprintf('%.'.ZBX_FLOAT_DIG.'G', $graph['yaxismax']);
 		$data['ymin_itemid'] = $graph['ymin_itemid'];
 		$data['ymax_itemid'] = $graph['ymax_itemid'];
 		$data['show_work_period'] = $graph['show_work_period'];
@@ -451,10 +532,12 @@ elseif (isset($_REQUEST['form'])) {
 		$data['percent_right'] = $graph['percent_right'];
 		$data['templateid'] = $graph['templateid'];
 		$data['templates'] = [];
+		$data['discover'] = $graph['discover'];
 
 		if ($data['parent_discoveryid'] === null) {
 			$data['flags'] = $graph['flags'];
 			$data['discoveryRule'] = $graph['discoveryRule'];
+			$data['graphDiscovery'] = $graph['graphDiscovery'];
 		}
 
 		// if no host has been selected for the navigation panel, use the first graph host
@@ -464,46 +547,10 @@ elseif (isset($_REQUEST['form'])) {
 		}
 
 		// templates
-		if (!empty($data['templateid'])) {
-			$parentGraphid = $data['templateid'];
-			do {
-				$parentGraph = getGraphByGraphId($parentGraphid);
-
-				// parent graph prototype link
-				if (getRequest('parent_discoveryid')) {
-					$parentGraphPrototype = API::GraphPrototype()->get([
-						'output' => ['graphid'],
-						'graphids' => $parentGraph['graphid'],
-						'selectTemplates' => API_OUTPUT_EXTEND,
-						'selectDiscoveryRule' => ['itemid']
-					]);
-					if ($parentGraphPrototype) {
-						$parentGraphPrototype = reset($parentGraphPrototype);
-						$parentTemplate = reset($parentGraphPrototype['templates']);
-
-						$link = new CLink($parentTemplate['name'],
-							'graphs.php?form=update&graphid='.$parentGraphPrototype['graphid'].'&hostid='.$parentTemplate['templateid'].'&parent_discoveryid='.$parentGraphPrototype['discoveryRule']['itemid']
-						);
-					}
-				}
-				// parent graph link
-				else {
-					$parentTemplate = get_hosts_by_graphid($parentGraph['graphid']);
-					$parentTemplate = DBfetch($parentTemplate);
-
-					$link = new CLink($parentTemplate['name'],
-						'graphs.php?form=update&graphid='.$parentGraph['graphid'].'&hostid='.$parentTemplate['hostid']
-					);
-				}
-				if (isset($link)) {
-					$data['templates'][] = $link;
-					$data['templates'][] = ' &rArr; ';
-				}
-				$parentGraphid = $parentGraph['templateid'];
-			} while ($parentGraphid != 0);
-			$data['templates'] = array_reverse($data['templates']);
-			array_shift($data['templates']);
-		}
+		$flag = ($data['parent_discoveryid'] === null) ? ZBX_FLAG_DISCOVERY_NORMAL : ZBX_FLAG_DISCOVERY_PROTOTYPE;
+		$data['templates'] = makeGraphTemplatesHtml($graph['graphid'], getGraphParentTemplates([$graph], $flag),
+			$flag
+		);
 
 		// items
 		$data['items'] = API::GraphItem()->get([
@@ -541,7 +588,8 @@ elseif (isset($_REQUEST['form'])) {
 		$data['percent_left'] = 0;
 		$data['percent_right'] = 0;
 		$data['visible'] = getRequest('visible');
-		$data['items'] = getRequest('items', []);
+		$data['items'] = $gitems;
+		$data['discover'] = getRequest('discover', DB::getDefault('graphs', 'discover'));
 		$data['templates'] = [];
 
 		if (isset($data['visible']['percent_left'])) {
@@ -610,94 +658,117 @@ elseif (isset($_REQUEST['form'])) {
 	// is template
 	$data['is_template'] = ($data['hostid'] == 0) ? false : isTemplate($data['hostid']);
 
-	// Read groupid for selected host or template if groupid filter is set to 'All' (is equal 0).
-
-	if ($data['hostid'] && !$data['groupid']) {
-		$db_hostgroup = API::HostGroup()->get([
-			'output' => ['groupid'],
-			'hostids' => $data['hostid'],
-			'templateids' => $data['hostid']
-		]);
-
-		if ($db_hostgroup) {
-			$data['groupid'] = $db_hostgroup[0]['groupid'];
-		}
-	}
-
 	// render view
-	$graphView = new CView('configuration.graph.edit', $data);
-	$graphView->render();
-	$graphView->show();
+	echo (new CView('configuration.graph.edit', $data))->getOutput();
 }
 else {
-	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
-	$sortOrder = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_UP));
-
-	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
-	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
-
 	$config = select_config();
 
 	$data = [
-		'pageFilter' => $pageFilter,
-		'hostid' => ($pageFilter->hostid > 0) ? $pageFilter->hostid : $hostId,
-		'parent_discoveryid' => isset($discoveryRule) ? $discoveryRule['itemid'] : null,
+		'filter' => $filter,
+		'hostid' => $hostid,
+		'parent_discoveryid' => hasRequest('parent_discoveryid') ? $discoveryRule['itemid'] : null,
 		'graphs' => [],
-		'sort' => $sortField,
-		'sortorder' => $sortOrder
+		'sort' => $sort_field,
+		'sortorder' => $sort_order,
+		'profileIdx' => 'web.graphs.filter',
+		'active_tab' => CProfile::get('web.graphs.filter.active', 1)
 	];
 
-	// get graphs
+	// Select graphs.
 	$options = [
-		'hostids' => ($data['hostid'] == 0) ? null : $data['hostid'],
-		'groupids' => ($data['hostid'] == 0 && $pageFilter->groupid > 0) ? $pageFilter->groupids : null,
-		'discoveryids' => isset($discoveryRule) ? $discoveryRule['itemid'] : null,
-		'editable' => true,
 		'output' => ['graphid', 'name', 'graphtype'],
+		'hostids' => $filter['hosts'] ? array_keys($filter['hosts']) : null,
+		'groupids' => $filter_groupids,
+		'discoveryids' => hasRequest('parent_discoveryid') ? $discoveryRule['itemid'] : null,
+		'editable' => true,
 		'limit' => $config['search_limit'] + 1
 	];
 
-	$data['graphs'] = isset($discoveryRule)
+	$data['graphs'] = hasRequest('parent_discoveryid')
 		? API::GraphPrototype()->get($options)
 		: API::Graph()->get($options);
 
-	if ($sortField == 'graphtype') {
+	if ($sort_field === 'graphtype') {
 		foreach ($data['graphs'] as $gnum => $graph) {
 			$data['graphs'][$gnum]['graphtype'] = graphType($graph['graphtype']);
 		}
 	}
 
-	order_result($data['graphs'], $sortField, $sortOrder);
+	order_result($data['graphs'], $sort_field, $sort_order);
 
-	$url = (new CUrl('graphs.php'))
-		->setArgument('groupid', $pageFilter->groupid)
-		->setArgument('hostid', $data['hostid']);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$data['paging'] = getPagingLine($data['graphs'], $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
 
-	// get graphs after paging
+	$data['paging'] = CPagerHelper::paginate($page_num, $data['graphs'], $sort_order, new CUrl('graphs.php'));
+
+	// Get graphs after paging.
 	$options = [
-		'graphids' => zbx_objectValues($data['graphs'], 'graphid'),
-		'output' => ['graphid', 'name', 'templateid', 'graphtype', 'width', 'height'],
+		'output' => ['graphid', 'name', 'templateid', 'graphtype', 'width', 'height', 'discover'],
 		'selectDiscoveryRule' => ['itemid', 'name'],
 		'selectHosts' => ($data['hostid'] == 0) ? ['name'] : null,
-		'selectTemplates' => ($data['hostid'] == 0) ? ['name'] : null
+		'selectTemplates' => ($data['hostid'] == 0) ? ['name'] : null,
+		'graphids' => zbx_objectValues($data['graphs'], 'graphid'),
+		'preservekeys' => true
 	];
 
-	$data['graphs'] = empty($_REQUEST['parent_discoveryid'])
-		? API::Graph()->get($options)
-		: API::GraphPrototype()->get($options);
+	$data['graphs'] = hasRequest('parent_discoveryid')
+		? API::GraphPrototype()->get($options)
+		: API::Graph()->get($options + ['selectGraphDiscovery' => ['ts_delete']]);
 
 	foreach ($data['graphs'] as $gnum => $graph) {
 		$data['graphs'][$gnum]['graphtype'] = graphType($graph['graphtype']);
 	}
 
-	order_result($data['graphs'], $sortField, $sortOrder);
+	if (!hasRequest('parent_discoveryid')) {
+		$items = API::Item()->get([
+			'output' => ['itemid'],
+			'selectGraphs' => ['graphid'],
+			'selectItemDiscovery' => ['ts_delete'],
+			'graphids' => array_keys($data['graphs']),
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_CREATED]
+		]);
+
+		foreach ($items as $item) {
+			$ts_delete = $item['itemDiscovery']['ts_delete'];
+
+			if ($ts_delete == 0) {
+				continue;
+			}
+
+			foreach (array_column($item['graphs'], 'graphid') as $graphid) {
+				if (!array_key_exists('ts_delete', $data['graphs'][$graphid]['graphDiscovery'])) {
+					$data['graphs'][$graphid]['graphDiscovery']['ts_delete'] = $ts_delete;
+				}
+				else {
+					$graph_ts_delete = $data['graphs'][$graphid]['graphDiscovery']['ts_delete'];
+					$data['graphs'][$graphid]['graphDiscovery']['ts_delete'] = ($graph_ts_delete > 0)
+						? min($ts_delete, $graph_ts_delete)
+						: $ts_delete;
+				}
+			}
+		}
+	}
+
+	order_result($data['graphs'], $sort_field, $sort_order);
+
+	$data['parent_templates'] = getGraphParentTemplates($data['graphs'], ($data['parent_discoveryid'] === null)
+		? ZBX_FLAG_DISCOVERY_NORMAL
+		: ZBX_FLAG_DISCOVERY_PROTOTYPE
+	);
 
 	// render view
-	$graphView = new CView('configuration.graph.list', $data);
-	$graphView->render();
-	$graphView->show();
+	echo (new CView('configuration.graph.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
