@@ -44,6 +44,7 @@ $fields = [
 	'update' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'clone' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'delete' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel' =>			[T_ZBX_STR, O_OPT, P_SYS,		null,	null],
 	// other
 	'form' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form_refresh' =>	[T_ZBX_INT, O_OPT, null,	null,		null],
@@ -159,7 +160,6 @@ elseif (hasRequest('action')) {
 	elseif (getRequest('action') == 'hostgroup.massenable' || getRequest('action') == 'hostgroup.massdisable') {
 		$enable = (getRequest('action') == 'hostgroup.massenable');
 		$status = $enable ? HOST_STATUS_MONITORED : HOST_STATUS_NOT_MONITORED;
-		$auditAction = $enable ? AUDIT_ACTION_ENABLE : AUDIT_ACTION_DISABLE;
 
 		$groupIds = getRequest('groups', []);
 
@@ -179,14 +179,6 @@ elseif (hasRequest('action')) {
 					'hosts' => $hosts,
 					'status' => $status
 				]);
-
-				if ($result) {
-					foreach ($hosts as $host) {
-						add_audit_ext($auditAction, AUDIT_RESOURCE_HOST, $host['hostid'], $host['host'], 'hosts',
-							['status' => $host['status']], ['status' => $status]
-						);
-					}
-				}
 			}
 
 			$result = DBend($result);
@@ -250,9 +242,7 @@ if (hasRequest('form')) {
 	}
 
 	// render view
-	$view = new CView('configuration.hostgroups.edit', $data);
-	$view->render();
-	$view->show();
+	echo (new CView('configuration.hostgroups.edit', $data))->getOutput();
 }
 /*
  * Display list
@@ -298,7 +288,21 @@ else {
 	]);
 	order_result($groups, $sortField, $sortOrder);
 
-	$data['paging'] = getPagingLine($groups, $sortOrder, new CUrl('hostgroups.php'));
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
+
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$data['paging'] = CPagerHelper::paginate($page_num, $groups, $sortOrder, new CUrl('hostgroups.php'));
+
 	$groupIds = zbx_objectValues($groups, 'groupid');
 
 	// get hosts and templates count
@@ -329,9 +333,7 @@ else {
 	unset($group);
 
 	// render view
-	$view = new CView('configuration.hostgroups.list', $data);
-	$view->render();
-	$view->show();
+	echo (new CView('configuration.hostgroups.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
