@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,18 +24,78 @@
  */
 class CHistoryManager {
 
+	/**
+	 * Returns a subset of $items having history data within the $period of time.
+	 *
+	 * @param array $items   An array of items with the 'itemid' and 'value_type' properties.
+	 * @param int   $period  The maximum period of time to search for history values within.
+	 *
+	 * @return array  An array with items IDs as keys and the original item data as values.
+	 */
+	public function getItemsHavingValues(array $items, $period = null) {
+		$items = zbx_toHash($items, 'itemid');
+
+		$results = [];
+		$grouped_items = $this->getItemsGroupedByStorage($items);
+
+		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $grouped_items)) {
+			$results += $this->getLastValuesFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC], 1, $period);
+		}
+
+		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $grouped_items)) {
+			$results += $this->getItemsHavingValuesFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL], $period);
+		}
+
+		return array_intersect_key($items, $results);
+	}
+
+	/**
+	 * SQL specific implementation of getItemsHavingValues.
+	 *
+	 * @see CHistoryManager::getItemsHavingValues
+	 */
+	private function getItemsHavingValuesFromSql(array $items, $period = null) {
+		$results = [];
+
+		if ($period) {
+			$period = time() - $period;
+		}
+
+		$items = zbx_toHash($items, 'itemid');
+
+		$itemids_by_type = [];
+
+		foreach ($items as $itemid => $item) {
+			$itemids_by_type[$item['value_type']][] = $itemid;
+		}
+
+		foreach ($itemids_by_type as $type => $type_itemids) {
+			$type_results = DBfetchColumn(DBselect(
+				'SELECT itemid'.
+				' FROM '.self::getTableName($type).
+				' WHERE '.dbConditionInt('itemid', $type_itemids).
+					($period ? ' AND clock>'.$period : '').
+				' GROUP BY itemid'
+			), 'itemid');
+
+			$results += array_intersect_key($items, array_flip($type_results));
+		}
+
+		return $results;
+	}
+
 	/**
 	 * Returns the last $limit history objects for the given items.
 	 *
-	 * @param array $items     an array of items with the 'itemid' and 'value_type' properties
-	 * @param int   $limit     max object count to be returned
-	 * @param int   $period    the maximum period to retrieve data for
+	 * @param array $items   An array of items with the 'itemid' and 'value_type' properties.
+	 * @param int   $limit   Max object count to be returned.
+	 * @param int   $period  The maximum period to retrieve data for.
 	 *
-	 * @return array    an array with items IDs as keys and arrays of history objects as values
+	 * @return array  An array with items IDs as keys and arrays of history objects as values.
 	 */
 	public function getLastValues(array $items, $limit = 1, $period = null) {
 		$results = [];
-		$grouped_items = self::getItemsGroupedByStorage($items);
+		$grouped_items = $this->getItemsGroupedByStorage($items);
 
 		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $grouped_items)) {
 			$results += $this->getLastValuesFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC], $limit,
@@ -140,18 +200,95 @@ class CHistoryManager {
 	private function getLastValuesFromSql($items, $limit, $period) {
 		$results = [];
 
-		foreach ($items as $item) {
-			$values = DBfetchArray(DBselect(
-				'SELECT *'.
-				' FROM '.self::getTableName($item['value_type']).' h'.
-				' WHERE h.itemid='.zbx_dbstr($item['itemid']).
-					($period ? ' AND h.clock>'.(time() - $period) : '').
-				' ORDER BY h.clock DESC',
-				$limit
-			));
+		if ($period) {
+			$period = time() - $period;
+		}
 
-			if ($values) {
-				$results[$item['itemid']] = $values;
+		if ($limit == 1) {
+			foreach ($items as $item) {
+				// Executing two subsequent queries individually for the sake of performance.
+
+				$clock_max = DBfetch(DBselect(
+					'SELECT MAX(h.clock)'.
+					' FROM '.self::getTableName($item['value_type']).' h'.
+					' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+						($period ? ' AND h.clock>'.$period : '')
+				), false);
+
+				if ($clock_max) {
+					$clock_max = reset($clock_max);
+
+					if ($clock_max !== null) {
+						$values = DBfetchArray(DBselect(
+							'SELECT *'.
+							' FROM '.self::getTableName($item['value_type']).' h'.
+							' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+								' AND h.clock='.zbx_dbstr($clock_max).
+							' ORDER BY h.ns DESC',
+							$limit
+						));
+
+						if ($values) {
+							$results[$item['itemid']] = $values;
+						}
+					}
+				}
+			}
+		}
+		else {
+			foreach ($items as $item) {
+				// Cannot order by h.ns directly here due to performance issues.
+				$values = DBfetchArray(DBselect(
+					'SELECT *'.
+					' FROM '.self::getTableName($item['value_type']).' h'.
+					' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+						($period ? ' AND h.clock>'.$period : '').
+					' ORDER BY h.clock DESC',
+					$limit + 1
+				));
+
+				if ($values) {
+					$count = count($values);
+					$clock = $values[$count - 1]['clock'];
+
+					if ($count == $limit + 1 && $values[$count - 2]['clock'] == $clock) {
+						/*
+						 * The last selected entries having the same clock means the selection (not just the order)
+						 * of the last entries is possibly wrong due to unordered by nanoseconds.
+						 */
+
+						do {
+							unset($values[--$count]);
+						} while ($values && $values[$count - 1]['clock'] == $clock);
+
+						$db_values = DBselect(
+							'SELECT *'.
+							' FROM '.self::getTableName($item['value_type']).' h'.
+							' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+								' AND h.clock='.$clock.
+							' ORDER BY h.ns DESC',
+							$limit - $count
+						);
+
+						while ($db_value = DBfetch($db_values)) {
+							$values[] = $db_value;
+							$count++;
+						}
+					}
+
+					CArrayHelper::sort($values, [
+						['field' => 'clock', 'order' => ZBX_SORT_DOWN],
+						['field' => 'ns', 'order' => ZBX_SORT_DOWN]
+					]);
+
+					$values = array_values($values);
+
+					while ($count > $limit) {
+						unset($values[--$count]);
+					}
+
+					$results[$item['itemid']] = $values;
+				}
 			}
 		}
 
@@ -159,18 +296,20 @@ class CHistoryManager {
 	}
 
 	/**
-	 * Returns the history value of the item at the given time. If no value exists at the given time, the function
-	 * will return the previous value.
+	 * Returns the history data of the item at the given time. If no data exists at the given time, the function will
+	 * return the previous data.
 	 *
 	 * The $item parameter must have the value_type and itemid properties set.
 	 *
-	 * @param array $item     item to get value for
-	 * @param int   $clock    timestamp (seconds)
-	 * @param int   $ns       timestamp (nanoseconds)
+	 * @param array  $item
+	 * @param string $item['itemid']
+	 * @param int    $item['value_type']
+	 * @param int    $clock
+	 * @param int    $ns
 	 *
-	 * @return string    value at specified time of first value before specified time
+	 * @return array|null  Item data at specified time of first data before specified time. null if data is not found.
 	 */
-	public function getValueAt($item, $clock, $ns) {
+	public function getValueAt(array $item, $clock, $ns) {
 		switch (self::getDataSourceType($item['value_type'])) {
 			case ZBX_HISTORY_SOURCE_ELASTIC:
 				return $this->getValueAtFromElasticsearch($item, $clock, $ns);
@@ -185,7 +324,7 @@ class CHistoryManager {
 	 *
 	 * @see CHistoryManager::getValueAt
 	 */
-	private function getValueAtFromElasticsearch($item, $clock, $ns) {
+	private function getValueAtFromElasticsearch(array $item, $clock, $ns) {
 		$query = [
 			'sort' => [
 				'clock' => ZBX_SORT_DOWN,
@@ -224,7 +363,7 @@ class CHistoryManager {
 					'range' => [
 						'clock' => [
 							'lt' => $clock
-						]
+						] + (ZBX_HISTORY_PERIOD ? ['gte' => $clock - ZBX_HISTORY_PERIOD] : [])
 					]
 				]
 			]
@@ -241,7 +380,7 @@ class CHistoryManager {
 			$result = CElasticsearchHelper::query('POST', reset($endpoints), $query);
 
 			if (count($result) === 1 && is_array($result[0]) && array_key_exists('value', $result[0])) {
-				return $result[0]['value'];
+				return $result[0];
 			}
 		}
 
@@ -253,22 +392,22 @@ class CHistoryManager {
 	 *
 	 * @see CHistoryManager::getValueAt
 	 */
-	private function getValueAtFromSql($item, $clock, $ns) {
-		$value = null;
+	private function getValueAtFromSql(array $item, $clock, $ns) {
+		$result = null;
 		$table = self::getTableName($item['value_type']);
 
-		$sql = 'SELECT value'.
+		$sql = 'SELECT *'.
 				' FROM '.$table.
 				' WHERE itemid='.zbx_dbstr($item['itemid']).
 					' AND clock='.zbx_dbstr($clock).
 					' AND ns='.zbx_dbstr($ns);
 
 		if (($row = DBfetch(DBselect($sql, 1))) !== false) {
-			$value = $row['value'];
+			$result = $row;
 		}
 
-		if ($value !== null) {
-			return $value;
+		if ($result !== null) {
+			return $result;
 		}
 
 		$max_clock = 0;
@@ -286,7 +425,8 @@ class CHistoryManager {
 			$sql = 'SELECT MAX(clock) AS clock'.
 					' FROM '.$table.
 					' WHERE itemid='.zbx_dbstr($item['itemid']).
-						' AND clock<'.zbx_dbstr($clock);
+						' AND clock<'.zbx_dbstr($clock).
+						(ZBX_HISTORY_PERIOD ? ' AND clock>='.zbx_dbstr($clock - ZBX_HISTORY_PERIOD) : '');
 
 			if (($row = DBfetch(DBselect($sql))) !== false) {
 				$max_clock = $row['clock'];
@@ -294,18 +434,18 @@ class CHistoryManager {
 		}
 
 		if ($max_clock == 0) {
-			return $value;
+			return $result;
 		}
 
 		if ($clock == $max_clock) {
-			$sql = 'SELECT value'.
+			$sql = 'SELECT *'.
 					' FROM '.$table.
 					' WHERE itemid='.zbx_dbstr($item['itemid']).
 						' AND clock='.zbx_dbstr($clock).
 						' AND ns<'.zbx_dbstr($ns);
 		}
 		else {
-			$sql = 'SELECT value'.
+			$sql = 'SELECT *'.
 					' FROM '.$table.
 					' WHERE itemid='.zbx_dbstr($item['itemid']).
 						' AND clock='.zbx_dbstr($max_clock).
@@ -313,10 +453,10 @@ class CHistoryManager {
 		}
 
 		if (($row = DBfetch(DBselect($sql, 1))) !== false) {
-			$value = $row['value'];
+			$result = $row;
 		}
 
-		return $value;
+		return $result;
 	}
 
 	/**
@@ -324,35 +464,27 @@ class CHistoryManager {
 	 *
 	 * The $item parameter must have the value_type, itemid and source properties set.
 	 *
-	 * @param array  $items        items to get aggregated values for
-	 * @param int    $time_from    minimal timestamp (seconds) to get data from
-	 * @param int    $time_to      maximum timestamp (seconds) to get data from
-	 * @param int    $width        graph width in pixels (is not required for pie charts)
+	 * @param array  $items      Items to get aggregated values for.
+	 * @param int    $time_from  Minimal timestamp (seconds) to get data from.
+	 * @param int    $time_to    Maximum timestamp (seconds) to get data from.
+	 * @param string $function   Function for data aggregation.
+	 * @param string $interval   Aggregation interval in seconds.
 	 *
-	 * @return array    history value aggregation for graphs
+	 * @return array  History value aggregation for graphs.
 	 */
-	public function getGraphAggregation(array $items, $time_from, $time_to, $width = null) {
-		if ($width !== null) {
-			$size = $time_to - $time_from;
-			$delta = $size - $time_from % $size;
-		}
-		else {
-			$size = null;
-			$delta = null;
-		}
-
-		$grouped_items = self::getItemsGroupedByStorage($items);
+	public function getGraphAggregationByInterval(array $items, $time_from, $time_to, $function, $interval) {
+		$grouped_items = $this->getItemsGroupedByStorage($items);
 
 		$results = [];
 		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $grouped_items)) {
-			$results += $this->getGraphAggregationFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC],
-					$time_from, $time_to, $width, $size, $delta
+			$results = $this->getGraphAggregationByIntervalFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC],
+				$time_from, $time_to, $function, $interval
 			);
 		}
 
 		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $grouped_items)) {
-			$results += $this->getGraphAggregationFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL], $time_from, $time_to,
-					$width, $size, $delta
+			$results += $this->getGraphAggregationByIntervalFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL],
+				$time_from, $time_to, $function, $interval
 			);
 		}
 
@@ -360,11 +492,264 @@ class CHistoryManager {
 	}
 
 	/**
-	 * Elasticsearch specific implementation of getGraphAggregation.
+	 * Elasticsearch specific implementation of getGraphAggregationByInterval.
 	 *
-	 * @see CHistoryManager::getGraphAggregation
+	 * @see CHistoryManager::getGraphAggregationByInterval
 	 */
-	private function getGraphAggregationFromElasticsearch(array $items, $time_from, $time_to, $width, $size, $delta) {
+	private function getGraphAggregationByIntervalFromElasticsearch(array $items, $time_from, $time_to, $function,
+			$interval) {
+		$terms = [];
+
+		foreach ($items as $item) {
+			$terms[$item['value_type']][] = $item['itemid'];
+		}
+
+		$aggs = ['clock' => ['max' => ['field' => 'clock']]];
+
+		switch ($function) {
+			case GRAPH_AGGREGATE_MIN:
+				$aggs['value'] = ['min' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_MAX:
+				$aggs['value'] = ['max' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_AVG:
+				$aggs['value'] = ['avg' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_SUM:
+				$aggs['value'] = ['sum' => ['field' => 'value']];
+				break;
+			case GRAPH_AGGREGATE_FIRST:
+				$aggs['value'] = ['top_hits' => ['size' => 1, 'sort' => ['clock' => ['order' => 'asc']]]];
+				$aggs['clock'] = ['min' => ['field' => 'clock']];
+				break;
+			case GRAPH_AGGREGATE_LAST:
+				$aggs['value'] = ['top_hits' => ['size' => 1, 'sort' => ['clock' => ['order' => 'desc']]]];
+				break;
+		}
+
+		$query = [
+			'aggs' => [
+				'group_by_itemid' => [
+					'terms' => [
+						// Assure that aggregations for all terms are returned.
+						'size' => count($items),
+						'field' => 'itemid'
+					]
+				]
+			],
+			'size' => 0
+		];
+
+		// Clock value is divided by 1000 as it is stored as milliseconds.
+		$formula = '((doc[\'clock\'].date.getMillis()/1000) - ((doc[\'clock\'].date.getMillis()/1000)%params.interval))';
+
+		$query['aggs']['group_by_itemid']['aggs'] = [
+			'group_by_script' => [
+				'terms' => [
+					'size' => (($time_to - $time_from) / $interval) + 1,
+					'script' => [
+						'inline' => $formula,
+						'params' => [
+							'interval' => $interval
+						]
+					]
+				],
+				'aggs' => $aggs
+			]
+		];
+
+		$results = [];
+		foreach (self::getElasticsearchEndpoints(array_keys($terms)) as $type => $endpoint) {
+			$query['query']['bool']['must'] = [
+				[
+					'terms' => [
+						'itemid' => $terms[$type]
+					]
+				],
+				[
+					'range' => [
+						'clock' => [
+							'gte' => $time_from,
+							'lte' => $time_to
+						]
+					]
+				]
+			];
+
+			$data = CElasticsearchHelper::query('POST', $endpoint, $query);
+
+			foreach ($data['group_by_itemid']['buckets'] as $item) {
+				if (!is_array($item['group_by_script']) || !array_key_exists('buckets', $item['group_by_script'])
+					|| !is_array($item['group_by_script']['buckets'])) {
+					continue;
+				}
+
+				foreach ($item['group_by_script']['buckets'] as $point) {
+					$row = [
+						'itemid' => $item['key'],
+						'tick' => (int)$point['key'],
+						'count' => $point['doc_count'],
+						'clock' => (int)$point['clock']['value_as_string']
+					];
+
+					if ($function == GRAPH_AGGREGATE_FIRST || $function == GRAPH_AGGREGATE_LAST) {
+						$row['value'] = $point['value']['hits']['hits'][0]['_source']['value'];
+					}
+					else {
+						$row['value'] = array_key_exists('value', $point) ? $point['value']['value'] : null;
+					}
+
+					$results[$item['key']]['data'][] = $row;
+				}
+
+				if (array_key_exists($item['key'], $results)) {
+					$results[$item['key']]['source'] = 'history';
+				}
+			}
+		}
+
+		return $results;
+	}
+
+	/**
+	 * SQL specific implementation of getGraphAggregationByWidth.
+	 *
+	 * @see CHistoryManager::getGraphAggregationByInterval
+	 */
+	private function getGraphAggregationByIntervalFromSql(array $items, $time_from, $time_to, $function, $interval) {
+		$items_by_table = [];
+		foreach ($items as $item) {
+			$items_by_table[$item['value_type']][$item['source']][] = $item['itemid'];
+		}
+
+		$result = [];
+
+		foreach ($items_by_table as $value_type => $items_by_source) {
+			foreach ($items_by_source as $source => $itemids) {
+				$sql_select = ['itemid'];
+				$sql_group_by = ['itemid'];
+
+				$calc_field = zbx_dbcast_2bigint('clock').'-'.zbx_sql_mod(zbx_dbcast_2bigint('clock'), $interval);
+				$sql_select[] = $calc_field.' AS tick';
+				$sql_group_by[] = $calc_field;
+
+				if ($source === 'history') {
+					switch ($function) {
+						case GRAPH_AGGREGATE_MIN:
+							$sql_select[] = 'MIN(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_MAX:
+							$sql_select[] = 'MAX(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_AVG:
+							$sql_select[] = 'AVG(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_COUNT:
+							$sql_select[] = 'COUNT(*) AS count, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_SUM:
+							$sql_select[] = 'SUM(value) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_FIRST:
+							$sql_select[] = 'MIN(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_LAST:
+							$sql_select[] = 'MAX(clock) AS clock';
+							break;
+					}
+					$sql_from = ($value_type == ITEM_VALUE_TYPE_UINT64) ? 'history_uint' : 'history';
+				}
+				else {
+					switch ($function) {
+						case GRAPH_AGGREGATE_MIN:
+							$sql_select[] = 'MIN(value_min) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_MAX:
+							$sql_select[] = 'MAX(value_max) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_AVG:
+							$sql_select[] = 'AVG(value_avg) AS value, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_COUNT:
+							$sql_select[] = 'SUM(num) AS count, MAX(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_SUM:
+							$sql_select[] = '(value_avg * num) AS value, MAX(clock) AS clock';
+							$sql_group_by = array_merge($sql_group_by, ['value_avg', 'num']);
+							break;
+						case GRAPH_AGGREGATE_FIRST:
+							$sql_select[] = 'MIN(clock) AS clock';
+							break;
+						case GRAPH_AGGREGATE_LAST:
+							$sql_select[] = 'MAX(clock) AS clock';
+							break;
+					}
+					$sql_from = ($value_type == ITEM_VALUE_TYPE_UINT64) ? 'trends_uint' : 'trends';
+				}
+
+				$sql = 'SELECT '.implode(', ', $sql_select).
+					' FROM '.$sql_from.
+					' WHERE '.dbConditionInt('itemid', $itemids).
+					' AND clock >= '.zbx_dbstr($time_from).
+					' AND clock <= '.zbx_dbstr($time_to).
+					' GROUP BY '.implode(', ', $sql_group_by);
+
+				if ($function == GRAPH_AGGREGATE_FIRST || $function == GRAPH_AGGREGATE_LAST) {
+					$sql = 'SELECT DISTINCT h.itemid, h.'.($source === 'history' ? 'value' : 'value_avg').' AS value, h.clock, hi.tick'.
+						' FROM '.$sql_from.' h'.
+						' JOIN('.$sql.') hi ON h.itemid = hi.itemid AND h.clock = hi.clock';
+				}
+
+				$sql_result = DBselect($sql);
+
+				while (($row = DBfetch($sql_result)) !== false) {
+					$result[$row['itemid']]['source'] = $source;
+					$result[$row['itemid']]['data'][] = $row;
+				}
+			}
+		}
+
+		return $result;
+	}
+
+	/**
+	 * Returns history value aggregation for graphs.
+	 *
+	 * The $item parameter must have the value_type, itemid and source properties set.
+	 *
+	 * @param array $items      Items to get aggregated values for.
+	 * @param int   $time_from  Minimal timestamp (seconds) to get data from.
+	 * @param int   $time_to    Maximum timestamp (seconds) to get data from.
+	 * @param int   $width      Graph width in pixels (is not required for pie charts).
+	 *
+	 * @return array  History value aggregation for graphs.
+	 */
+	public function getGraphAggregationByWidth(array $items, $time_from, $time_to, $width = null) {
+		$grouped_items = $this->getItemsGroupedByStorage($items);
+
+		$results = [];
+		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $grouped_items)) {
+			$results += $this->getGraphAggregationByWidthFromElasticsearch($grouped_items[ZBX_HISTORY_SOURCE_ELASTIC],
+					$time_from, $time_to, $width
+			);
+		}
+
+		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $grouped_items)) {
+			$results += $this->getGraphAggregationByWidthFromSql($grouped_items[ZBX_HISTORY_SOURCE_SQL],
+				$time_from, $time_to, $width
+			);
+		}
+
+		return $results;
+	}
+
+	/**
+	 * Elasticsearch specific implementation of getGraphAggregationByWidth.
+	 *
+	 * @see CHistoryManager::getGraphAggregationByWidth
+	 */
+	private function getGraphAggregationByWidthFromElasticsearch(array $items, $time_from, $time_to, $width) {
 		$terms = [];
 
 		foreach ($items as $item) {
@@ -426,7 +811,10 @@ class CHistoryManager {
 			'size' => 0
 		];
 
-		if ($width !== null && $size !== null && $delta !== null) {
+		if ($width !== null) {
+			$size = $time_to - $time_from;
+			$delta = $size - $time_from % $size;
+
 			// Additional grouping for line graphs.
 			$aggs['max_clock'] = [
 				'max' => [
@@ -435,7 +823,7 @@ class CHistoryManager {
 			];
 
 			// Clock value is divided by 1000 as it is stored as milliseconds.
-			$formula = 'Math.floor((params.width*((doc[\'clock\'].date.getMillis()/1000+params.delta)%params.size))'.
+			$formula = 'Math.floor((params.width*((doc[\'clock\'].value.getMillis()/1000+params.delta)%params.size))'.
 					'/params.size)';
 
 			$script = [
@@ -480,39 +868,41 @@ class CHistoryManager {
 
 			$data = CElasticsearchHelper::query('POST', $endpoint, $query);
 
-			if ($width !== null && $size !== null && $delta !== null) {
-				foreach ($data['group_by_itemid']['buckets'] as $item) {
-					if (!is_array($item['group_by_script']) || !array_key_exists('buckets', $item['group_by_script'])
-							|| !is_array($item['group_by_script']['buckets'])) {
-						continue;
-					}
+			if (array_key_exists('group_by_itemid', $data)) {
+				if ($width !== null) {
+					foreach ($data['group_by_itemid']['buckets'] as $item) {
+						if (!is_array($item['group_by_script']) || !array_key_exists('buckets', $item['group_by_script'])
+								|| !is_array($item['group_by_script']['buckets'])) {
+							continue;
+						}
 
-					$results[$item['key']]['source'] = 'history';
-					foreach ($item['group_by_script']['buckets'] as $point) {
-						$results[$item['key']]['data'][] = [
-							'itemid' => $item['key'],
-							'i' => $point['key'],
-							'count' => $point['doc_count'],
-							'min' => $point['min_value']['value'],
-							'avg' => $point['avg_value']['value'],
-							'max' => $point['max_value']['value'],
-							// Field value_as_string is used to get value as seconds instead of milliseconds.
-							'clock' => $point['max_clock']['value_as_string']
-						];
+						$results[$item['key']]['source'] = 'history';
+						foreach ($item['group_by_script']['buckets'] as $point) {
+							$results[$item['key']]['data'][] = [
+								'itemid' => $item['key'],
+								'i' => $point['key'],
+								'count' => $point['doc_count'],
+								'min' => $point['min_value']['value'],
+								'avg' => $point['avg_value']['value'],
+								'max' => $point['max_value']['value'],
+								// Field value_as_string is used to get value as seconds instead of milliseconds.
+								'clock' => $point['max_clock']['value_as_string']
+							];
+						}
 					}
 				}
-			}
-			else {
-				foreach ($data['group_by_itemid']['buckets'] as $item) {
-					$results[$item['key']]['source'] = 'history';
-					$results[$item['key']]['data'][] = [
-						'itemid' => $item['key'],
-						'min' => $item['min_value']['value'],
-						'avg' => $item['avg_value']['value'],
-						'max' => $item['max_value']['value'],
-						// Field value_as_string is used to get value as seconds instead of milliseconds.
-						'clock' => $item['max_clock']['value_as_string']
-					];
+				else {
+					foreach ($data['group_by_itemid']['buckets'] as $item) {
+						$results[$item['key']]['source'] = 'history';
+						$results[$item['key']]['data'][] = [
+							'itemid' => $item['key'],
+							'min' => $item['min_value']['value'],
+							'avg' => $item['avg_value']['value'],
+							'max' => $item['max_value']['value'],
+							// Field value_as_string is used to get value as seconds instead of milliseconds.
+							'clock' => $item['max_clock']['value_as_string']
+						];
+					}
 				}
 			}
 		}
@@ -521,15 +911,18 @@ class CHistoryManager {
 	}
 
 	/**
-	 * SQL specific implementation of getGraphAggregation.
+	 * SQL specific implementation of getGraphAggregationByWidth.
 	 *
-	 * @see CHistoryManager::getGraphAggregation
+	 * @see CHistoryManager::getGraphAggregationByWidth
 	 */
-	private function getGraphAggregationFromSql(array $items, $time_from, $time_to, $width, $size, $delta) {
+	private function getGraphAggregationByWidthFromSql(array $items, $time_from, $time_to, $width) {
 		$group_by = 'itemid';
 		$sql_select_extra = '';
 
-		if ($width !== null && $size !== null && $delta !== null) {
+		if ($width !== null) {
+			$size = $time_to - $time_from;
+			$delta = $size - $time_from % $size;
+
 			// Required for 'group by' support of Oracle.
 			$calc_field = 'round('.$width.'*'.zbx_sql_mod(zbx_dbcast_2bigint('clock').'+'.$delta, $size)
 					.'/('.$size.'),0)';
@@ -576,11 +969,11 @@ class CHistoryManager {
 	 *
 	 * The $item parameter must have the value_type and itemid properties set.
 	 *
-	 * @param array  $item         item to get aggregated value for
-	 * @param string $aggregation  aggregation to be applied (min / max / avg)
-	 * @param int    $time_from    timestamp (seconds)
+	 * @param array  $item         Item to get aggregated value for.
+	 * @param string $aggregation  Aggregation to be applied (min / max / avg).
+	 * @param int    $time_from    Timestamp (seconds).
 	 *
-	 * @return string    aggregated history value
+	 * @return string  Aggregated history value.
 	 */
 	public function getAggregatedValue(array $item, $aggregation, $time_from) {
 		switch (self::getDataSourceType($item['value_type'])) {
@@ -662,186 +1055,15 @@ class CHistoryManager {
 		return null;
 	}
 
-	/**
-	 * Return the time of the 1st appearance of items in history or trends.
-	 *
-	 * @param array $items
-	 * @param array $items[]['itemid']
-	 * @param array $items[]['hostid']
-	 * @param array $items[]['value_type']
-	 * @param array $items[]['history']
-	 * @param array $items[]['trends']
-	 *
-	 * @return int    timestamp (unixtime)
-	 */
-	public function getMinClock(array $items) {
-		$item_types = [
-			ITEM_VALUE_TYPE_FLOAT => [],
-			ITEM_VALUE_TYPE_STR => [],
-			ITEM_VALUE_TYPE_LOG => [],
-			ITEM_VALUE_TYPE_UINT64 => [],
-			ITEM_VALUE_TYPE_TEXT => []
-		];
-		$max = ['history' => 0, 'trends' => 0];
-		$simple_interval_parser = new CSimpleIntervalParser();
-
-		$items = CMacrosResolverHelper::resolveTimeUnitMacros($items, ['history', 'trends']);
-
-		foreach ($items as $item) {
-			$item_types[$item['value_type']][] = $item['itemid'];
-
-			if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
-				if ($simple_interval_parser->parse($item['history']) == CParser::PARSE_SUCCESS) {
-					$item['history'] = (int) timeUnitToSeconds($item['history']);
-					$max['history'] = max($max['history'], $item['history']);
-				}
-
-				if ($simple_interval_parser->parse($item['trends']) == CParser::PARSE_SUCCESS) {
-					$item['trends'] = (int) timeUnitToSeconds($item['trends']);
-					$max['trends'] = max($max['trends'], $item['trends']);
-				}
-			}
-		}
-
-		$source = ($max['history'] > $max['trends']) ? 'history' : 'trends';
-		$storage_items = [];
-
-		foreach ($item_types as $type => $itemids) {
-			if (!$itemids) {
-				continue;
-			}
-
-			$history_source = self::getDataSourceType($type);
-			$storage_items[$history_source][$type] = $itemids;
-		}
-
-		$min_clock = [];
-
-		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $storage_items)) {
-			$min_clock[] = $this->getMinClockFromElasticsearch($storage_items[ZBX_HISTORY_SOURCE_ELASTIC]);
-		}
-
-		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $storage_items)) {
-			$min_clock[] = $this->getMinClockFromSql($storage_items[ZBX_HISTORY_SOURCE_SQL], $source);
-		}
-
-		$min_clock = min($min_clock);
-
-		// If DB clock column is corrupted having negative numbers, return min clock from max possible history storage.
-		if ($min_clock == 0) {
-			if ($item_types[ITEM_VALUE_TYPE_FLOAT] || $item_types[ITEM_VALUE_TYPE_UINT64]) {
-				$min_clock = time() - max($max['history'], $max['trends']);
-
-				/*
-				 * In case history storage exceeds the maximum time difference between current year and minimum 1970
-				 * (for example year 2014 - 200 years < year 1970), correct year to 1970 (unix time timestamp 0).
-				 */
-				if ($min_clock < 0) {
-					$min_clock = 0;
-				}
-			}
-			else {
-				$min_clock = time() - SEC_PER_YEAR;
-			}
-		}
-
-		return $min_clock;
-	}
-
-	/**
-	 * Elasticsearch specific implementation of getMinClock.
-	 *
-	 * @see CHistoryManager::getMinClock
-	 */
-	private function getMinClockFromElasticsearch(array $items) {
-		$query = [
-			'aggs'=> [
-				'min_clock' => [
-					'min' => [
-						'field' => 'clock'
-					]
-				]
-			],
-			'size'=> 0
-		];
-
-		$min_clock = [];
-
-		foreach (self::getElasticsearchEndpoints(array_keys($items)) as $type => $endpoint) {
-			$query['query']['terms']['itemid'] = $items[$type];
-			$data = CElasticsearchHelper::query('POST', $endpoint, $query);
-
-			// Field value_as_string is used as a workaround for date aggregation being presented as milliseconds.
-			if (array_key_exists('min_clock', $data) && array_key_exists('value_as_string', $data['min_clock'])) {
-				$min_clock[] = $data['min_clock']['value_as_string'];
-			}
-		}
-
-		if ($min_clock) {
-			return min($min_clock);
-		}
-
-		return null;
-	}
-
-	/**
-	 * SQL specific implementation of getMinClock.
-	 *
-	 * @see CHistoryManager::getMinClock
-	 */
-	private function getMinClockFromSql(array $items, $source) {
-		$sql_unions = [];
-
-		foreach ($items as $type => $itemids) {
-			if (!$itemids) {
-				continue;
-			}
-
-			switch ($type) {
-				case ITEM_VALUE_TYPE_FLOAT:
-					$sql_from = $source;
-					break;
-				case ITEM_VALUE_TYPE_STR:
-					$sql_from = 'history_str';
-					break;
-				case ITEM_VALUE_TYPE_LOG:
-					$sql_from = 'history_log';
-					break;
-				case ITEM_VALUE_TYPE_UINT64:
-					$sql_from = $source.'_uint';
-					break;
-				case ITEM_VALUE_TYPE_TEXT:
-					$sql_from = 'history_text';
-					break;
-				default:
-					$sql_from = 'history';
-			}
-
-			foreach ($itemids as $itemid) {
-				$sql_unions[] =
-					'SELECT MIN(h.clock) AS clock'.
-					' FROM '.$sql_from.' h'.
-					' WHERE h.itemid='.zbx_dbstr($itemid);
-			}
-		}
-
-		$row = DBfetch(DBselect(
-			'SELECT MIN(h.clock) AS min_clock'.
-			' FROM ('.implode(' UNION ALL ', $sql_unions).') h'
-		));
-
-		return $row['min_clock'];
-	}
-
 	/**
 	 * Clear item history and trends by provided item IDs. History is deleted from both SQL and Elasticsearch.
 	 *
-	 * @param array $itemids    item ids to delete history for
+	 * @param array $items  Key - itemid, value - value_type.
 	 *
 	 * @return bool
 	 */
-	public function deleteHistory(array $itemids) {
-		return $this->deleteHistoryFromSql($itemids) && $this->deleteHistoryFromElasticsearch($itemids);
+	public function deleteHistory(array $items) {
+		return $this->deleteHistoryFromSql($items) && $this->deleteHistoryFromElasticsearch(array_keys($items));
 	}
 
 	/**
@@ -853,7 +1075,7 @@ class CHistoryManager {
 		global $HISTORY;
 
 		if (is_array($HISTORY) && array_key_exists('types', $HISTORY) && is_array($HISTORY['types'])
-				&& count($HISTORY['types'] > 0)) {
+				&& count($HISTORY['types']) > 0) {
 
 			$query = [
 				'query' => [
@@ -883,22 +1105,47 @@ class CHistoryManager {
 	 *
 	 * @see CHistoryManager::deleteHistory
 	 */
-	private function deleteHistoryFromSql(array $itemids) {
-		return DBexecute('DELETE FROM trends WHERE '.dbConditionInt('itemid', $itemids))
-				&& DBexecute('DELETE FROM trends_uint WHERE '.dbConditionInt('itemid', $itemids))
-				&& DBexecute('DELETE FROM history_text WHERE '.dbConditionInt('itemid', $itemids))
-				&& DBexecute('DELETE FROM history_log WHERE '.dbConditionInt('itemid', $itemids))
-				&& DBexecute('DELETE FROM history_uint WHERE '.dbConditionInt('itemid', $itemids))
-				&& DBexecute('DELETE FROM history_str WHERE '.dbConditionInt('itemid', $itemids))
-				&& DBexecute('DELETE FROM history WHERE '.dbConditionInt('itemid', $itemids));
+	private function deleteHistoryFromSql(array $items) {
+		global $DB;
+		$config = select_config();
+
+		$item_tables = array_map('self::getTableName', array_unique($items));
+		$table_names = array_flip(self::getTableName());
+
+		if (in_array(ITEM_VALUE_TYPE_UINT64, $items)) {
+			$item_tables[] = 'trends_uint';
+			$table_names['trends_uint'] = ITEM_VALUE_TYPE_UINT64;
+		}
+
+		if (in_array(ITEM_VALUE_TYPE_FLOAT, $items)) {
+			$item_tables[] = 'trends';
+			$table_names['trends'] = ITEM_VALUE_TYPE_FLOAT;
+		}
+
+		if ($DB['TYPE'] == ZBX_DB_POSTGRESQL && $config['db_extension'] == ZBX_DB_EXTENSION_TIMESCALEDB
+				&& PostgresqlDbBackend::isCompressed($item_tables)) {
+			error(_('Some of the history for this item may be compressed, deletion is not available.'));
+
+			return false;
+		}
+
+		foreach ($item_tables as $table_name) {
+			$itemids = array_keys(array_intersect($items, [(string) $table_names[$table_name]]));
+
+			if (!DBexecute('DELETE FROM '.$table_name.' WHERE '.dbConditionInt('itemid', $itemids))) {
+				return false;
+			}
+		}
+
+		return true;
 	}
 
 	/**
 	 * Get type name by value type id.
 	 *
-	 * @param int $value_type    value type id
+	 * @param int $value_type  Value type id.
 	 *
-	 * @return string    value type name
+	 * @return string  Value type name.
 	 */
 	public static function getTypeNameByTypeId($value_type) {
 		$mapping = [
@@ -920,9 +1167,9 @@ class CHistoryManager {
 	/**
 	 * Get type id by value type name.
 	 *
-	 * @param int $type_name    value type name
+	 * @param int $type_name  Value type name.
 	 *
-	 * @return int    value type id
+	 * @return int  Value type id.
 	 */
 	public static function getTypeIdByTypeName($type_name) {
 		$mapping = [
@@ -944,9 +1191,9 @@ class CHistoryManager {
 	/**
 	 * Get data source (SQL or Elasticsearch) type based on value type id.
 	 *
-	 * @param int $value_type    value type id
+	 * @param int $value_type  Value type id.
 	 *
-	 * @return string    data source type
+	 * @return string  Data source type.
 	 */
 	public static function getDataSourceType($value_type) {
 		static $cache = [];
@@ -1011,9 +1258,9 @@ class CHistoryManager {
 	/**
 	 * Get endpoints for Elasticsearch requests.
 	 *
-	 * @param mixed $value_types    value type(s)
+	 * @param mixed $value_types  Value type(s).
 	 *
-	 * @return array    Elasticsearch query endpoints
+	 * @return array  Elasticsearch query endpoints.
 	 */
 	public static function getElasticsearchEndpoints($value_types, $action = '_search') {
 		if (!is_array($value_types)) {
@@ -1031,7 +1278,7 @@ class CHistoryManager {
 
 		foreach ($indices as $type => $index) {
 			if (($url = self::getElasticsearchUrl($index)) !== null) {
-				$endponts[$type] = $url.$index.'/values/'.$action;
+				$endponts[$type] = $url.$index.'*/'.$action;
 			}
 		}
 
@@ -1041,11 +1288,11 @@ class CHistoryManager {
 	/**
 	 * Return the name of the table where the data for the given value type is stored.
 	 *
-	 * @param int $value_type    value type
+	 * @param int $value_type  Value type.
 	 *
-	 * @return string    table name
+	 * @return string|array  Table name | all tables.
 	 */
-	public static function getTableName($value_type) {
+	public static function getTableName($value_type = null) {
 		$tables = [
 			ITEM_VALUE_TYPE_LOG => 'history_log',
 			ITEM_VALUE_TYPE_TEXT => 'history_text',
@@ -1054,15 +1301,15 @@ class CHistoryManager {
 			ITEM_VALUE_TYPE_UINT64 => 'history_uint'
 		];
 
-		return $tables[$value_type];
+		return ($value_type === null) ? $tables : $tables[$value_type];
 	}
 
 	/**
 	 * Returns the items grouped by the storage type.
 	 *
-	 * @param array $items     an array of items with the 'value_type' property
+	 * @param array $items  An array of items with the 'value_type' property.
 	 *
-	 * @return array    an array with storage type as a keys and item arrays as a values
+	 * @return array  An array with storage type as a keys and item arrays as a values.
 	 */
 	private function getItemsGroupedByStorage(array $items) {
 		$grouped_items = [];
