@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -426,8 +426,8 @@ class CEvent extends CApiService {
 
 		// tags
 		if ($options['tags'] !== null && $options['tags']) {
-			$sqlParts['where'][] = self::getTagsWhereCondition($options['tags'], $options['evaltype'], 'event_tag',
-				'et', 'e', 'eventid'
+			$sqlParts['where'][] = CApiTagHelper::addWhereCondition($options['tags'], $options['evaltype'], 'e',
+				'event_tag', 'eventid'
 			);
 		}
 
@@ -475,7 +475,7 @@ class CEvent extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($event = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -493,73 +493,6 @@ class CEvent extends CApiService {
 		return $result;
 	}
 
-	/**
-	 * Returns SQL condition for tag filters.
-	 *
-	 * @param array  $tags
-	 * @param string $tags[]['tag']
-	 * @param int    $tags[]['operator']
-	 * @param string $tags[]['value']
-	 * @param int    $evaltype
-	 * @param string $table
-	 * @param string $alias
-	 * @param string $parent_alias
-	 * @param string $field
-	 *
-	 * @return array
-	 */
-	public static function getTagsWhereCondition(array $tags, $evaltype, $table, $alias, $parent_alias, $field) {
-		$values_by_tag = [];
-
-		foreach ($tags as $tag) {
-			$operator = array_key_exists('operator', $tag) ? $tag['operator'] : TAG_OPERATOR_LIKE;
-			$value = array_key_exists('value', $tag) ? $tag['value'] : '';
-
-			if (!array_key_exists($tag['tag'], $values_by_tag) || is_array($values_by_tag[$tag['tag']])) {
-				if ($operator == TAG_OPERATOR_EQUAL) {
-					$values_by_tag[$tag['tag']][] = $alias.'.value='.zbx_dbstr($value);
-				}
-				elseif ($value !== '') {
-					$value = str_replace('!', '!!', $value);
-					$value = str_replace('%', '!%', $value);
-					$value = str_replace('_', '!_', $value);
-					$value = '%'.mb_strtoupper($value).'%';
-
-					$values_by_tag[$tag['tag']][] = 'UPPER('.$alias.'.value) LIKE '.zbx_dbstr($value)." ESCAPE '!'";
-				}
-				// ($value === '') - all other conditions can be omitted
-				else {
-					$values_by_tag[$tag['tag']] = false;
-				}
-			}
-		}
-
-		$sql_where = [];
-
-		foreach ($values_by_tag as $tag => $values) {
-			if (!is_array($values) || count($values) == 0) {
-				$values = '';
-			}
-			elseif (count($values) == 1) {
-				$values = ' AND '.$values[0];
-			}
-			else {
-				$values = $values ? ' AND ('.implode(' OR ', $values).')' : '';
-			}
-
-			$sql_where[] = 'EXISTS ('.
-				'SELECT NULL'.
-				' FROM '.$table.' '.$alias.
-				' WHERE '.$parent_alias.'.'.$field.'='.$alias.'.'.$field.
-					' AND '.$alias.'.tag='.zbx_dbstr($tag).$values.
-			')';
-		}
-
-		$sql_where = implode(($evaltype == TAG_EVAL_TYPE_OR) ? ' OR ' : ' AND ', $sql_where);
-
-		return (count($values_by_tag) > 1 && $evaltype == TAG_EVAL_TYPE_OR) ? '('.$sql_where.')' : $sql_where;
-	}
-
 	/**
 	 * Validates the input parameters for the get() method.
 	 *
@@ -607,6 +540,7 @@ class CEvent extends CApiService {
 	 *                                       - 0x02 - ZBX_PROBLEM_UPDATE_ACKNOWLEDGE
 	 *                                       - 0x04 - ZBX_PROBLEM_UPDATE_MESSAGE
 	 *                                       - 0x08 - ZBX_PROBLEM_UPDATE_SEVERITY
+	 *                                       - 0x10 - ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
 	 *
 	 * @return array
 	 */
@@ -629,6 +563,7 @@ class CEvent extends CApiService {
 		]);
 
 		$ack_eventids = [];
+		$unack_eventids = [];
 		$sev_change_eventids = [];
 		$acknowledges = [];
 		$time = time();
@@ -651,6 +586,13 @@ class CEvent extends CApiService {
 				$ack_eventids[] = $eventid;
 			}
 
+			// Perform ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE action flag.
+			if (($data['action'] & ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE) == ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
+					&& $event['acknowledged'] == EVENT_ACKNOWLEDGED) {
+				$action |= ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE;
+				$unack_eventids[] = $eventid;
+			}
+
 			// Perform ZBX_PROBLEM_UPDATE_MESSAGE action flag.
 			if (($data['action'] & ZBX_PROBLEM_UPDATE_MESSAGE) == ZBX_PROBLEM_UPDATE_MESSAGE) {
 				$action |= ZBX_PROBLEM_UPDATE_MESSAGE;
@@ -682,6 +624,19 @@ class CEvent extends CApiService {
 
 		// Make changes in problem and events tables.
 		if ($acknowledges) {
+			// Unacknowledge problems and events.
+			if ($unack_eventids) {
+				DB::update('problem', [
+					'values' => ['acknowledged' => EVENT_NOT_ACKNOWLEDGED],
+					'where' => ['eventid' => $unack_eventids]
+				]);
+
+				DB::update('events', [
+					'values' => ['acknowledged' => EVENT_NOT_ACKNOWLEDGED],
+					'where' => ['eventid' => $unack_eventids]
+				]);
+			}
+
 			// Acknowledge problems and events.
 			if ($ack_eventids) {
 				DB::update('problem', [
@@ -778,6 +733,7 @@ class CEvent extends CApiService {
 	 *                                           - 0x02 - ZBX_PROBLEM_UPDATE_ACKNOWLEDGE
 	 *                                           - 0x04 - ZBX_PROBLEM_UPDATE_MESSAGE
 	 *                                           - 0x08 - ZBX_PROBLEM_UPDATE_SEVERITY
+	 *                                           - 0x10 - ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
 	 *
 	 * @throws APIException                    If the input is invalid.
 	 */
@@ -796,9 +752,9 @@ class CEvent extends CApiService {
 		$data['eventids'] = zbx_toArray($data['eventids']);
 		$data['eventids'] = array_keys(array_flip($data['eventids']));
 
-		// Chack that at least one valid flag is set.
+		// Check that at least one valid flag is set.
 		$action_mask = ZBX_PROBLEM_UPDATE_CLOSE | ZBX_PROBLEM_UPDATE_ACKNOWLEDGE | ZBX_PROBLEM_UPDATE_MESSAGE
-				| ZBX_PROBLEM_UPDATE_SEVERITY;
+				| ZBX_PROBLEM_UPDATE_SEVERITY | ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE;
 
 		if (($data['action'] & $action_mask) != $data['action']) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
@@ -810,6 +766,15 @@ class CEvent extends CApiService {
 		$has_message_action = (($data['action'] & ZBX_PROBLEM_UPDATE_MESSAGE) == ZBX_PROBLEM_UPDATE_MESSAGE);
 		$has_severity_action = (($data['action'] & ZBX_PROBLEM_UPDATE_SEVERITY) == ZBX_PROBLEM_UPDATE_SEVERITY);
 
+		if (($data['action'] & ZBX_PROBLEM_UPDATE_ACKNOWLEDGE) &&
+				($data['action'] & ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect value for field "%1$s": %2$s.', 'action',
+				_s('value must be one of %1$s', implode(', ', [ZBX_PROBLEM_UPDATE_ACKNOWLEDGE,
+					ZBX_PROBLEM_UPDATE_UNACKNOWLEDGE
+				]))
+			));
+		}
+
 		$events = $this->get([
 			'output' => [],
 			'selectRelatedObject' => $has_close_action ? ['manual_close'] : null,
@@ -935,31 +900,26 @@ class CEvent extends CApiService {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
 		if (!$options['countOutput']) {
+			// Select fields from event_recovery table using LEFT JOIN.
 			if ($this->outputIsRequested('r_eventid', $options['output'])) {
-				// Select fields from event_recovery table using LEFT JOIN.
-
 				$sqlParts['select']['r_eventid'] = 'er1.r_eventid';
-				$sqlParts['left_join'][] = ['from' => 'event_recovery er1', 'on' => 'er1.eventid=e.eventid'];
-				$sqlParts['left_table'] = 'e';
+				$sqlParts['left_join'][] = ['alias' => 'er1', 'table' => 'event_recovery', 'using' => 'eventid'];
+				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 			}
 
-			if ($this->outputIsRequested('c_eventid', $options['output'])
-					|| $this->outputIsRequested('correlationid', $options['output'])
-					|| $this->outputIsRequested('userid', $options['output'])) {
-				// Select fields from event_recovery table using LEFT JOIN.
+			// Select fields from event_recovery table using LEFT JOIN.
+			$left_join = false;
 
-				if ($this->outputIsRequested('c_eventid', $options['output'])) {
-					$sqlParts['select']['c_eventid'] = 'er2.c_eventid';
-				}
-				if ($this->outputIsRequested('correlationid', $options['output'])) {
-					$sqlParts['select']['correlationid'] = 'er2.correlationid';
-				}
-				if ($this->outputIsRequested('userid', $options['output'])) {
-					$sqlParts['select']['userid'] = 'er2.userid';
+			foreach (['c_eventid', 'correlationid', 'userid'] as $field) {
+				if ($this->outputIsRequested($field, $options['output'])) {
+					$sqlParts['select'][$field] = 'er2.'.$field;
+					$left_join = true;
 				}
+			}
 
-				$sqlParts['left_join'][] = ['from' => 'event_recovery er2', 'on' => 'er2.r_eventid=e.eventid'];
-				$sqlParts['left_table'] = 'e';
+			if ($left_join) {
+				$sqlParts['left_join'][] = ['alias' => 'er2', 'table' => 'event_recovery', 'using' => 'r_eventid'];
+				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 			}
 
 			if ($options['selectRelatedObject'] !== null || $options['selectHosts'] !== null) {
@@ -976,6 +936,23 @@ class CEvent extends CApiService {
 
 		$eventids = array_keys($result);
 
+		// Adding operational data.
+		if ($this->outputIsRequested('opdata', $options['output'])) {
+			$events = DBFetchArrayAssoc(DBselect(
+				'SELECT e.eventid,e.clock,e.ns,t.triggerid,t.expression,t.opdata'.
+				' FROM events e'.
+				' JOIN triggers t ON t.triggerid=e.objectid'.
+				' WHERE '.dbConditionInt('e.eventid', $eventids)
+			), 'eventid');
+
+			foreach ($result as $eventid => $event) {
+				$result[$eventid]['opdata'] =
+					(array_key_exists($eventid, $events) && $events[$eventid]['opdata'] !== '')
+						? CMacrosResolverHelper::resolveTriggerOpdata($events[$eventid], ['events' => true])
+						: '';
+			}
+		}
+
 		// adding hosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
 			// trigger events
@@ -1079,7 +1056,7 @@ class CEvent extends CApiService {
 				]);
 				$sqlParts['order'][] = 'a.clock DESC';
 
-				$acknowledges = DBFetchArrayAssoc(DBselect($this->createSelectQueryFromParts($sqlParts)), 'acknowledgeid');
+				$acknowledges = DBFetchArrayAssoc(DBselect(self::createSelectQueryFromParts($sqlParts)), 'acknowledgeid');
 
 				// if the user data is requested via extended output or specified fields, join the users table
 				$userFields = ['alias', 'name', 'surname'];
@@ -1178,6 +1155,43 @@ class CEvent extends CApiService {
 			unset($row);
 		}
 
+		// Resolve webhook urls.
+		if ($this->outputIsRequested('urls', $options['output'])) {
+			$tags_options = [
+				'output' => ['eventid', 'tag', 'value'],
+				'filter' => ['eventid' => $eventids]
+			];
+			$tags = DBselect(DB::makeSql('event_tag', $tags_options));
+
+			$events = [];
+
+			foreach ($result as $event) {
+				$events[$event['eventid']]['tags'] = [];
+			}
+
+			while ($tag = DBfetch($tags)) {
+				$events[$tag['eventid']]['tags'][] = [
+					'tag' => $tag['tag'],
+					'value' => $tag['value']
+				];
+			}
+
+			$urls = DB::select('media_type', [
+				'output' => ['event_menu_url', 'event_menu_name'],
+				'filter' => [
+					'type' => MEDIA_TYPE_WEBHOOK,
+					'status' => MEDIA_TYPE_STATUS_ACTIVE,
+					'show_event_menu' => ZBX_EVENT_MENU_SHOW
+				]
+			]);
+
+			$events = CMacrosResolverHelper::resolveMediaTypeUrls($events, $urls);
+
+			foreach ($events as $eventid => $event) {
+				$result[$eventid]['urls'] = $event['urls'];
+			}
+		}
+
 		// Adding event tags.
 		if ($options['selectTags'] !== null && $options['selectTags'] != API_OUTPUT_COUNT) {
 			if ($options['selectTags'] === API_OUTPUT_EXTEND) {
@@ -1239,7 +1253,7 @@ class CEvent extends CApiService {
 	 * @param array $sqlParts
 	 * @param int   $value
 	 *
-	 * @return string
+	 * @return array
 	 */
 	protected static function addTagFilterSqlParts(array $usrgrpids, array $sqlParts, $value) {
 		$tag_filters = self::getTagFilters($usrgrpids);
