@@ -484,10 +484,14 @@ abstract class CGraphGeneral extends CApiService {
 		switch (get_class($this)) {
 			case 'CGraph':
 				$error_cannot_set = _('Cannot set "%1$s" for graph "%2$s".');
+				$api_input_rules = ['type' => API_OBJECT, 'fields' => []];
 				break;
 
 			case 'CGraphPrototype':
 				$error_cannot_set = _('Cannot set "%1$s" for graph prototype "%2$s".');
+				$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+					'discover' => ['type' => API_INT32, 'in' => implode(',', [GRAPH_DISCOVER, GRAPH_NO_DISCOVER])]
+				]];
 				break;
 
 			default:
@@ -496,10 +500,17 @@ abstract class CGraphGeneral extends CApiService {
 
 		$read_only_fields = ['templateid', 'flags'];
 
-		foreach ($graphs as $graph) {
+		foreach ($graphs as $key => $graph) {
 			$this->checkNoParameters($graph, $read_only_fields, $error_cannot_set, $graph['name']);
 
+			$data = array_intersect_key($graph, $api_input_rules['fields']);
+
+			if (!CApiInputValidator::validate($api_input_rules, $data, '/'.($key + 1), $error)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+			}
+
 			$templatedGraph = false;
+
 			if (isset($graph['gitems'])) {
 				// check if new items are from same templated host
 				$graphHosts = API::Host()->get([
@@ -615,10 +626,14 @@ abstract class CGraphGeneral extends CApiService {
 		switch (get_class($this)) {
 			case 'CGraph':
 				$error_cannot_update = _('Cannot update "%1$s" for graph "%2$s".');
+				$api_input_rules = ['type' => API_OBJECT, 'fields' => []];
 				break;
 
 			case 'CGraphPrototype':
 				$error_cannot_update = _('Cannot update "%1$s" for graph prototype "%2$s".');
+				$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+					'discover' => ['type' => API_INT32, 'in' => implode(',', [GRAPH_DISCOVER, GRAPH_NO_DISCOVER])]
+				]];
 				break;
 
 			default:
@@ -627,9 +642,15 @@ abstract class CGraphGeneral extends CApiService {
 
 		$read_only_fields = ['templateid', 'flags'];
 
-		foreach ($graphs as $graph) {
+		foreach ($graphs as $key => $graph) {
 			$this->checkNoParameters($graph, $read_only_fields, $error_cannot_update, $graph['name']);
 
+			$data = array_intersect_key($graph, $api_input_rules['fields']);
+
+			if (!CApiInputValidator::validate($api_input_rules, $data, '/'.($key + 1), $error)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+			}
+
 			$templatedGraph = false;
 
 			if (isset($graph['gitems'])) {
