@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 
 /**
  * Class containing methods for operations with screen items.
- *
- * @package API
  */
 class CScreenItem extends CApiService {
 
@@ -34,17 +32,16 @@ class CScreenItem extends CApiService {
 	 *
 	 * @var array
 	 */
-	protected static $resourceTypes = array(
+	protected static $resourceTypes = [
 		SCREEN_RESOURCE_GRAPH,
 		SCREEN_RESOURCE_SIMPLE_GRAPH,
 		SCREEN_RESOURCE_MAP,
 		SCREEN_RESOURCE_PLAIN_TEXT,
-		SCREEN_RESOURCE_HOSTS_INFO,
-		SCREEN_RESOURCE_TRIGGERS_INFO,
+		SCREEN_RESOURCE_HOST_INFO,
+		SCREEN_RESOURCE_TRIGGER_INFO,
 		SCREEN_RESOURCE_SERVER_INFO,
 		SCREEN_RESOURCE_CLOCK,
-		SCREEN_RESOURCE_SCREEN,
-		SCREEN_RESOURCE_TRIGGERS_OVERVIEW,
+		SCREEN_RESOURCE_TRIGGER_OVERVIEW,
 		SCREEN_RESOURCE_DATA_OVERVIEW,
 		SCREEN_RESOURCE_URL,
 		SCREEN_RESOURCE_ACTIONS,
@@ -54,25 +51,25 @@ class CScreenItem extends CApiService {
 		SCREEN_RESOURCE_HOST_TRIGGERS,
 		SCREEN_RESOURCE_LLD_GRAPH,
 		SCREEN_RESOURCE_LLD_SIMPLE_GRAPH
-	);
+	];
 
-	protected $sortColumns = array(
+	protected $sortColumns = [
 		'screenitemid',
 		'screenid'
-	);
+	];
 
 	public function __construct() {
 		parent::__construct();
 
-		$this->getOptions = zbx_array_merge($this->getOptions, array(
+		$this->getOptions = zbx_array_merge($this->getOptions, [
 			'screenitemids'	=> null,
 			'screenids'		=> null,
-			'editable'		=> null,
+			'editable'		=> false,
 			'sortfield'		=> '',
 			'sortorder'		=> '',
-			'preservekeys'	=> null,
-			'countOutput'	=> null
-		));
+			'preservekeys'	=> false,
+			'countOutput'	=> false
+		]);
 	}
 
 	/**
@@ -86,7 +83,7 @@ class CScreenItem extends CApiService {
 	 *
 	 * @return array
 	 */
-	public function get(array $options = array()) {
+	public function get(array $options = []) {
 		$options = zbx_array_merge($this->getOptions, $options);
 
 		// build and execute query
@@ -94,15 +91,15 @@ class CScreenItem extends CApiService {
 		$res = DBselect($sql, $options['limit']);
 
 		// fetch results
-		$result = array();
+		$result = [];
 		while ($row = DBfetch($res)) {
 			// count query, return a single result
-			if ($options['countOutput'] !== null) {
+			if ($options['countOutput']) {
 				$result = $row['rowscount'];
 			}
 			// normal select query
 			else {
-				if ($options['preservekeys'] !== null) {
+				if ($options['preservekeys']) {
 					$result[$row['screenitemid']] = $row;
 				}
 				else {
@@ -128,7 +125,7 @@ class CScreenItem extends CApiService {
 
 		$screenItemIds = DB::insert($this->tableName(), $screenItems);
 
-		return array('screenitemids' => $screenItemIds);
+		return ['screenitemids' => $screenItemIds];
 	}
 
 	/**
@@ -139,10 +136,17 @@ class CScreenItem extends CApiService {
 	 * @param array $screenItems
 	 */
 	protected function validateCreate(array $screenItems) {
-		$screenItemDBfields = array(
+		$screenItemDBfields = [
 			'screenid' => null,
 			'resourcetype' => null
-		);
+		];
+
+		$defaults = [
+			'x' => 0,
+			'y' => 0,
+			'colspan' => 1,
+			'rowspan' => 1
+		];
 
 		foreach ($screenItems as &$screenItem) {
 			if (!check_db_fields($screenItemDBfields, $screenItem)) {
@@ -150,40 +154,42 @@ class CScreenItem extends CApiService {
 			}
 
 			unset($screenItem['screenitemid']);
+			$screenItem += $defaults;
 		}
 		unset($screenItem);
 
+		$this->validateItemsURL($screenItems);
+
 		$screenIds = array_keys(array_flip(zbx_objectValues($screenItems, 'screenid')));
 
-		$dbScreens = API::Screen()->get(array(
-			'output' => array('screenid', 'hsize', 'vsize', 'name'),
+		$dbScreens = API::Screen()->get([
+			'output' => ['screenid', 'hsize', 'vsize', 'name'],
 			'screenids' => $screenIds,
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 
 		if (count($dbScreens) < count($screenIds)) {
-			$dbTemplateScreens = API::TemplateScreen()->get(array(
-				'output' => array('screenid', 'hsize', 'vsize', 'name'),
+			$dbTemplateScreens = API::TemplateScreen()->get([
+				'output' => ['screenid', 'hsize', 'vsize', 'name'],
 				'screenids' => $screenIds,
 				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			if ($dbTemplateScreens) {
 				$dbScreens = zbx_array_merge($dbScreens, $dbTemplateScreens);
 			}
 		}
 
-		$dbScreenItems = $this->get(array(
-			'output' => array('screenitemid', 'screenid', 'x', 'y', 'rowspan', 'colspan'),
-			'screenids' => $screenIds,
-			'editable' => true,
+		$dbScreenItems = API::getApiService()->select($this->tableName(), [
+			'output' => ['screenitemid', 'screenid', 'x', 'y', 'rowspan', 'colspan'],
+			'filter' => ['screenid' => array_keys($dbScreens)],
 			'preservekeys' => true
-		));
+		]);
 
 		$this->checkInput($screenItems, $dbScreenItems);
-		$this->checkDuplicateResourceInCell($screenItems, $dbScreenItems, $dbScreens);
+		$this->checkDuplicateResourceInCell(array_merge($screenItems, $dbScreenItems), $dbScreens);
 
 		foreach ($screenItems as $screenItem) {
 			$this->checkSpans($screenItem, $dbScreens[$screenItem['screenid']]);
@@ -206,8 +212,8 @@ class CScreenItem extends CApiService {
 
 		$screenItems = zbx_toHash($screenItems, 'screenitemid');
 
-		$update = array();
-		$screenItemIds = array();
+		$update = [];
+		$screenItemIds = [];
 
 		foreach ($screenItems as $screenItem) {
 			$screenItemId = $screenItem['screenitemid'];
@@ -234,17 +240,17 @@ class CScreenItem extends CApiService {
 				}
 			}
 
-			$update[] = array(
+			$update[] = [
 				'values' => $screenItem,
-				'where' => array('screenitemid' => $screenItemId)
-			);
+				'where' => ['screenitemid' => $screenItemId]
+			];
 
 			$screenItemIds[] = $screenItemId;
 		}
 
 		DB::update($this->tableName(), $update);
 
-		return array('screenitemids' => $screenItemIds);
+		return ['screenitemids' => $screenItemIds];
 	}
 
 	/**
@@ -255,9 +261,9 @@ class CScreenItem extends CApiService {
 	 * @param array $screenItems
 	 */
 	protected function validateUpdate(array $screenItems) {
-		$screenItemDBfields = array(
+		$screenItemDBfields = [
 			'screenitemid' => null
-		);
+		];
 
 		foreach ($screenItems as $screenItem) {
 			if (!check_db_fields($screenItemDBfields, $screenItem)) {
@@ -265,41 +271,43 @@ class CScreenItem extends CApiService {
 			}
 		}
 
+		$this->validateItemsURL($screenItems);
+
 		$screenItems = zbx_toHash($screenItems, 'screenitemid');
 		$screenItemIds = array_keys($screenItems);
 
-		$dbScreens = API::Screen()->get(array(
-			'output' => array('screenid', 'hsize', 'vsize', 'name'),
+		$dbScreens = API::Screen()->get([
+			'output' => ['screenid', 'hsize', 'vsize', 'name'],
 			'screenitemids' => $screenItemIds,
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 
-		$dbTemplateScreens = API::TemplateScreen()->get(array(
-			'output' => array('screenid', 'hsize', 'vsize', 'name'),
+		$dbTemplateScreens = API::TemplateScreen()->get([
+			'output' => ['screenid', 'hsize', 'vsize', 'name'],
 			'screenitemids' => $screenItemIds,
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 
 		if ($dbTemplateScreens) {
 			$dbScreens = zbx_array_merge($dbScreens, $dbTemplateScreens);
 		}
 
-		$dbScreenItems = $this->get(array(
-			'output' => array('screenitemid', 'screenid', 'x', 'y', 'rowspan', 'colspan', 'resourcetype', 'resourceid',
-				'style'),
-			'screenitemids' => $screenItemIds,
-			'editable' => true,
+		$dbScreenItems = API::getApiService()->select($this->tableName(), [
+			'output' => ['screenitemid', 'screenid', 'x', 'y', 'rowspan', 'colspan', 'resourcetype', 'resourceid',
+				'style'
+			],
+			'filter' => ['screenid' => array_keys($dbScreens)],
 			'preservekeys' => true
-		));
+		]);
 
 		$screenItems = $this->extendObjects($this->tableName(), $screenItems,
-			array('screenid', 'x', 'y', 'rowspan', 'colspan', 'style')
+			['screenid', 'x', 'y', 'rowspan', 'colspan', 'style']
 		);
 
 		$this->checkInput($screenItems, $dbScreenItems);
-		$this->checkDuplicateResourceInCell($screenItems, $dbScreenItems, $dbScreens);
+		$this->checkDuplicateResourceInCell($screenItems + $dbScreenItems, $dbScreens);
 
 		foreach ($screenItems as $screenItem) {
 			$this->checkSpans($screenItem, $dbScreens[$screenItem['screenid']]);
@@ -317,11 +325,11 @@ class CScreenItem extends CApiService {
 	 * @return array
 	 */
 	public function updateByPosition(array $screenItems) {
-		$screenItemDBfields = array(
+		$screenItemDBfields = [
 			'screenid' => null,
 			'x' => null,
 			'y' => null
-		);
+		];
 
 		foreach ($screenItems as $screenItem) {
 			if (!check_db_fields($screenItemDBfields, $screenItem)) {
@@ -329,14 +337,14 @@ class CScreenItem extends CApiService {
 			}
 		}
 
-		$dbScreenItems = $this->get(array(
-			'output' => array('screenitemid', 'screenid', 'x', 'y'),
+		$dbScreenItems = $this->get([
+			'output' => ['screenitemid', 'screenid', 'x', 'y'],
 			'screenids' => zbx_objectValues($screenItems, 'screenid'),
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 
-		$create = $update = $affectedIds = array();
+		$create = $update = $affectedIds = [];
 
 		foreach ($screenItems as $screenItem) {
 			foreach ($dbScreenItems as $dbScreenItem) {
@@ -364,7 +372,7 @@ class CScreenItem extends CApiService {
 			$affectedIds = array_merge($affectedIds, $screenItems['screenitemids']);
 		}
 
-		return array('screenitemids' => $affectedIds);
+		return ['screenitemids' => $affectedIds];
 	}
 
 	/**
@@ -376,11 +384,11 @@ class CScreenItem extends CApiService {
 	 */
 	public function delete(array $screenItemIds) {
 		// check permissions
-		$dbScreenItems = $this->get(array(
-			'output' => array('screenitemid'),
+		$dbScreenItems = $this->get([
+			'output' => ['screenitemid'],
 			'screenitemids' => $screenItemIds,
 			'preservekeys' => true
-		));
+		]);
 
 		foreach ($screenItemIds as $screenItemId) {
 			if (!isset($dbScreenItems[$screenItemId])) {
@@ -389,62 +397,11 @@ class CScreenItem extends CApiService {
 		}
 
 		// delete screen items
-		DB::delete($this->tableName(), array(
+		DB::delete($this->tableName(), [
 			'screenitemid' => $screenItemIds
-		));
+		]);
 
-		return array('screenitemids' => $screenItemIds);
-	}
-
-	/**
-	 * Returns true if the given screen items exist and are available for reading.
-	 *
-	 * @param array $screenItemIds
-	 *
-	 * @return bool
-	 */
-	public function isReadable(array $screenItemIds) {
-		if (!is_array($screenItemIds)) {
-			return false;
-		}
-		elseif (empty($screenItemIds)) {
-			return true;
-		}
-
-		$screenItemIds = array_unique($screenItemIds);
-
-		$count = $this->get(array(
-			'screenitemids' => $screenItemIds,
-			'countOutput' => true
-		));
-
-		return (count($screenItemIds) == $count);
-	}
-
-	/**
-	 * Returns true if the given screen items exist and are available for writing.
-	 *
-	 * @param array $screenItemIds	An array if screen item IDs
-	 *
-	 * @return bool
-	 */
-	public function isWritable(array $screenItemIds) {
-		if (!is_array($screenItemIds)) {
-			return false;
-		}
-		elseif (empty($screenItemIds)) {
-			return true;
-		}
-
-		$screenItemIds = array_unique($screenItemIds);
-
-		$count = $this->get(array(
-			'screenitemids' => $screenItemIds,
-			'editable' => true,
-			'countOutput' => true
-		));
-
-		return (count($screenItemIds) == $count);
+		return ['screenitemids' => $screenItemIds];
 	}
 
 	/**
@@ -459,25 +416,25 @@ class CScreenItem extends CApiService {
 	 * @param array $screenItems
 	 * @param array $dbScreenItems
 	 */
-	protected function checkInput(array $screenItems, array $dbScreenItems = array()) {
-		$hostGroupsIds = array();
-		$hostIds = array();
-		$graphIds = array();
-		$itemIds = array();
-		$mapIds = array();
-		$screenIds = array();
-		$itemPrototypeIds = array();
-		$graphPrototypeIds = array();
+	protected function checkInput(array $screenItems, array $dbScreenItems = []) {
+		$hostGroupsIds = [];
+		$hostIds = [];
+		$graphIds = [];
+		$itemIds = [];
+		$mapIds = [];
+		$screenIds = [];
+		$itemPrototypeIds = [];
+		$graphPrototypeIds = [];
 
-		$screenItems = $this->extendFromObjects($screenItems, $dbScreenItems, array('resourcetype'));
+		$screenItems = $this->extendFromObjects($screenItems, $dbScreenItems, ['resourcetype']);
 
-		$validStyles = array(
-			SCREEN_RESOURCE_CLOCK => array(TIME_TYPE_LOCAL, TIME_TYPE_SERVER, TIME_TYPE_HOST),
-			SCREEN_RESOURCE_DATA_OVERVIEW => array(STYLE_TOP, STYLE_LEFT),
-			SCREEN_RESOURCE_TRIGGERS_OVERVIEW => array(STYLE_TOP, STYLE_LEFT),
-			SCREEN_RESOURCE_HOSTS_INFO => array(STYLE_VERTICAL, STYLE_HORIZONTAL),
-			SCREEN_RESOURCE_TRIGGERS_INFO => array(STYLE_VERTICAL, STYLE_HORIZONTAL)
-		);
+		$validStyles = [
+			SCREEN_RESOURCE_CLOCK => [TIME_TYPE_LOCAL, TIME_TYPE_SERVER, TIME_TYPE_HOST],
+			SCREEN_RESOURCE_DATA_OVERVIEW => [STYLE_TOP, STYLE_LEFT],
+			SCREEN_RESOURCE_TRIGGER_OVERVIEW => [STYLE_TOP, STYLE_LEFT],
+			SCREEN_RESOURCE_HOST_INFO => [STYLE_VERTICAL, STYLE_HORIZONTAL],
+			SCREEN_RESOURCE_TRIGGER_INFO => [STYLE_VERTICAL, STYLE_HORIZONTAL]
+		];
 
 		foreach ($screenItems as $screenItem) {
 			// check permissions
@@ -495,12 +452,12 @@ class CScreenItem extends CApiService {
 
 			// check resource id
 			switch ($screenItem['resourcetype']) {
-				case SCREEN_RESOURCE_HOSTS_INFO:
-				case SCREEN_RESOURCE_TRIGGERS_INFO:
-				case SCREEN_RESOURCE_TRIGGERS_OVERVIEW:
+				case SCREEN_RESOURCE_HOST_INFO:
+				case SCREEN_RESOURCE_TRIGGER_INFO:
+				case SCREEN_RESOURCE_TRIGGER_OVERVIEW:
 				case SCREEN_RESOURCE_HOSTGROUP_TRIGGERS:
 				case SCREEN_RESOURCE_DATA_OVERVIEW:
-					$overviewResources = array(SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW);
+					$overviewResources = [SCREEN_RESOURCE_TRIGGER_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW];
 					if (in_array($screenItem['resourcetype'], $overviewResources)) {
 						if (!$screenItem['resourceid']) {
 							self::exception(ZBX_API_ERROR_PARAMETERS, _(
@@ -580,14 +537,6 @@ class CScreenItem extends CApiService {
 					$mapIds[$screenItem['resourceid']] = $screenItem['resourceid'];
 					break;
 
-				case SCREEN_RESOURCE_SCREEN:
-					if (!$screenItem['resourceid']) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _('No screen ID provided for screen element.'));
-					}
-
-					$screenIds[$screenItem['resourceid']] = $screenItem['resourceid'];
-					break;
-
 				case SCREEN_RESOURCE_ACTIONS:
 				case SCREEN_RESOURCE_SERVER_INFO:
 				case SCREEN_RESOURCE_SYSTEM_STATUS:
@@ -627,7 +576,7 @@ class CScreenItem extends CApiService {
 			// is set and valid for create method, and is valid for update method, if set
 			$dbScreenItem = isset($screenItem['screenitemid']) ? $dbScreenItems[$screenItem['screenitemid']] : null;
 
-			$lldResources = array(SCREEN_RESOURCE_LLD_GRAPH, SCREEN_RESOURCE_LLD_SIMPLE_GRAPH);
+			$lldResources = [SCREEN_RESOURCE_LLD_GRAPH, SCREEN_RESOURCE_LLD_SIMPLE_GRAPH];
 			if (in_array($screenItem['resourcetype'], $lldResources)) {
 				$set = isset($screenItem['max_columns']);
 				$valid = ($set && $this->isValidMaxColumns($screenItem['max_columns']));
@@ -646,12 +595,11 @@ class CScreenItem extends CApiService {
 
 		// check host groups
 		if ($hostGroupsIds) {
-			$dbHostGroups = API::HostGroup()->get(array(
-				'output' => array('groupid'),
+			$dbHostGroups = API::HostGroup()->get([
+				'output' => ['groupid'],
 				'groupids' => $hostGroupsIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			foreach ($hostGroupsIds as $hostGroupsId) {
 				if (!isset($dbHostGroups[$hostGroupsId])) {
@@ -664,12 +612,11 @@ class CScreenItem extends CApiService {
 
 		// check hosts
 		if ($hostIds) {
-			$dbHosts = API::Host()->get(array(
-				'output' => array('hostid'),
+			$dbHosts = API::Host()->get([
+				'output' => ['hostid'],
 				'hostids' => $hostIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			foreach ($hostIds as $hostId) {
 				if (!isset($dbHosts[$hostId])) {
@@ -682,12 +629,11 @@ class CScreenItem extends CApiService {
 
 		// check graphs
 		if ($graphIds) {
-			$dbGraphs = API::Graph()->get(array(
-				'output' => array('graphid'),
+			$dbGraphs = API::Graph()->get([
+				'output' => ['graphid'],
 				'graphids' => $graphIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			foreach ($graphIds as $graphId) {
 				if (!isset($dbGraphs[$graphId])) {
@@ -700,12 +646,11 @@ class CScreenItem extends CApiService {
 
 		// check graph prototypes
 		if ($graphPrototypeIds) {
-			$dbGraphPrototypes = API::GraphPrototype()->get(array(
-				'output' => array('graphid'),
+			$dbGraphPrototypes = API::GraphPrototype()->get([
+				'output' => ['graphid'],
 				'graphids' => $graphPrototypeIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			foreach ($graphPrototypeIds as $graphPrototypeId) {
 				if (!isset($dbGraphPrototypes[$graphPrototypeId])) {
@@ -718,13 +663,12 @@ class CScreenItem extends CApiService {
 
 		// check items
 		if ($itemIds) {
-			$dbItems = API::Item()->get(array(
-				'output' => array('itemid'),
+			$dbItems = API::Item()->get([
+				'output' => ['itemid'],
 				'itemids' => $itemIds,
-				'editable' => true,
 				'preservekeys' => true,
 				'webitems' => true
-			));
+			]);
 
 			foreach ($itemIds as $itemId) {
 				if (!isset($dbItems[$itemId])) {
@@ -737,12 +681,11 @@ class CScreenItem extends CApiService {
 
 		// check item prototypes
 		if ($itemPrototypeIds) {
-			$dbItemPrototypes = API::ItemPrototype()->get(array(
-				'output' => array('itemid'),
+			$dbItemPrototypes = API::ItemPrototype()->get([
+				'output' => ['itemid'],
 				'itemids' => $itemPrototypeIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			foreach ($itemPrototypeIds as $itemPrototypeId) {
 				if (!isset($dbItemPrototypes[$itemPrototypeId])) {
@@ -755,12 +698,11 @@ class CScreenItem extends CApiService {
 
 		// check maps
 		if ($mapIds) {
-			$dbMaps = API::Map()->get(array(
-				'output' => array('sysmapid'),
+			$dbMaps = API::Map()->get([
+				'output' => ['sysmapid'],
 				'sysmapids' => $mapIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			foreach ($mapIds as $mapId) {
 				if (!isset($dbMaps[$mapId])) {
@@ -773,20 +715,18 @@ class CScreenItem extends CApiService {
 
 		// check screens
 		if ($screenIds) {
-			$dbScreens = API::Screen()->get(array(
-				'output' => array('screenid'),
+			$dbScreens = API::Screen()->get([
+				'output' => ['screenid'],
 				'screenids' => $screenIds,
-				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			if (count($dbScreens) < count($screenIds)) {
-				$dbTemplateScreens = API::TemplateScreen()->get(array(
-					'output' => array('screenid'),
+				$dbTemplateScreens = API::TemplateScreen()->get([
+					'output' => ['screenid'],
 					'screenids' => $screenIds,
-					'editable' => true,
 					'preservekeys' => true
-				));
+				]);
 
 				if ($dbTemplateScreens) {
 					$dbScreens = zbx_array_merge($dbScreens, $dbTemplateScreens);
@@ -903,77 +843,46 @@ class CScreenItem extends CApiService {
 	/**
 	 * Check duplicates screen items in one cell.
 	 *
-	 * @throws APIException
+	 * @param array  $screen_items
+	 * @param string $screen_items['screenid']
+	 * @param int    $screen_items['x']
+	 * @param int    $screen_items['y']
+	 * @param int    $screen_items['colspan']
+	 * @param int    $screen_items['rowspan']
+	 * @param array  $screens
+	 * @param string $screens['screenid']
+	 * @param string $screens['name']
 	 *
-	 * @param array $screenItems
-	 * @param array $dbScreenItems
-	 * @param array $dbScreens
+	 * @throws APIException if input is invalid.
 	 */
-	protected function checkDuplicateResourceInCell(array $screenItems, array $dbScreenItems, array $dbScreens) {
-		foreach ($screenItems as &$screenItem) {
-			if (!isset($screenItem['x'])) {
-				$screenItem['x'] = isset($screenItem['screenitemid'])
-					? $dbScreenItems[$screenItem['screenitemid']]['x']
-					: 0;
+	protected function checkDuplicateResourceInCell(array $screen_items, array $screens) {
+		$calculated = [];
+
+		foreach ($screen_items as $screen_item) {
+			$screenid = $screen_item['screenid'];
+			$checked = [
+				'left' => $screen_item['x'],
+				'top' => $screen_item['y'],
+				'right' => $screen_item['x'] + $screen_item['colspan'] - 1,
+				'bottom' => $screen_item['y'] + $screen_item['rowspan'] - 1
+			];
+			if (!array_key_exists($screenid, $calculated)) {
+				$calculated[$screenid] = [];
 			}
-			if (!isset($screenItem['y'])) {
-				$screenItem['y'] = isset($screenItem['screenitemid'])
-					? $dbScreenItems[$screenItem['screenitemid']]['y']
-					: 0;
-			}
-		}
-		unset($screenItem);
 
-		foreach ($screenItems as $key => $screenItem) {
-			// check between input and input
-			foreach ($screenItems as $key2 => $screenItem2) {
-				if ($key == $key2) {
-					continue;
-				}
-
-				if ($screenItem['x'] == $screenItem2['x'] &&
-					$screenItem['y'] == $screenItem2['y'] &&
-					$screenItem['screenid'] == $screenItem2['screenid']
-				) {
-					$screenId = isset($screenItem['screenitemid'])
-						? $dbScreenItems[$screenItem['screenitemid']]['screenid']
-						: $screenItem['screenid'];
-
-					self::exception(
-						ZBX_API_ERROR_PARAMETERS,
-						_s(
-							'Screen "%1$s" cell X - %2$s Y - %3$s is already taken.',
-							$dbScreens[$screenId]['name'],
-							$screenItem['x'],
-							$screenItem['y']
+			foreach ($calculated[$screenid] as $entry) {
+				$overlaps_x = ($checked['left'] <= $entry['right'] && $checked['right'] >= $entry['left']);
+				$overlaps_y = ($checked['top'] <= $entry['bottom'] && $checked['bottom'] >= $entry['top']);
+				if ($overlaps_x && $overlaps_y) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Screen "%1$s" cell X - %2$s Y - %3$s is already taken.', $screens[$screenid]['name'],
+							$checked['left'], $checked['top']
 						)
 					);
 				}
 			}
 
-			// check between input and db
-			foreach ($dbScreenItems as $dbScreenItem) {
-				if (isset($screenItem['screenitemid'])
-						&& bccomp($screenItem['screenitemid'], $dbScreenItem['screenitemid']) == 0) {
-					continue;
-				}
-
-				if ($screenItem['x'] == $dbScreenItem['x'] && $screenItem['y'] == $dbScreenItem['y']) {
-					$screenId = isset($screenItem['screenitemid'])
-						? $dbScreenItems[$screenItem['screenitemid']]['screenid']
-						: $screenItem['screenid'];
-
-					self::exception(
-						ZBX_API_ERROR_PARAMETERS,
-						_s(
-							'Screen "%1$s" cell X - %2$s Y - %3$s is already taken.',
-							$dbScreens[$screenId]['name'],
-							$screenItem['x'],
-							$screenItem['y']
-						)
-					);
-				}
-			}
+			$calculated[$screenid][] = $checked;
 		}
 	}
 
@@ -1034,4 +943,20 @@ class CScreenItem extends CApiService {
 	protected function isValidMaxColumns($maxColumns) {
 		return ($maxColumns >= SCREEN_SURROGATE_MAX_COLUMNS_MIN && $maxColumns <= SCREEN_SURROGATE_MAX_COLUMNS_MAX);
 	}
+
+	/**
+	 * Validates URL fields for submitted screen items.
+	 *
+	 * @throws APIException for invalid URL
+	 *
+	 * @param array $screen_items	Array of screen items.
+	 */
+	protected function validateItemsURL($screen_items) {
+		foreach ($screen_items as $screen_item) {
+			if ($screen_item['resourcetype'] == SCREEN_RESOURCE_URL && array_key_exists('url', $screen_item)
+					&& !CHtmlUrlValidator::validate($screen_item['url'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong value for url field.'));
+			}
+		}
+	}
 }
