@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,6 +58,7 @@ class CConfigurationExport {
 			'screens' => [],
 			'images' => [],
 			'maps' => [],
+			'mediaTypes' => [],
 			'valueMaps' => []
 		];
 
@@ -74,39 +75,34 @@ class CConfigurationExport {
 			'screens' => [],
 			'images' => [],
 			'maps' => [],
+			'mediaTypes' => [],
 			'valueMaps' => []
 		];
 
 		$this->dataFields = [
-			'item' => ['hostid', 'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends',
-				'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname',
-				'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-				'snmpv3_privpassphrase', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password',
-				'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'inventory_link', 'flags',
+			'item' => ['hostid', 'type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+				'value_type', 'trapper_hosts', 'units', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+				'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'inventory_link', 'flags',
 				'logtimefmt', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
 				'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
 				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer',
 				'verify_host', 'allow_traps'
 			],
-			'drule' => ['itemid', 'hostid', 'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history',
-				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname',
-				'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-				'snmpv3_privpassphrase', 'formula', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
-				'password', 'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'inventory_link', 'flags',
-				'filter', 'lifetime', 'jmx_endpoint', 'timeout', 'url', 'query_fields', 'posts', 'status_codes',
-				'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode', 'request_method',
-				'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host',
-				'allow_traps'
+			'drule' => ['itemid', 'hostid', 'type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+				'value_type', 'trapper_hosts', 'units', 'formula', 'valuemapid', 'params', 'ipmi_sensor', 'authtype',
+				'username', 'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'inventory_link',
+				'flags', 'filter', 'lifetime', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields',
+				'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
+				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer',
+				'verify_host', 'allow_traps'
 			],
-			'item_prototype' => ['hostid', 'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history',
-				'trends', 'status', 'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname',
-				'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-				'snmpv3_privpassphrase', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password',
-				'publickey', 'privatekey', 'interfaceid', 'port', 'description', 'inventory_link', 'flags',
+			'item_prototype' => ['hostid', 'type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+				'value_type', 'trapper_hosts', 'units', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+				'password', 'publickey', 'privatekey', 'interfaceid', 'description', 'inventory_link', 'flags',
 				'logtimefmt', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
 				'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
 				'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer',
-				'verify_host', 'allow_traps'
+				'verify_host', 'allow_traps', 'discover'
 			]
 		];
 	}
@@ -140,24 +136,33 @@ class CConfigurationExport {
 		try {
 			$this->gatherData();
 
+			$schema = (new CImportValidatorFactory('xml'))
+				->getObject(ZABBIX_EXPORT_VERSION)
+				->getSchema();
+
+			$simple_triggers = [];
+			if ($this->data['triggers']) {
+				$simple_triggers = $this->builder->extractSimpleTriggers($this->data['triggers']);
+			}
+
 			if ($this->data['groups']) {
-				$this->builder->buildGroups($this->data['groups']);
+				$this->builder->buildGroups($schema['rules']['groups'], $this->data['groups']);
 			}
 
 			if ($this->data['templates']) {
-				$this->builder->buildTemplates($this->data['templates']);
+				$this->builder->buildTemplates($schema['rules']['templates'], $this->data['templates'], $simple_triggers);
 			}
 
 			if ($this->data['hosts']) {
-				$this->builder->buildHosts($this->data['hosts']);
+				$this->builder->buildHosts($schema['rules']['hosts'], $this->data['hosts'], $simple_triggers);
 			}
 
 			if ($this->data['triggers']) {
-				$this->builder->buildTriggers($this->data['triggers']);
+				$this->builder->buildTriggers($schema['rules']['triggers'], $this->data['triggers']);
 			}
 
 			if ($this->data['graphs']) {
-				$this->builder->buildGraphs($this->data['graphs']);
+				$this->builder->buildGraphs($schema['rules']['graphs'], $this->data['graphs']);
 			}
 
 			if ($this->data['screens']) {
@@ -172,8 +177,12 @@ class CConfigurationExport {
 				$this->builder->buildMaps($this->data['maps']);
 			}
 
+			if ($this->data['mediaTypes']) {
+				$this->builder->buildMediaTypes($schema['rules']['media_types'], $this->data['mediaTypes']);
+			}
+
 			if ($this->data['valueMaps']) {
-				$this->builder->buildValueMaps($this->data['valueMaps']);
+				$this->builder->buildValueMaps($schema['rules']['value_maps'], $this->data['valueMaps']);
 			}
 
 			return $this->writer->write($this->builder->getExport());
@@ -218,6 +227,10 @@ class CConfigurationExport {
 		if ($options['maps']) {
 			$this->gatherMaps($options['maps']);
 		}
+
+		if ($options['mediaTypes']) {
+			$this->gatherMediaTypes($options['mediaTypes']);
+		}
 	}
 
 	/**
@@ -258,15 +271,16 @@ class CConfigurationExport {
 	/**
 	 * Get templates for export from database.
 	 *
-	 * @param array $templateIds
+	 * @param array $templateids
 	 */
-	protected function gatherTemplates(array $templateIds) {
+	protected function gatherTemplates(array $templateids) {
 		$templates = API::Template()->get([
-			'templateids' => $templateIds,
 			'output' => ['host', 'name', 'description'],
-			'selectMacros' => API_OUTPUT_EXTEND,
 			'selectGroups' => ['groupid', 'name'],
 			'selectParentTemplates' => API_OUTPUT_EXTEND,
+			'selectMacros' => API_OUTPUT_EXTEND,
+			'selectTags' => ['tag', 'value'],
+			'templateids' => $templateids,
 			'preservekeys' => true
 		]);
 
@@ -303,13 +317,14 @@ class CConfigurationExport {
 			'output' => [
 				'proxy_hostid', 'host', 'status', 'ipmi_authtype', 'ipmi_privilege', 'ipmi_username', 'ipmi_password',
 				'name', 'description', 'tls_connect', 'tls_accept', 'tls_issuer', 'tls_subject', 'tls_psk_identity',
-				'tls_psk'
+				'tls_psk', 'inventory_mode'
 			],
-			'selectInterfaces' => ['interfaceid', 'main', 'type', 'useip', 'ip', 'dns', 'port', 'bulk'],
-			'selectInventory' => true,
+			'selectInterfaces' => API_OUTPUT_EXTEND,
+			'selectInventory' => API_OUTPUT_EXTEND,
 			'selectMacros' => API_OUTPUT_EXTEND,
 			'selectGroups' => ['groupid', 'name'],
 			'selectParentTemplates' => API_OUTPUT_EXTEND,
+			'selectTags' => ['tag', 'value'],
 			'hostids' => $hostIds,
 			'preservekeys' => true
 		]);
@@ -431,7 +446,7 @@ class CConfigurationExport {
 		$items = API::Item()->get([
 			'output' => $this->dataFields['item'],
 			'selectApplications' => ['name', 'flags'],
-			'selectPreprocessing' => ['type', 'params'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 			'hostids' => array_keys($hosts),
 			'inherited' => false,
 			'webitems' => true,
@@ -461,6 +476,7 @@ class CConfigurationExport {
 		$items = $this->prepareItems($items);
 
 		foreach ($items as $item) {
+			$item['host'] = $hosts[$item['hostid']]['host'];
 			$hosts[$item['hostid']]['items'][] = $item;
 		}
 
@@ -537,14 +553,37 @@ class CConfigurationExport {
 		$discovery_rules = API::DiscoveryRule()->get([
 			'output' => $this->dataFields['drule'],
 			'selectFilter' => ['evaltype', 'formula', 'conditions'],
+			'selectLLDMacroPaths' => ['lld_macro', 'path'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
+			'selectOverrides' => ['name', 'step', 'stop', 'filter', 'operations'],
 			'hostids' => array_keys($hosts),
 			'inherited' => false,
 			'preservekeys' => true
 		]);
 
+		$itemids = [];
+		foreach ($hosts as $host_data) {
+			foreach ($host_data['items'] as $item) {
+				$itemids[$item['itemid']] = $item['key_'];
+			}
+		};
+
 		$discovery_rules = $this->prepareDiscoveryRules($discovery_rules);
 
 		foreach ($discovery_rules as $discovery_rule) {
+			if ($discovery_rule['type'] == ITEM_TYPE_DEPENDENT) {
+				if (!array_key_exists($discovery_rule['master_itemid'], $itemids)) {
+					// Do not export dependent discovery rule with master item from template.
+					continue;
+				}
+
+				$discovery_rule['master_item'] = ['key_' => $itemids[$discovery_rule['master_itemid']]];
+			}
+
+			foreach ($discovery_rule['itemPrototypes'] as $itemid => $item_prototype) {
+				$discovery_rule['itemPrototypes'][$itemid]['host'] = $hosts[$discovery_rule['hostid']]['host'];
+			}
+
 			$hosts[$discovery_rule['hostid']]['discoveryRules'][] = $discovery_rule;
 		}
 
@@ -559,27 +598,113 @@ class CConfigurationExport {
 	 * @return array
 	 */
 	protected function prepareDiscoveryRules(array $items) {
+		$templateids = [];
+
 		foreach ($items as &$item) {
 			$item['itemPrototypes'] = [];
 			$item['graphPrototypes'] = [];
 			$item['triggerPrototypes'] = [];
 			$item['hostPrototypes'] = [];
 
-			// unset unnecessary condition fields
+			// Unset unnecessary condition fields.
 			foreach ($item['filter']['conditions'] as &$condition) {
 				unset($condition['item_conditionid'], $condition['itemid']);
 			}
 			unset($condition);
+
+			// Unset unnecessary filter field and prepare the operations.
+			if ($item['overrides']) {
+				foreach ($item['overrides'] as &$override) {
+					if (array_key_exists('filter', $override)) {
+						if (!$override['filter']['conditions']) {
+							unset($override['filter']);
+						}
+						unset($override['filter']['eval_formula']);
+					}
+
+					foreach ($override['operations'] as &$operation) {
+						if (array_key_exists('opstatus', $operation)) {
+							$operation['status'] = $operation['opstatus']['status'];
+							unset($operation['opstatus']);
+						}
+						if (array_key_exists('opdiscover', $operation)) {
+							$operation['discover'] = $operation['opdiscover']['discover'];
+							unset($operation['opdiscover']);
+						}
+						if (array_key_exists('opperiod', $operation)) {
+							$operation['delay'] = $operation['opperiod']['delay'];
+							unset($operation['opperiod']);
+						}
+						if (array_key_exists('ophistory', $operation)) {
+							$operation['history'] = $operation['ophistory']['history'];
+							unset($operation['ophistory']);
+						}
+						if (array_key_exists('optrends', $operation)) {
+							$operation['trends'] = $operation['optrends']['trends'];
+							unset($operation['optrends']);
+						}
+						if (array_key_exists('opseverity', $operation)) {
+							$operation['severity'] = $operation['opseverity']['severity'];
+							unset($operation['opseverity']);
+						}
+						if (array_key_exists('optag', $operation)) {
+							$operation['tags'] = [];
+							foreach ($operation['optag'] as $tag) {
+								$operation['tags'][] = $tag;
+							}
+							unset($operation['optag']);
+						}
+						if (array_key_exists('optemplate', $operation)) {
+							foreach ($operation['optemplate'] as $template) {
+								$templateids[$template['templateid']] = true;
+							}
+						}
+						if (array_key_exists('opinventory', $operation)) {
+							$operation['inventory_mode'] = $operation['opinventory']['inventory_mode'];
+							unset($operation['opinventory']);
+						}
+					}
+					unset($operation);
+				}
+				unset($override);
+
+			}
 		}
 		unset($item);
 
+		if ($templateids) {
+			$templates = API::Template()->get([
+				'output' => ['name'],
+				'templateids' => array_keys($templateids),
+				'preservekeys' => true
+			]);
+			foreach ($items as &$item) {
+				if ($item['overrides']) {
+					foreach ($item['overrides'] as &$override) {
+						foreach ($override['operations'] as &$operation) {
+							if (array_key_exists('optemplate', $operation)) {
+								$operation['templates'] = [];
+								foreach ($operation['optemplate'] as $template) {
+									$operation['templates'][] = ['name' => $templates[$template['templateid']]['name']];
+								}
+								unset($operation['optemplate']);
+							}
+						}
+						unset($operation);
+					}
+					unset($override);
+				}
+			}
+			unset($item);
+		}
+
 		// gather item prototypes
 		$item_prototypes = API::ItemPrototype()->get([
 			'output' => $this->dataFields['item_prototype'],
 			'selectApplications' => ['name'],
 			'selectApplicationPrototypes' => ['name'],
 			'selectDiscoveryRule' => ['itemid'],
-			'selectPreprocessing' => ['type', 'params'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 			'discoveryids' => zbx_objectValues($items, 'itemid'),
 			'inherited' => false,
 			'preservekeys' => true
@@ -677,8 +802,9 @@ class CConfigurationExport {
 
 		// gather trigger prototypes
 		$triggers = API::TriggerPrototype()->get([
-			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'recovery_mode',
-				'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close'
+			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'flags',
+				'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata',
+				'discover'
 			],
 			'selectDiscoveryRule' => API_OUTPUT_EXTEND,
 			'selectDependencies' => ['expression', 'description', 'recovery_expression'],
@@ -703,6 +829,7 @@ class CConfigurationExport {
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
 			'selectDiscoveryRule' => API_OUTPUT_EXTEND,
 			'selectTemplates' => API_OUTPUT_EXTEND,
+			'selectMacros' => API_OUTPUT_EXTEND,
 			'inherited' => false,
 			'preservekeys' => true
 		]);
@@ -792,7 +919,7 @@ class CConfigurationExport {
 			$httptest['application'] =
 				($httptest['applicationid'] != 0 && array_key_exists($httptest['applicationid'], $db_applications))
 					? ['name' => $db_applications[$httptest['applicationid']]['name']]
-					: null;
+					: [];
 			unset($httptest['applicationid']);
 		}
 		unset($httptest);
@@ -942,11 +1069,11 @@ class CConfigurationExport {
 		$hostIds = array_merge($hostIds, $templateIds);
 
 		$triggers = API::Trigger()->get([
-			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'recovery_mode',
-				'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close'
+			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type', 'flags',
+				'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata'
 			],
 			'selectDependencies' => ['expression', 'description', 'recovery_expression'],
-			'selectItems' => ['itemid', 'flags', 'type'],
+			'selectItems' => ['itemid', 'flags', 'type', 'templateid'],
 			'selectTags' => ['tag', 'value'],
 			'hostids' => $hostIds,
 			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
@@ -1057,6 +1184,34 @@ class CConfigurationExport {
 		$this->data['images'] = $images;
 	}
 
+	/**
+	 * Get media types for export builder from database.
+	 *
+	 * @param array $mediatypeids
+	 *
+	 * return array
+	 */
+	protected function gatherMediaTypes(array $mediatypeids) {
+		$this->data['mediaTypes'] = API::MediaType()->get([
+			'output' => ['name', 'type', 'smtp_server', 'smtp_port', 'smtp_helo', 'smtp_email', 'smtp_security',
+				'smtp_verify_peer', 'smtp_verify_host', 'smtp_authentication', 'username', 'passwd', 'content_type',
+				'exec_path', 'exec_params', 'gsm_modem', 'status', 'maxsessions', 'maxattempts', 'attempt_interval',
+				'script', 'timeout', 'process_tags', 'show_event_menu', 'event_menu_url', 'event_menu_name',
+				'description', 'parameters'
+			],
+			'selectMessageTemplates' => ['eventsource', 'recovery', 'subject', 'message'],
+			'mediatypeids' => $mediatypeids,
+			'preservekeys' => true
+		]);
+
+		foreach ($this->data['mediaTypes'] as &$media_type) {
+			if ($media_type['type'] == MEDIA_TYPE_WEBHOOK) {
+				CArrayHelper::sort($media_type['parameters'], ['name']);
+			}
+		}
+		unset($media_type);
+	}
+
 	/**
 	 * Get screens for export from database.
 	 *
@@ -1095,7 +1250,6 @@ class CConfigurationExport {
 	 * @param array $exportScreens
 	 */
 	protected function prepareScreenExport(array &$exportScreens) {
-		$screenIds = [];
 		$sysmapIds = [];
 		$groupIds = [];
 		$hostIds = [];
@@ -1134,10 +1288,6 @@ class CConfigurationExport {
 							$sysmapIds[$screenItem['resourceid']] = $screenItem['resourceid'];
 							break;
 
-						case SCREEN_RESOURCE_SCREEN:
-							$screenIds[$screenItem['resourceid']] = $screenItem['resourceid'];
-							break;
-
 						case SCREEN_RESOURCE_CLOCK:
 							if ($screenItem['style'] == TIME_TYPE_HOST) {
 								$itemIds[$screenItem['resourceid']] = $screenItem['resourceid'];
@@ -1148,7 +1298,6 @@ class CConfigurationExport {
 			}
 		}
 
-		$screens = $this->getScreensReferences($screenIds);
 		$sysmaps = $this->getMapsReferences($sysmapIds);
 		$groups = $this->getGroupsReferences($groupIds);
 		$hosts = $this->getHostsReferences($hostIds);
@@ -1188,10 +1337,6 @@ class CConfigurationExport {
 							$screenItem['resourceid'] = $sysmaps[$screenItem['resourceid']];
 							break;
 
-						case SCREEN_RESOURCE_SCREEN:
-							$screenItem['resourceid'] = $screens[$screenItem['resourceid']];
-							break;
-
 						case SCREEN_RESOURCE_CLOCK:
 							if ($screenItem['style'] == TIME_TYPE_HOST) {
 								$screenItem['resourceid'] = $items[$screenItem['resourceid']];
@@ -1370,34 +1515,6 @@ class CConfigurationExport {
 		return $ids;
 	}
 
-	/**
-	 * Get screens references by screen ids.
-	 *
-	 * @param array $screenIds
-	 *
-	 * @return array
-	 */
-	protected function getScreensReferences(array $screenIds) {
-		$ids = [];
-
-		$screens = API::Screen()->get([
-			'screenids' => $screenIds,
-			'output' => API_OUTPUT_EXTEND,
-			'preservekeys' => true
-		]);
-
-		// Access denied for some objects?
-		if (count($screens) != count($screenIds)) {
-			throw new CConfigurationExportException();
-		}
-
-		foreach ($screens as $id => $screen) {
-			$ids[$id] = ['name' => $screen['name']];
-		}
-
-		return $ids;
-	}
-
 	/**
 	 * Get maps references by map ids.
 	 *
