@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -64,6 +64,15 @@ class CImportReferencer {
 	protected $httptestsRefs;
 	protected $httpstepsRefs;
 
+	/**
+	 * Initializes references for items.
+	 */
+	public function initItemsReferences() {
+		if ($this->itemsRefs === null) {
+			$this->selectItems();
+		}
+	}
+
 	/**
 	 * Get group id by name.
 	 *
@@ -603,16 +612,6 @@ class CImportReferencer {
 		$this->templateScreens = array_unique(array_merge($this->templateScreens, $screens));
 	}
 
-	/**
-	 * Add screen name association with screen id.
-	 *
-	 * @param string $name
-	 * @param string $screenId
-	 */
-	public function addScreenRef($name, $screenId) {
-		$this->screensRefs[$name] = $screenId;
-	}
-
 	/**
 	 * Add template screen name association with template screen ID.
 	 *
@@ -863,7 +862,7 @@ class CImportReferencer {
 			$this->valueMapsRefs = [];
 
 			$valuemaps = API::ValueMap()->get([
-				'output' => ['valeumapid', 'name'],
+				'output' => ['valuemapid', 'name'],
 				'filter' => ['name' => $this->valueMaps]
 			]);
 
@@ -1102,7 +1101,7 @@ class CImportReferencer {
 					' FROM hosts h,host_discovery hd,items i'.
 					' WHERE h.hostid=hd.hostid'.
 						' AND hd.parent_itemid=i.itemid'.
-						' AND '.implode(' OR ', $sqlWhere)
+						' AND ('.implode(' OR ', $sqlWhere).')'
 				);
 				while ($data = DBfetch($query)) {
 					$this->hostPrototypesRefs[$data['parent_hostid']][$data['parent_itemid']][$data['host']] = $data['hostid'];
@@ -1177,7 +1176,7 @@ class CImportReferencer {
 					'SELECT ht.hostid,hs.httptestid,hs.name,hs.httpstepid'.
 					' FROM httptest ht,httpstep hs'.
 					' WHERE ht.httptestid=hs.httptestid'.
-						' AND '.implode(' OR ', $sql_where)
+						' AND ('.implode(' OR ', $sql_where).')'
 				);
 				while ($db_httpstep = DBfetch($db_httpsteps)) {
 					$this->httpstepsRefs[$db_httpstep['hostid']][$db_httpstep['httptestid']][$db_httpstep['name']] =
