@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -316,6 +316,10 @@ class CHelpItems {
 					'key' => 'vfs.fs.size[fs,<mode>]',
 					'description' => _('Disk space in bytes or in percentage from total. Returns integer for bytes; float for percentage')
 				],
+				[
+					'key' => 'vfs.fs.get',
+					'description' => _('File systems and their spaces in bytes and percentage from total. Returns JSON')
+				],
 				[
 					'key' => 'vm.memory.size[<mode>]',
 					'description' => _('Memory size in bytes or in percentage from total. Returns integer for bytes; float for percentage')
@@ -340,6 +344,10 @@ class CHelpItems {
 					'key' => 'wmi.get[<namespace>,<query>]',
 					'description' => _('Execute WMI query and return the first selected object. Returns integer, float, string or text (depending on the request)')
 				],
+				[
+					'key' => 'wmi.getall[<namespace>,<query>]',
+					'description' => _('Execute WMI query and return the JSON document with all selected objects')
+				],
 				[
 					'key' => 'zabbix.stats[<ip>,<port>]',
 					'description' => _('Returns a JSON object containing Zabbix server or proxy internal metrics.')
@@ -375,11 +383,11 @@ class CHelpItems {
 					'description' => _('Maximum number of processes supported by OS. Returns integer')
 				],
 				[
-					'key' => 'log[file,<regexp>,<encoding>,<maxlines>,<mode>,<output>,<maxdelay>]',
+					'key' => 'log[file,<regexp>,<encoding>,<maxlines>,<mode>,<output>,<maxdelay>,<options>]',
 					'description' => _('Log file monitoring. Returns log')
 				],
 				[
-					'key' => 'log.count[file,<regexp>,<encoding>,<maxproclines>,<mode>,<maxdelay>]',
+					'key' => 'log.count[file,<regexp>,<encoding>,<maxproclines>,<mode>,<maxdelay>,<options>]',
 					'description' => _('Count of matched lines in log file monitoring. Returns integer')
 				],
 				[
@@ -622,6 +630,10 @@ class CHelpItems {
 					'key' => 'vfs.fs.inode[fs,<mode>]',
 					'description' => _('Number or percentage of inodes. Returns integer for number; float for percentage')
 				],
+				[
+					'key' => 'vfs.fs.get',
+					'description' => _('File systems and their spaces in bytes and percentage from total. Returns JSON')
+				],
 				[
 					'key' => 'vfs.fs.size[fs,<mode>]',
 					'description' => _('Disk space in bytes or in percentage from total. Returns integer for bytes; float for percentage')
@@ -706,6 +718,10 @@ class CHelpItems {
 					'key' => 'vmware.fullname[<url>]',
 					'description' => _('VMware service full name, <url> - VMware service URL')
 				],
+				[
+					'key' => 'vmware.dc.discovery[<url>]',
+					'description' => _('VMware datacenters and their IDs. Returns JSON')
+				],
 				[
 					'key' => 'vmware.datastore.read[<url>,<datastore>,<mode>]',
 					'description' => _('VMware datastore read statistics, <url> - VMware service URL, <datastore> - datastore name, <mode> - latency/maxlatency - average or maximum')
@@ -968,7 +984,7 @@ class CHelpItems {
 				],
 				[
 					'key' => 'zabbix[host,discovery,interfaces]',
-					'description' => _('Returns a JSON object describing the host network interfaces configured in Zabbix. Can be used for LLD.')
+					'description' => _('Returns a JSON array describing the host network interfaces configured in Zabbix. Can be used for LLD.')
 				],
 				[
 					'key' => 'zabbix[host,<type>,available]',
@@ -996,7 +1012,7 @@ class CHelpItems {
 				],
 				[
 					'key' => 'zabbix[proxy,<name>,<param>]',
-					'description' => _('Time of proxy last access. Name - proxy name. Param - lastaccess. Unix timestamp.')
+					'description' => _('Time of proxy last access. Name - proxy name. Valid params are: lastaccess - Unix timestamp, delay - seconds.')
 				],
 				[
 					'key' => 'zabbix[proxy_history]',
@@ -1046,6 +1062,10 @@ class CHelpItems {
 					'key' => 'zabbix[vcache,cache,<parameter>]',
 					'description' => _('Value cache effectiveness. Valid parameters are: requests, hits and misses.')
 				],
+				[
+					'key' => 'zabbix[version]',
+					'description' => _('Version of Zabbix server or proxy')
+				],
 				[
 					'key' => 'zabbix[vmware,buffer,<mode>]',
 					'description' => _('VMware cache statistics. Valid modes are: total, free, pfree, used and pused.')
@@ -1057,12 +1077,16 @@ class CHelpItems {
 			],
 			ITEM_TYPE_DB_MONITOR => [
 				[
-					'key' => 'db.odbc.select[<unique short description>,dsn]',
+					'key' => 'db.odbc.select[<unique short description>,<dsn>,<connection string>]',
 					'description' => _('Return first column of the first row of the SQL query result.')
 				],
 				[
-					'key' => 'db.odbc.discovery[<unique short description>,dsn]',
-					'description' => _('Transform SQL query result into a JSON object for low-level discovery.')
+					'key' => 'db.odbc.discovery[<unique short description>,<dsn>,<connection string>]',
+					'description' => _('Transform SQL query result into a JSON array for low-level discovery.')
+				],
+				[
+					'key' => 'db.odbc.get[<unique short description>,<dsn>,<connection string>]',
+					'description' => _('Transform SQL query result into a JSON array.')
 				]
 			],
 			ITEM_TYPE_JMX => [
@@ -1072,7 +1096,17 @@ class CHelpItems {
 				],
 				[
 					'key' => 'jmx.discovery[<discovery mode>,<object name>]',
-					'description' => _('Return a JSON object describing the MBean objects or their attributes. Can be used for LLD.')
+					'description' => _('Return a JSON array with LLD macros describing the MBean objects or their attributes. Can be used for LLD.')
+				],
+				[
+					'key' => 'jmx.get[<discovery mode>,<object name>]',
+					'description' => _('Return a JSON array with MBean objects or their attributes. Compared to jmx.discovery it does not define LLD macros. Can be used for LLD.')
+				]
+			],
+			ITEM_TYPE_IPMI => [
+				[
+					'key' => 'ipmi.get',
+					'description' => _('IPMI sensor IDs and other sensor-related parameters. Returns JSON.')
 				]
 			]
 		];
