@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -56,66 +56,55 @@ class CScreenHttpTest extends CScreenBase {
 		$httptests = [];
 		$paging = [];
 
-		if ($this->data['hosts_selected']) {
-			$config = select_config();
+		$config = select_config();
 
-			$options = [
-				'output' => ['httptestid', 'name', 'hostid'],
-				'selectHosts' => ['name', 'status'],
-				'selectSteps' => API_OUTPUT_COUNT,
-				'templated' => false,
-				'preservekeys' => true,
-				'filter' => ['status' => HTTPTEST_STATUS_ACTIVE],
-				'limit' => $config['search_limit'] + 1
-			];
+		$options = [
+			'output' => ['httptestid', 'name', 'hostid'],
+			'selectHosts' => ['name', 'status'],
+			'selectSteps' => API_OUTPUT_COUNT,
+			'templated' => false,
+			'preservekeys' => true,
+			'filter' => ['status' => HTTPTEST_STATUS_ACTIVE],
+			'limit' => $config['search_limit'] + 1
+		];
 
-			if ($this->hostid != 0) {
-				$options['hostids'] = [$this->hostid];
-			}
-			elseif ($this->groupid) {
-				$options['groupids'] = zbx_toArray($this->groupid);
-			}
+		$options['hostids'] = $this->data['hostids'] ? $this->data['hostids'] : null;
+		$options['groupids'] = $this->data['groupids'] ? $this->data['groupids'] : null;
 
-			$httptests = API::HttpTest()->get($options);
+		$httptests = API::HttpTest()->get($options);
 
-			foreach ($httptests as &$httptest) {
-				$httptest['host'] = reset($httptest['hosts']);
-				$httptest['hostname'] = $httptest['host']['name'];
-				unset($httptest['hosts']);
-			}
-			unset($httptest);
-
-			order_result($httptests, $sort_field, $sort_order);
-
-			$url = (new CUrl('zabbix.php'))
-				->setArgument('action', 'web.view')
-				->setArgument('groupid', $this->data['groupid'])
-				->setArgument('hostid', $this->hostid)
-				->setArgument('fullscreen', $this->data['fullscreen']);
-
-			$paging = getPagingLine($httptests, $sort_order, $url);
-			$httptests = resolveHttpTestMacros($httptests, true, false);
-			order_result($httptests, $sort_field, $sort_order);
-
-			// Fetch the latest results of the web scenario.
-			$last_httptest_data = Manager::HttpTest()->getLastData(array_keys($httptests));
-
-			foreach ($httptests as &$httptest) {
-				if (array_key_exists($httptest['httptestid'], $last_httptest_data)) {
-					$httptest['lastcheck'] = $last_httptest_data[$httptest['httptestid']]['lastcheck'];
-					$httptest['lastfailedstep'] = $last_httptest_data[$httptest['httptestid']]['lastfailedstep'];
-					$httptest['error'] = $last_httptest_data[$httptest['httptestid']]['error'];
-				}
-			}
-			unset($httptest);
+		foreach ($httptests as &$httptest) {
+			$httptest['host'] = reset($httptest['hosts']);
+			$httptest['hostname'] = $httptest['host']['name'];
+			unset($httptest['hosts']);
 		}
+		unset($httptest);
+
+		order_result($httptests, $sort_field, $sort_order);
+
+		$paging = CPagerHelper::paginate($this->page, $httptests, $sort_order,
+			(new CUrl('zabbix.php'))->setArgument('action', 'web.view')
+		);
+
+		$httptests = resolveHttpTestMacros($httptests, true, false);
+		order_result($httptests, $sort_field, $sort_order);
+
+		// Fetch the latest results of the web scenario.
+		$last_httptest_data = Manager::HttpTest()->getLastData(array_keys($httptests));
+
+		foreach ($httptests as &$httptest) {
+			if (array_key_exists($httptest['httptestid'], $last_httptest_data)) {
+				$httptest['lastcheck'] = $last_httptest_data[$httptest['httptestid']]['lastcheck'];
+				$httptest['lastfailedstep'] = $last_httptest_data[$httptest['httptestid']]['lastfailedstep'];
+				$httptest['error'] = $last_httptest_data[$httptest['httptestid']]['error'];
+			}
+		}
+		unset($httptest);
 
 		// Create table.
 		$table = (new CTableInfo())
 			->setHeader([
-				$this->hostid == 0
-					? make_sorting_header(_('Host'), 'hostname', $sort_field, $sort_order, 'zabbix.php?action=web.view')
-					: null,
+				make_sorting_header(_('Host'), 'hostname', $sort_field, $sort_order, 'zabbix.php?action=web.view'),
 				make_sorting_header(_('Name'), 'name', $sort_field, $sort_order, 'zabbix.php?action=web.view'),
 				_('Number of steps'),
 				_('Last check'),
@@ -123,17 +112,6 @@ class CScreenHttpTest extends CScreenBase {
 			]);
 
 		foreach ($httptests as $httptest) {
-			if ($this->hostid == 0) {
-				$hostname = $httptest['hostname'];
-
-				if ($httptest['host']['status'] == HOST_STATUS_NOT_MONITORED) {
-					$hostname = (new CSpan($hostname))->addClass(ZBX_STYLE_RED);
-				}
-			}
-			else {
-				$hostname = null;
-			}
-
 			if (array_key_exists('lastfailedstep', $httptest) && $httptest['lastfailedstep'] !== null) {
 				$lastcheck = zbx_date2str(DATE_TIME_FORMAT_SECONDS, $httptest['lastcheck']);
 
@@ -162,7 +140,9 @@ class CScreenHttpTest extends CScreenBase {
 			}
 
 			$table->addRow(new CRow([
-				$hostname,
+				($httptest['host']['status'] == HOST_STATUS_NOT_MONITORED)
+					? (new CSpan($httptest['hostname']))->addClass(ZBX_STYLE_RED)
+					: $httptest['hostname'],
 				new CLink($httptest['name'], 'httpdetails.php?httptestid='.$httptest['httptestid']),
 				$httptest['steps'],
 				$lastcheck,
