@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -35,15 +35,25 @@ class CScreenSystemStatus extends CScreenBase {
 		$page['file'] = $this->pageFile;
 
 		$config = select_config();
-		$data = getSystemStatusData([], $config);
-		$table = makeSystemStatus([], $data, $config, $this->pageFile.'?screenid='.$this->screenid);
+		$severity_config = [
+			'severity_name_0' => $config['severity_name_0'],
+			'severity_name_1' => $config['severity_name_1'],
+			'severity_name_2' => $config['severity_name_2'],
+			'severity_name_3' => $config['severity_name_3'],
+			'severity_name_4' => $config['severity_name_4'],
+			'severity_name_5' => $config['severity_name_5']
+		];
+		$data = getSystemStatusData([]);
+		$table = makeSystemStatus([], $data, $severity_config, $this->pageFile.'?screenid='.$this->screenid);
 
 		$footer = (new CList())
-			->addItem(_s('Updated: %s', zbx_date2str(TIME_FORMAT_SECONDS)))
+			->addItem(_s('Updated: %1$s', zbx_date2str(TIME_FORMAT_SECONDS)))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_FOOT);
 
+		$script = new CScriptTag('monitoringScreen.refreshOnAcknowledgeCreate();');
+
 		return $this->getOutput(
-			(new CUiWidget('hat_syssum', [$table, $footer]))->setHeader(_('System status'))
+			(new CUiWidget('hat_syssum', [$table, $footer, $script]))->setHeader(_('Problems by severity'))
 		);
 	}
 }
