@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -183,7 +183,7 @@ class CWebUser {
 			'alias' => ZBX_GUEST_USER,
 			'userid' => 0,
 			'lang' => 'en_gb',
-			'type' => '0',
+			'type' => 0,
 			'debug_mode' => false
 		];
 	}
@@ -196,7 +196,7 @@ class CWebUser {
 	 * @return int
 	 */
 	public static function getType() {
-		return self::$data['type'];
+		return self::$data ? self::$data['type'] : 0;
 	}
 
 	/**
@@ -205,7 +205,7 @@ class CWebUser {
 	 * @return bool
 	 */
 	public static function getDebugMode() {
-		return (self::$data['debug_mode']);
+		return (self::$data && self::$data['debug_mode']);
 	}
 
 	/**
@@ -214,7 +214,7 @@ class CWebUser {
 	 * @return bool
 	 */
 	public static function isLoggedIn() {
-		return (self::$data['userid']);
+		return (self::$data && self::$data['userid']);
 	}
 
 	/**
@@ -223,7 +223,7 @@ class CWebUser {
 	 * @return bool
 	 */
 	public static function isGuest() {
-		return (self::$data['alias'] == ZBX_GUEST_USER);
+		return (self::$data && self::$data['alias'] == ZBX_GUEST_USER);
 	}
 
 	/**
@@ -258,4 +258,15 @@ class CWebUser {
 	public static function getLang() {
 		return (self::$data) ? substr(self::$data['lang'], 0, strpos(self::$data['lang'], '_')) : 'en';
 	}
+
+	/**
+	 * Get user ip address.
+	 *
+	 * @return string
+	 */
+	public static function getIp(): string {
+		return (array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER) && $_SERVER['HTTP_X_FORWARDED_FOR'] !== '')
+			? $_SERVER['HTTP_X_FORWARDED_FOR']
+			: $_SERVER['REMOTE_ADDR'];
+	}
 }
