@@ -33,19 +33,15 @@ function eventSource($source = null) {
 	$sources = [
 		EVENT_SOURCE_TRIGGERS => _('trigger'),
 		EVENT_SOURCE_DISCOVERY => _('discovery'),
-		EVENT_SOURCE_AUTO_REGISTRATION => _('auto registration'),
+		EVENT_SOURCE_AUTOREGISTRATION => _('autoregistration'),
 		EVENT_SOURCE_INTERNAL => _x('internal', 'event source')
 	];
 
 	if ($source === null) {
 		return $sources;
 	}
-	elseif (isset($sources[$source])) {
-		return $sources[$source];
-	}
-	else {
-		return _('Unknown');
-	}
+
+	return array_key_exists($source, $sources) ?  $sources[$source] : _('Unknown');
 }
 
 /**
@@ -63,7 +59,7 @@ function eventObject($object = null) {
 		EVENT_OBJECT_TRIGGER => _('trigger'),
 		EVENT_OBJECT_DHOST => _('discovered host'),
 		EVENT_OBJECT_DSERVICE => _('discovered service'),
-		EVENT_OBJECT_AUTOREGHOST => _('auto-registered host'),
+		EVENT_OBJECT_AUTOREGHOST => _('autoregistered host'),
 		EVENT_OBJECT_ITEM => _('item'),
 		EVENT_OBJECT_LLDRULE => _('low-level discovery rule')
 	];
@@ -89,7 +85,7 @@ function eventSourceObjects() {
 		['source' => EVENT_SOURCE_TRIGGERS, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DHOST],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DSERVICE],
-		['source' => EVENT_SOURCE_AUTO_REGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
+		['source' => EVENT_SOURCE_AUTOREGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_ITEM],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE]
@@ -139,30 +135,31 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 
 /**
  *
- * @param array  $event								An array of event data.
- * @param string $event['eventid']					Event ID.
- * @param string $event['correlationid']			OK Event correlation ID.
- * @param string $event['userid]					User ID who generated the OK event.
- * @param string $event['name']						Event name.
- * @param string $event['acknowledged']				State of acknowledgement.
- * @param string $backurl							A link back after acknowledgement has been clicked.
+ * @param array  $event                   An array of event data.
+ * @param string $event['eventid']        Event ID.
+ * @param string $event['objectid']       Object ID.
+ * @param string $event['correlationid']  OK Event correlation ID.
+ * @param string $event['userid']         User ID who generated the OK event.
+ * @param string $event['name']           Event name.
+ * @param string $event['acknowledged']   State of acknowledgement.
+ * @param CCOl   $event['opdata']         Operational data with expanded macros.
+ * @param string $event['comments']       Trigger description with expanded macros.
  *
  * @return CTableInfo
  */
-function make_event_details($event, $backurl) {
-	$event_update_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'acknowledge.edit')
-		->setArgument('eventids', [$event['eventid']])
-		->setArgument('backurl', $backurl)
-		->getUrl();
-
+function make_event_details(array $event) {
 	$config = select_config();
+	$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 	$table = (new CTableInfo())
 		->addRow([
 			_('Event'),
 			$event['name']
 		])
+		->addRow([
+			_('Operational data'),
+			$event['opdata']
+		])
 		->addRow([
 			_('Severity'),
 			getSeverityCell($event['severity'], $config)
@@ -173,9 +170,10 @@ function make_event_details($event, $backurl) {
 		])
 		->addRow([
 			_('Acknowledged'),
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $event_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+			(new CLink($is_acknowledged ? _('Yes') : _('No')))
+				->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
 				->addClass(ZBX_STYLE_LINK_ALT)
+				->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
 		]);
 
 	if ($event['r_eventid'] != 0) {
@@ -232,12 +230,14 @@ function make_event_details($event, $backurl) {
 
 	$tags = makeTags([$event]);
 
-	$table->addRow([_('Tags'), $tags[$event['eventid']]]);
+	$table
+		->addRow([_('Tags'), $tags[$event['eventid']]])
+		->addRow([_('Description'), (new CDiv(zbx_str2links($event['comments'])))]);
 
 	return $table;
 }
 
-function make_small_eventlist($startEvent, $backurl) {
+function make_small_eventlist(array $startEvent) {
 	$config = select_config();
 
 	$table = (new CTableInfo())
@@ -316,7 +316,7 @@ function make_small_eventlist($startEvent, $backurl) {
 		'preservekeys' => true
 	]);
 	$mediatypes = API::Mediatype()->get([
-		'output' => ['description', 'maxattempts'],
+		'output' => ['name', 'maxattempts'],
 		'mediatypeids' => array_keys($actions['mediatypeids']),
 		'preservekeys' => true
 	]);
@@ -346,20 +346,20 @@ function make_small_eventlist($startEvent, $backurl) {
 			$value_clock = $event['r_clock'];
 		}
 
+		$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 		$cell_status = new CSpan($value_str);
 
 		/*
 		 * Add colors to span depending on configuration and trigger parameters. No blinking added to status,
 		 * since the page is not on autorefresh.
 		 */
-		addTriggerValueStyle($cell_status, $value, $value_clock, $event['acknowledged'] == EVENT_ACKNOWLEDGED);
+		addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
-		// Create link to Problem update page.
-		$problem_update_url = (new CUrl('zabbix.php'))
-			->setArgument('action', 'acknowledge.edit')
-			->setArgument('eventids', [$event['eventid']])
-			->setArgument('backurl', $backurl)
-			->getUrl();
+		// Create acknowledge link.
+		$problem_update_link = (new CLink($is_acknowledged ? _('Yes') : _('No')))
+			->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+			->addClass(ZBX_STYLE_LINK_ALT)
+			->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);');
 
 		$table->addRow([
 			(new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
@@ -373,9 +373,7 @@ function make_small_eventlist($startEvent, $backurl) {
 			$cell_status,
 			zbx_date2age($event['clock']),
 			$duration,
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT),
+			$problem_update_link,
 			makeEventActionsIcons($event['eventid'], $actions['data'], $mediatypes, $users, $severity_config)
 		]);
 	}
@@ -388,10 +386,9 @@ function make_small_eventlist($startEvent, $backurl) {
  *
  * @param array  $trigger                    An array of trigger data.
  * @param string $trigger['triggerid']       Trigger ID to select events.
- * @param string $trigger['description']     Trigger description.
+ * @param string $trigger['comments']        Trigger description.
  * @param string $trigger['url']             Trigger URL.
  * @param string $eventid_till
- * @param string $backurl                    URL to return to.
  * @param bool   $show_timeline              Show time line flag.
  * @param int    $show_tags                  Show tags flag. Possible values:
  *                                             - PROBLEMS_SHOW_TAGS_NONE;
@@ -410,9 +407,8 @@ function make_small_eventlist($startEvent, $backurl) {
  *
  * @return CDiv
  */
-function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline = true,
-		$show_tags = PROBLEMS_SHOW_TAGS_3, array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL,
-		$tag_priority = '') {
+function make_popup_eventlist($trigger, $eventid_till, $show_timeline = true, $show_tags = PROBLEMS_SHOW_TAGS_3,
+		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
 	// Show trigger description and URL.
 	$div = new CDiv();
 
@@ -426,7 +422,7 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 	}
 
 	if ($trigger['url'] !== '') {
-		$trigger_url = CHtmlUrlValidator::validate($trigger['url'], false)
+		$trigger_url = CHtmlUrlValidator::validate($trigger['url'], ['allow_user_macro' => false])
 			? $trigger['url']
 			: 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.', zbx_jsvalue($trigger['url'], false, false)).
 				'\');';
@@ -563,10 +559,11 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				$cell_r_clock = '';
 			}
 
+			$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 			$cell_status = new CSpan($value_str);
 
 			// add colors and blinking to span depending on configuration and trigger parameters
-			addTriggerValueStyle($cell_status, $value, $value_clock, $problem['acknowledged'] == EVENT_ACKNOWLEDGED);
+			addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
 			if ($show_timeline) {
 				if ($last_clock != 0) {
@@ -591,16 +588,10 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 			}
 
 			// Create acknowledge link.
-			$problem_update_url = (new CUrl('zabbix.php'))
-				->setArgument('action', 'acknowledge.edit')
-				->setArgument('eventids', [$problem['eventid']])
-				->setArgument('backurl', $backurl)
-				->getUrl();
-
-			$acknowledged = $problem['acknowledged'] == EVENT_ACKNOWLEDGED;
-			$problem_update_link = (new CLink($acknowledged ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT);
+			$problem_update_link = (new CLink($is_acknowledged ? _('Yes') : _('No')))
+				->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+				->addClass(ZBX_STYLE_LINK_ALT)
+				->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);');
 
 			$table->addRow(array_merge($row, [
 				$cell_r_clock,
@@ -683,24 +674,26 @@ function orderEventTagsByPriority(array $event_tags, array $priorities) {
  * Create element with tags.
  *
  * @param array  $list
- * @param string $list[]['eventid'|'triggerid']
+ * @param string $list[][$key]
  * @param array  $list[]['tags']
  * @param string $list[]['tags'][]['tag']
  * @param string $list[]['tags'][]['value']
  * @param bool   $html
- * @param string $key                            Name of tags source ID. Possible values:
- *                                                - 'eventid' - for events and problems (default);
- *                                                - 'triggerid' - for triggers.
- * @param int    $list_tag_count                 Maximum number of tags to display.
- * @param array  $filter_tags                    An array of tag filtering data.
+ * @param string $key                        Name of tag source ID. Possible values:
+ *                                            - 'eventid' - for events and problems (default);
+ *                                            - 'hostid' - for hosts;
+ *                                            - 'templateid' - for templates;
+ *                                            - 'triggerid' - for triggers.
+ * @param int    $list_tag_count             Maximum number of tags to display.
+ * @param array  $filter_tags                An array of tag filtering data.
  * @param string $filter_tags[]['tag']
  * @param int    $filter_tags[]['operator']
  * @param string $filter_tags[]['value']
- * @param int    $tag_name_format                Tag name format. Possible values:
- *                                                - PROBLEMS_TAG_NAME_FULL (default);
- *                                                - PROBLEMS_TAG_NAME_SHORTENED;
- *                                                - PROBLEMS_TAG_NAME_NONE.
- * @param string $tag_priority                   A list of comma-separated tag names.
+ * @param int    $tag_name_format            Tag name format. Possible values:
+ *                                            - PROBLEMS_TAG_NAME_FULL (default);
+ *                                            - PROBLEMS_TAG_NAME_SHORTENED;
+ *                                            - PROBLEMS_TAG_NAME_NONE.
+ * @param string $tag_priority               A list of comma-separated tag names.
  *
  * @return array
  */
@@ -710,6 +703,7 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 
 	if ($html) {
 		// Convert $filter_tags to a more usable format.
+
 		$f_tags = [];
 
 		foreach ($filter_tags as $tag) {
@@ -726,10 +720,14 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 	}
 
 	foreach ($list as $element) {
-		CArrayHelper::sort($element['tags'], ['tag', 'value']);
-
 		$tags[$element[$key]] = [];
 
+		if (!$element['tags']) {
+			continue;
+		}
+
+		CArrayHelper::sort($element['tags'], ['tag', 'value']);
+
 		if ($html) {
 			// Show first n tags and "..." with hint box if there are more.
 
@@ -748,7 +746,9 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 					$tags[$element[$key]][] = (new CSpan($value))
 						->addClass(ZBX_STYLE_TAG)
 						->setHint(getTagString($tag));
+
 					$tags_shown++;
+
 					if ($tags_shown >= $list_tag_count) {
 						break;
 					}
@@ -771,7 +771,7 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 					(new CButton(null))
 						->addClass(ZBX_STYLE_ICON_WZRD_ACTION)
 						->setHint(new CDiv($hint_content), '', true, 'max-width: 500px')
-					))->addClass(ZBX_STYLE_REL_CONTAINER);
+				))->addClass(ZBX_STYLE_REL_CONTAINER);
 			}
 		}
 		else {
