@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -75,9 +75,10 @@ function get_item_logtype_style($logtype) {
 }
 
 /**
- * Get item type string name by item type number, or array of all item types if null passed
+ * Get item type string name by item type number, or array of all item types if null passed.
  *
  * @param int|null $type
+ *
  * @return array|string
  */
 function item_type2str($type = null) {
@@ -85,9 +86,7 @@ function item_type2str($type = null) {
 		ITEM_TYPE_ZABBIX => _('Zabbix agent'),
 		ITEM_TYPE_ZABBIX_ACTIVE => _('Zabbix agent (active)'),
 		ITEM_TYPE_SIMPLE => _('Simple check'),
-		ITEM_TYPE_SNMPV1 => _('SNMPv1 agent'),
-		ITEM_TYPE_SNMPV2C => _('SNMPv2 agent'),
-		ITEM_TYPE_SNMPV3 => _('SNMPv3 agent'),
+		ITEM_TYPE_SNMP => _('SNMP agent'),
 		ITEM_TYPE_SNMPTRAP => _('SNMP trap'),
 		ITEM_TYPE_INTERNAL => _('Zabbix internal'),
 		ITEM_TYPE_TRAPPER => _('Zabbix trapper'),
@@ -103,15 +102,12 @@ function item_type2str($type = null) {
 		ITEM_TYPE_HTTPTEST => _('Web monitoring'),
 		ITEM_TYPE_DEPENDENT => _('Dependent item')
 	];
-	if (is_null($type)) {
+
+	if ($type === null) {
 		return $types;
 	}
-	elseif (isset($types[$type])) {
-		return $types[$type];
-	}
-	else {
-		return _('Unknown');
-	}
+
+	return array_key_exists($type, $types) ? $types[$type] : _('Unknown');
 }
 
 /**
@@ -357,9 +353,7 @@ function interfaceType2str($type) {
 
 function itemTypeInterface($type = null) {
 	$types = [
-		ITEM_TYPE_SNMPV1 => INTERFACE_TYPE_SNMP,
-		ITEM_TYPE_SNMPV2C => INTERFACE_TYPE_SNMP,
-		ITEM_TYPE_SNMPV3 => INTERFACE_TYPE_SNMP,
+		ITEM_TYPE_SNMP =>  INTERFACE_TYPE_SNMP,
 		ITEM_TYPE_SNMPTRAP => INTERFACE_TYPE_SNMP,
 		ITEM_TYPE_IPMI => INTERFACE_TYPE_IPMI,
 		ITEM_TYPE_ZABBIX => INTERFACE_TYPE_AGENT,
@@ -391,17 +385,15 @@ function itemTypeInterface($type = null) {
  */
 function copyItemsToHosts($src_itemids, $dst_hostids) {
 	$items = API::Item()->get([
-		'output' => ['type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
-			'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname', 'snmpv3_securitylevel',
-			'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol', 'snmpv3_privpassphrase',
-			'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey',
-			'privatekey', 'flags', 'port', 'description', 'inventory_link', 'jmx_endpoint', 'master_itemid', 'timeout',
-			'url', 'query_fields', 'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers',
-			'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password',
-			'verify_peer', 'verify_host', 'allow_traps'
+		'output' => ['type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status', 'value_type',
+			'trapper_hosts', 'units', 'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+			'password', 'publickey', 'privatekey', 'flags', 'description', 'inventory_link', 'jmx_endpoint',
+			'master_itemid', 'timeout', 'url', 'query_fields', 'posts', 'status_codes', 'follow_redirects',
+			'post_type', 'http_proxy', 'headers', 'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file',
+			'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'allow_traps'
 		],
 		'selectApplications' => ['applicationid'],
-		'selectPreprocessing' => ['type', 'params'],
+		'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 		'itemids' => $src_itemids,
 		'preservekeys' => true
 	]);
@@ -539,7 +531,7 @@ function copyItemsToHosts($src_itemids, $dst_hostids) {
 
 					// Master item does not exist on destination host or has not been selected for copying.
 					if (!$item_found) {
-						error(_s('Item "%1$s" has master item and cannot be copied.', $item['name']));
+						error(_s('Item "%1$s" cannot be copied without its master item.', $item['name']));
 
 						return false;
 					}
@@ -562,17 +554,16 @@ function copyItemsToHosts($src_itemids, $dst_hostids) {
 
 function copyItems($srcHostId, $dstHostId) {
 	$srcItems = API::Item()->get([
-		'output' => ['type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
-			'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname', 'snmpv3_securitylevel',
-			'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol', 'snmpv3_privpassphrase',
-			'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey',
-			'privatekey', 'flags', 'port', 'description', 'inventory_link', 'jmx_endpoint', 'master_itemid',
-			'templateid', 'url', 'query_fields', 'timeout', 'posts', 'status_codes', 'follow_redirects', 'post_type',
-			'http_proxy', 'headers', 'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file',
-			'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'allow_traps'
+		'output' => ['type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status', 'value_type',
+			'trapper_hosts', 'units', 'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+			'password', 'publickey', 'privatekey', 'flags', 'description', 'inventory_link', 'jmx_endpoint',
+			'master_itemid', 'templateid', 'url', 'query_fields', 'timeout', 'posts', 'status_codes',
+			'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode', 'request_method',
+			'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host',
+			'allow_traps'
 		],
 		'selectApplications' => ['applicationid'],
-		'selectPreprocessing' => ['type', 'params'],
+		'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 		'hostids' => $srcHostId,
 		'webitems' => true,
 		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
@@ -927,7 +918,9 @@ function makeItemTemplatePrefix($itemid, array $parent_templates, $flag) {
 
 	if ($template['permission'] == PERM_READ_WRITE) {
 		if ($flag == ZBX_FLAG_DISCOVERY_RULE) {
-			$url = (new CUrl('host_discovery.php'))->setArgument('hostid', $template['hostid']);
+			$url = (new CUrl('host_discovery.php'))
+				->setArgument('filter_set', '1')
+				->setArgument('filter_hostids', [$template['hostid']]);
 		}
 		elseif ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 			$url = (new CUrl('disc_prototypes.php'))
@@ -936,8 +929,8 @@ function makeItemTemplatePrefix($itemid, array $parent_templates, $flag) {
 		// ZBX_FLAG_DISCOVERY_NORMAL
 		else {
 			$url = (new CUrl('items.php'))
-				->setArgument('hostid', $template['hostid'])
-				->setArgument('filter_set', 1);
+				->setArgument('filter_set', '1')
+				->setArgument('filter_hostids', [$template['hostid']]);
 		}
 
 		$name = (new CLink(CHtml::encode($template['name']), $url))->addClass(ZBX_STYLE_LINK_ALT);
@@ -1003,72 +996,114 @@ function makeItemTemplatesHtml($itemid, array $parent_templates, $flag) {
 }
 
 /**
- * Retrieve overview table object for items.
+ * @param array $db_hosts
+ * @param array $db_items
+ * @param array $items_by_name
+ * @param int   $show_suppressed
  *
- * @param array  $groupids
- * @param string $application      IDs of applications to filter items by.
- * @param int    $viewMode
- * @param int    $show_suppressed  Whether to show suppressed problems.
- *
- * @return CTableInfo
+ * @return array
  */
-
-function getItemsDataOverview(array $groupids, $application, $viewMode,
-		$show_suppressed = ZBX_PROBLEM_SUPPRESSED_TRUE) {
-	// application filter
-	if ($application !== '') {
-		$applicationids = array_keys(API::Application()->get([
-			'output' => [],
-			'groupids' => $groupids ? $groupids : null,
-			'search' => ['name' => $application],
-			'preservekeys' => true
-		]));
-		$groupids = [];
-	}
-	else {
-		$applicationids = null;
+function getDataOverviewCellData(array &$db_hosts, array &$db_items, array &$items_by_name, int $show_suppressed): array {
+	$visible_items = [];
+	foreach ($items_by_name as $hostid_to_itemids) {
+		foreach ($hostid_to_itemids as $itemid) {
+			$visible_items[$itemid] = $db_items[$itemid];
+		}
 	}
 
-	$db_items = API::Item()->get([
-		'output' => ['itemid', 'hostid', 'key_', 'name', 'value_type', 'units', 'valuemapid'],
-		'selectHosts' => ['name'],
-		'groupids' => $groupids ? $groupids : null,
-		'applicationids' => $applicationids,
-		'monitored' => true,
-		'webitems' => true,
-		'preservekeys' => true
-	]);
+	$history = Manager::History()->getLastValues($visible_items, 1, ZBX_HISTORY_PERIOD);
 
 	$db_triggers = getTriggersWithActualSeverity([
 		'output' => ['triggerid', 'priority', 'value'],
 		'selectItems' => ['itemid'],
-		'groupids' => $groupids ? $groupids : null,
-		'applicationids' => $applicationids,
+		'itemids' => array_keys($visible_items),
 		'monitored' => true,
 		'preservekeys' => true
 	], ['show_suppressed' => $show_suppressed]);
-
-	foreach ($db_triggers as $db_trigger) {
+	$itemid_to_triggerids = [];
+	foreach ($db_triggers as $triggerid => $db_trigger) {
 		foreach ($db_trigger['items'] as $item) {
-			if (array_key_exists($item['itemid'], $db_items)) {
-				$db_item = &$db_items[$item['itemid']];
-
-				// a little tricky check for attempt to overwrite active trigger (value=1) with
-				// inactive or active trigger with lower priority.
-				if (!array_key_exists('triggerid', $db_item)
-						|| ($db_item['value'] == TRIGGER_VALUE_FALSE && $db_trigger['value'] == TRIGGER_VALUE_TRUE)
-						|| (($db_item['value'] == TRIGGER_VALUE_FALSE || $db_trigger['value'] == TRIGGER_VALUE_TRUE)
-							&& $db_item['priority'] < $db_trigger['priority'])) {
-					$db_item['triggerid'] = $db_trigger['triggerid'];
-					$db_item['priority'] = $db_trigger['priority'];
-					$db_item['value'] = $db_trigger['value'];
-				}
-
-				unset($db_item);
+			if (!array_key_exists($item['itemid'], $itemid_to_triggerids)) {
+				$itemid_to_triggerids[$item['itemid']] = [];
 			}
+			$itemid_to_triggerids[$item['itemid']][] = $triggerid;
 		}
 	}
 
+	foreach ($items_by_name as $name => $hostid_to_itemids) {
+		foreach ($db_hosts as $host) {
+			if (!array_key_exists($host['hostid'], $hostid_to_itemids)) {
+				continue;
+			}
+
+			$itemid = $hostid_to_itemids[$host['hostid']];
+			$visible_items[$itemid]['value'] = array_key_exists($itemid, $history)
+				? $history[$itemid][0]['value']
+				: null;
+			$trigger = null;
+
+			if (array_key_exists($itemid, $itemid_to_triggerids)) {
+				$max_priority = -1;
+				$max_priority_triggerid = -1;
+				foreach ($itemid_to_triggerids[$itemid] as $triggerid) {
+					$trigger = $db_triggers[$triggerid];
+
+					// Bump lower priority triggers of value "true" ahead of triggers with value "false".
+					$multiplier = ($trigger['value'] == TRIGGER_VALUE_TRUE) ? TRIGGER_SEVERITY_COUNT : 0;
+					if ($trigger['priority'] + $multiplier > $max_priority) {
+						$max_priority_triggerid = $triggerid;
+						$max_priority = $trigger['priority'] + $multiplier;
+					}
+				}
+				$trigger = $db_triggers[$max_priority_triggerid];
+			}
+			$visible_items[$itemid]['trigger'] = $trigger;
+		}
+	}
+
+	return $visible_items;
+}
+
+/**
+ * @param array  $groupids
+ * @param array  $hostids
+ * @param string $application
+ *
+ * @return array
+ */
+function getDataOverviewItems(?array $groupids = null, ?array $hostids = null, ?string $application = ''): array {
+	$config = select_config();
+
+	if ($application !== '') {
+		$applicationids = array_keys(API::Application()->get([
+			'output' => [],
+			'hostids' => $hostids,
+			'groupids' => $groupids,
+			'search' => ['name' => $application],
+			'preservekeys' => true
+		]));
+
+		$db_items = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units', 'valuemapid'],
+			'applicationids' => $applicationids,
+			'monitored' => true,
+			'webitems' => true,
+			'limit' => $config['search_limit'],
+			'preservekeys' => true
+		]);
+	}
+	else {
+		$db_items = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units', 'valuemapid'],
+			'hostids' => $hostids,
+			'groupids' => $groupids,
+			'monitored' => true,
+			'webitems' => true,
+			'limit' => $config['search_limit'],
+			'preservekeys' => true
+		]);
+	}
+
 	$db_items = CMacrosResolverHelper::resolveItemNames($db_items);
 
 	CArrayHelper::sort($db_items, [
@@ -1076,179 +1111,152 @@ function getItemsDataOverview(array $groupids, $application, $viewMode,
 		['field' => 'itemid', 'order' => ZBX_SORT_UP]
 	]);
 
-	// fetch latest values
-	$history = Manager::History()->getLastValues(zbx_toHash($db_items, 'itemid'), 1, ZBX_HISTORY_PERIOD);
-
-	// fetch data for the host JS menu
-	$hosts = API::Host()->get([
-		'output' => ['name', 'hostid'],
-		'monitored_hosts' => true,
-		'groupids' => $groupids ? $groupids : null,
-		'applicationids' => $applicationids,
-		'with_monitored_items' => true,
-		'preservekeys' => true
-	]);
-
-	$items = [];
-	$item_counter = [];
-	$host_items = [];
-	$host_names = [];
-
-	foreach ($db_items as $db_item) {
-		$item_name = $db_item['name_expanded'];
-		$host_name = $db_item['hosts'][0]['name'];
-		$host_names[$db_item['hostid']] = $host_name;
-
-		if (!array_key_exists($host_name, $item_counter)) {
-			$item_counter[$host_name] = [];
-		}
-
-		if (!array_key_exists($item_name, $item_counter[$host_name])) {
-			$item_counter[$host_name][$item_name] = 0;
-		}
-
-		if (!array_key_exists($item_name, $host_items) || !array_key_exists($host_name, $host_items[$item_name])) {
-			$host_items[$item_name][$host_name] = [];
-		}
-
-		if (!array_key_exists($db_item['itemid'], $host_items[$item_name][$host_name])) {
-			if (array_key_exists($db_item['itemid'], $host_items[$item_name][$host_name])) {
-				$item_place = $host_items[$item_name][$host_name][$db_item['itemid']]['item_place'];
-			}
-			else {
-				$item_place = $item_counter[$host_name][$item_name];
-				$item_counter[$host_name][$item_name]++;
-			}
-
-			$item = [
-				'itemid' => $db_item['itemid'],
-				'value_type' => $db_item['value_type'],
-				'value' => isset($history[$db_item['itemid']]) ? $history[$db_item['itemid']][0]['value'] : null,
-				'units' => $db_item['units'],
-				'valuemapid' => $db_item['valuemapid'],
-				'item_place' => $item_place
-			];
-
-			if (array_key_exists('triggerid', $db_item)) {
-				$item += [
-					'triggerid' => $db_item['triggerid'],
-					'severity' => $db_item['priority'],
-					'tr_value' => $db_item['value']
-				];
-			}
-			else {
-				$item += [
-					'triggerid' => null,
-					'severity' => null,
-					'tr_value' => null
-				];
-			}
-
-			$items[$item_name][$item_place][$host_name] = $item;
-
-			$host_items[$item_name][$host_name][$db_item['itemid']] = $items[$item_name][$item_place][$host_name];
-		}
-	}
-
-	$table = (new CTableInfo())->setHeadingColumn(0);
-	if (!$host_names) {
-		return $table;
-	}
-	$table->makeVerticalRotation();
-
-	order_result($host_names);
-
-	if ($viewMode == STYLE_TOP) {
-		$header = [_('Items')];
-		foreach ($host_names as $host_name) {
-			$header[] = (new CColHeader($host_name))
-				->addClass('vertical_rotation')
-				->setTitle($host_name);
-		}
-		$table->setHeader($header);
-
-		foreach ($items as $item_name => $item_data) {
-			foreach ($item_data as $ithosts) {
-				$tableRow = [(new CColHeader($item_name))->addClass(ZBX_STYLE_NOWRAP)];
-				foreach ($host_names as $host_name) {
-					$tableRow = getItemDataOverviewCells($tableRow, $ithosts, $host_name);
-				}
-				$table->addRow($tableRow);
-			}
-		}
-	}
-	else {
-		$header = [_('Hosts')];
-		foreach ($items as $item_name => $item_data) {
-			foreach ($item_data as $ithosts) {
-				$header[] = (new CColHeader($item_name))
-					->addClass('vertical_rotation')
-					->setTitle($item_name);
-			}
-		}
-		$table->setHeader($header);
-
-		foreach ($host_names as $hostId => $host_name) {
-			$host = $hosts[$hostId];
-
-			$name = (new CLinkAction($host['name']))->setMenuPopup(CMenuPopupHelper::getHost($hostId));
-
-			$tableRow = [(new CColHeader($name))->addClass(ZBX_STYLE_NOWRAP)];
-			foreach ($items as $item_data) {
-				foreach ($item_data as $ithosts) {
-					$tableRow = getItemDataOverviewCells($tableRow, $ithosts, $host_name);
-				}
-			}
-			$table->addRow($tableRow);
-		}
-	}
-
-	return $table;
+	return $db_items;
 }
 
-function getItemDataOverviewCells($tableRow, $ithosts, $hostName) {
+/**
+ * @param array  $groupids
+ * @param array  $hostids
+ * @param array  $itemids
+ * @param string $application
+ *
+ * @return array
+ */
+function getDataOverviewHosts(?array $groupids, ?array $hostids, ?array $itemids, ?string $application = ''): array {
+	$config = select_config();
+	if ($application !== '') {
+		$applicationids = array_keys(API::Application()->get([
+			'output' => [],
+			'hostids' => $hostids ? $hostids : null,
+			'groupids' => $groupids ? $groupids : null,
+			'search' => ['name' => $application],
+			'preservekeys' => true
+		]));
+
+		$db_hosts = API::Host()->get([
+			'output' => ['name', 'hostid'],
+			'groupids' => $groupids,
+			'applicationids' => $applicationids,
+			'monitored_hosts' => true,
+			'with_monitored_items' => true,
+			'preservekeys' => true,
+			'limit' => $config['search_limit']
+		]);
+	}
+	else {
+		$db_hosts = API::Host()->get([
+			'output' => ['name', 'hostid'],
+			'monitored_hosts' => true,
+			'hostids' => $hostids,
+			'itemids' => $itemids,
+			'groupids' => $groupids,
+			'with_monitored_items' => true,
+			'preservekeys' => true,
+			'limit' => $config['search_limit']
+		]);
+	}
+
+	CArrayHelper::sort($db_hosts, [
+		['field' => 'name', 'order' => ZBX_SORT_UP],
+	]);
+
+	return $db_hosts;
+}
+
+/**
+ * @param array  $groupids
+ * @param array  $hostids
+ * @param string $application
+ *
+ * @return array
+ */
+function getDataOverviewLeft(?array $groupids, ?array $hostids, string $application = ''): array {
+	$db_items = getDataOverviewItems($groupids, $hostids, $application);
+	$items_by_name = [];
+	foreach ($db_items as $itemid => $db_item) {
+		if (!array_key_exists($db_item['name'], $items_by_name)) {
+			$items_by_name[$db_item['name']] = [];
+		}
+		$items_by_name[$db_item['name']][$db_item['hostid']] = $itemid;
+	}
+
+	$hidden_items_cnt = count(array_splice($items_by_name, ZBX_MAX_TABLE_COLUMNS));
+
+	$itemids = [];
+	foreach ($items_by_name as $hostid_to_itemid) {
+		foreach ($hostid_to_itemid as $itemid) {
+			$itemids[] = $itemid;
+		}
+	}
+	$db_items = array_intersect_key($db_items, array_flip($itemids));
+
+	$db_hosts = getDataOverviewHosts(null, null, $itemids);
+	$db_hosts_ctn = count($db_hosts);
+	$db_hosts = array_slice($db_hosts, 0, ZBX_MAX_TABLE_COLUMNS, true);
+
+	$has_hidden_data = ($hidden_items_cnt || ($db_hosts_ctn > count($db_hosts)));
+
+	return [$db_items, $db_hosts, $items_by_name, $has_hidden_data];
+}
+
+/**
+ * @param array  $groupids
+ * @param array  $hostids
+ * @param string $application
+ *
+ * @return array
+ */
+function getDataOverviewTop(?array $groupids, ?array $hostids, string $application = ''): array {
+	$db_hosts = getDataOverviewHosts($groupids, $hostids, null, $application);
+	$hostids = array_keys($db_hosts);
+	$hidden_db_hosts_cnt = count(array_splice($hostids, ZBX_MAX_TABLE_COLUMNS));
+	$db_hosts = array_intersect_key($db_hosts, array_flip($hostids));
+
+	$db_items = getDataOverviewItems(null, $hostids, $application);
+	$items_by_name = [];
+	foreach ($db_items as $itemid => $db_item) {
+		if (!array_key_exists($db_item['name'], $items_by_name)) {
+			$items_by_name[$db_item['name']] = [];
+		}
+		$items_by_name[$db_item['name']][$db_item['hostid']] = $itemid;
+	}
+
+	$items_by_name_ctn = count($items_by_name);
+	$items_by_name = array_slice($items_by_name, 0, ZBX_MAX_TABLE_COLUMNS, true);
+
+	$has_hidden_data = ($hidden_db_hosts_cnt || ($items_by_name_ctn > count($items_by_name)));
+
+	return [$db_items, $db_hosts, $items_by_name, $has_hidden_data];
+}
+
+/**
+ * @param array $item
+ * @param array $trigger
+ *
+ * @return CCol
+ */
+function getItemDataOverviewCell(array $item, ?array $trigger = null): CCol {
 	$ack = null;
 	$css = '';
 	$value = UNKNOWN_VALUE;
 
-	if (isset($ithosts[$hostName])) {
-		$item = $ithosts[$hostName];
+	if ($trigger && $trigger['value'] == TRIGGER_VALUE_TRUE) {
+		$css = getSeverityStyle($trigger['priority']);
 
-		if ($item['tr_value'] == TRIGGER_VALUE_TRUE) {
-			$css = getSeverityStyle($item['severity']);
-
-			// Display event acknowledgement.
-			$ack = getTriggerLastProblems([$item['triggerid']], ['acknowledged']);
-
-			if ($ack) {
-				$ack = reset($ack);
-				$ack = ($ack['acknowledged'] == 1)
-					? [' ', (new CSpan())->addClass(ZBX_STYLE_ICON_ACKN)]
-					: null;
-			}
-		}
-
-		if ($item['value'] !== null) {
-			$value = formatHistoryValue($item['value'], $item);
+		if ($trigger['problem']['acknowledged'] == 1) {
+			$ack = [' ', (new CSpan())->addClass(ZBX_STYLE_ICON_ACKN)];
 		}
 	}
 
-	if ($value != UNKNOWN_VALUE) {
-		$value = $value;
+	if ($item['value'] !== null) {
+		$value = formatHistoryValue($item['value'], $item);
 	}
 
-	$column = (new CCol([$value, $ack]))->addClass($css);
-
-	if (isset($ithosts[$hostName])) {
-		$column
-			->setMenuPopup(CMenuPopupHelper::getHistory($item['itemid']))
-			->addClass(ZBX_STYLE_CURSOR_POINTER)
-			->addClass(ZBX_STYLE_NOWRAP);
-	}
-
-	$tableRow[] = $column;
-
-	return $tableRow;
+	return (new CCol([$value, $ack]))
+		->addClass($css)
+		->setMenuPopup(CMenuPopupHelper::getHistory($item['itemid']))
+		->addClass(ZBX_STYLE_CURSOR_POINTER)
+		->addClass(ZBX_STYLE_NOWRAP);
 }
 
 /**
@@ -1320,15 +1328,12 @@ function formatHistoryValue($value, array $item, $trim = true) {
 
 	// format value
 	if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
-		$value = convert_units([
-				'value' => $value,
-				'units' => $item['units']
+		$value = convertUnits([
+			'value' => $value,
+			'units' => $item['units']
 		]);
 	}
-	elseif ($item['value_type'] != ITEM_VALUE_TYPE_STR
-		&& $item['value_type'] != ITEM_VALUE_TYPE_TEXT
-		&& $item['value_type'] != ITEM_VALUE_TYPE_LOG) {
-
+	elseif (!in_array($item['value_type'], [ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_TEXT, ITEM_VALUE_TYPE_LOG])) {
 		$value = _('Unknown value type');
 	}
 
@@ -1336,7 +1341,8 @@ function formatHistoryValue($value, array $item, $trim = true) {
 	switch ($item['value_type']) {
 		case ITEM_VALUE_TYPE_STR:
 			$mapping = getMappedValue($value, $item['valuemapid']);
-		// break; is not missing here
+			// break; is not missing here
+
 		case ITEM_VALUE_TYPE_TEXT:
 		case ITEM_VALUE_TYPE_LOG:
 			if ($trim && mb_strlen($value) > 20) {
@@ -1346,7 +1352,9 @@ function formatHistoryValue($value, array $item, $trim = true) {
 			if ($mapping !== false) {
 				$value = $mapping.' ('.$value.')';
 			}
+
 			break;
+
 		default:
 			$value = applyValueMap($value, $item['valuemapid']);
 	}
@@ -1368,33 +1376,37 @@ function formatHistoryValue($value, array $item, $trim = true) {
  * @return string item functional value from history
  */
 function getItemFunctionalValue($item, $function, $parameter) {
-	// check whether function is allowed
+	// Check whether function is allowed and parameter is specified.
 	if (!in_array($function, ['min', 'max', 'avg']) || $parameter === '') {
 		return UNRESOLVED_MACRO_STRING;
 	}
 
-	$parameter = convertFunctionValue($parameter);
-
-	if (bccomp($parameter, 0) == 0) {
+	// Check whether item type is allowed for min, max and avg functions.
+	if ($item['value_type'] != ITEM_VALUE_TYPE_FLOAT && $item['value_type'] != ITEM_VALUE_TYPE_UINT64) {
 		return UNRESOLVED_MACRO_STRING;
 	}
 
-	// allowed item types for min, max and avg function
-	$history_tables = [ITEM_VALUE_TYPE_FLOAT => 'history', ITEM_VALUE_TYPE_UINT64 => 'history_uint'];
+	$number_parser = new CNumberParser(['with_suffix' => true]);
 
-	if (!array_key_exists($item['value_type'], $history_tables)) {
+	if ($number_parser->parse($parameter) != CParser::PARSE_SUCCESS) {
 		return UNRESOLVED_MACRO_STRING;
 	}
+
+	$parameter = $number_parser->calcValue();
+
+	$time_from = time() - $parameter;
+
+	if ($time_from < 0 || $time_from > ZBX_MAX_DATE) {
+		return UNRESOLVED_MACRO_STRING;
+	}
+
+	$result = Manager::History()->getAggregatedValue($item, $function, $time_from);
+
+	if ($result !== null) {
+		return convertUnits(['value' => $result, 'units' => $item['units']]);
+	}
 	else {
-		$result = Manager::History()->getAggregatedValue($item, $function, (time() - $parameter));
-
-		if ($result !== null) {
-			return convert_units(['value' => $result, 'units' => $item['units']]);
-		}
-		// no data in history
-		else {
-			return UNRESOLVED_MACRO_STRING;
-		}
+		return UNRESOLVED_MACRO_STRING;
 	}
 }
 
@@ -1681,73 +1693,147 @@ function getParamFieldLabelByType($itemType) {
 }
 
 /**
- * Get either one or all item preprocessing types. If grouped set to true, returns group labels. Returns empty string if
- * no specific type is found.
+ * Get either one or all item preprocessing types.
+ * If $grouped set to true, returns group labels. Returns empty string if no specific type is found.
  *
  * Usage examples:
- *    - get_preprocessing_types()              Returns array as defined.
- *    - get_preprocessing_types(4)             Returns string: 'Trim'.
- *    - get_preprocessing_types(<wrong type>)  Returns an empty string: ''.
- *    - get_preprocessing_types(null, false)   Returns subarrays in one array maintaining index:
- *                                               [5] => Regular expression
- *                                               [4] => Trim
- *                                               [2] => Right trim
- *                                               [3] => Left trim
- *                                               [11] => XML XPath
- *                                               [12] => JSON Path
- *                                               [1] => Custom multiplier
- *                                               [9] => Simple change
- *                                               [10] => Speed per second
- *                                               [6] => Boolean to decimal
- *                                               [7] => Octal to decimal
- *                                               [8] => Hexadecimal to decimal
+ *    - get_preprocessing_types(null, true, [5, 4, 2])             Returns array as defined.
+ *    - get_preprocessing_types(4, true, [5, 4, 2])                Returns string: 'Trim'.
+ *    - get_preprocessing_types(<wrong type>, true, [5, 4, 2])     Returns an empty string: ''.
+ *    - get_preprocessing_types(null, false, [5, 12, 15, 16, 20])  Returns subarrays in one array maintaining index:
+ *                                                                     [5] => Regular expression
+ *                                                                     [12] => JSONPath
+ *                                                                     [15] => Does not match regular expression
+ *                                                                     [16] => Check for error in JSON
+ *                                                                     [20] => Discard unchanged with heartbeat
  *
- * @param int  $type     Item preprocessing type.
- * @param bool $grouped  Group label flag.
+ * @param int   $type             Item preprocessing type.
+ * @param bool  $grouped          Group label flag. If specific type is given, this parameter does not matter.
+ * @param array $supported_types  Array of supported pre-processing types. If none are given, empty array is returned.
  *
- * @return mixed
+ * @return array|string
  */
-function get_preprocessing_types($type = null, $grouped = true) {
-	$groups = [
-		[
-			'label' => _('Text'),
-			'types' => [
-				ZBX_PREPROC_REGSUB => _('Regular expression'),
-				ZBX_PREPROC_TRIM => _('Trim'),
-				ZBX_PREPROC_RTRIM => _('Right trim'),
-				ZBX_PREPROC_LTRIM => _('Left trim')
-			]
+function get_preprocessing_types($type = null, $grouped = true, array $supported_types = []) {
+	$types = [
+		ZBX_PREPROC_REGSUB => [
+			'group' => _('Text'),
+			'name' => _('Regular expression')
 		],
-		[
-			'label' => _('Structured data'),
-			'types' => [
-				ZBX_PREPROC_XPATH => _('XML XPath'),
-				ZBX_PREPROC_JSONPATH => _('JSON Path')
-			]
+		ZBX_PREPROC_STR_REPLACE => [
+			'group' => _('Text'),
+			'name' => _('Replace')
 		],
-		[
-			'label' => _('Arithmetic'),
-			'types' => [
-				ZBX_PREPROC_MULTIPLIER => _('Custom multiplier')
-			]
+		ZBX_PREPROC_TRIM => [
+			'group' => _('Text'),
+			'name' => _('Trim')
 		],
-		[
-			'label' => _x('Change', 'noun'),
-			'types' => [
-				ZBX_PREPROC_DELTA_VALUE => _('Simple change'),
-				ZBX_PREPROC_DELTA_SPEED => _('Change per second')
-			]
+		ZBX_PREPROC_RTRIM => [
+			'group' => _('Text'),
+			'name' => _('Right trim')
 		],
-		[
-			'label' => _('Numeral systems'),
-			'types' => [
-				ZBX_PREPROC_BOOL2DEC => _('Boolean to decimal'),
-				ZBX_PREPROC_OCT2DEC => _('Octal to decimal'),
-				ZBX_PREPROC_HEX2DEC => _('Hexadecimal to decimal')
-			]
+		ZBX_PREPROC_LTRIM => [
+			'group' => _('Text'),
+			'name' => _('Left trim')
+		],
+		ZBX_PREPROC_XPATH => [
+			'group' => _('Structured data'),
+			'name' => _('XML XPath')
+		],
+		ZBX_PREPROC_JSONPATH => [
+			'group' => _('Structured data'),
+			'name' => _('JSONPath')
+		],
+		ZBX_PREPROC_CSV_TO_JSON => [
+			'group' => _('Structured data'),
+			'name' => _('CSV to JSON')
+		],
+		ZBX_PREPROC_MULTIPLIER => [
+			'group' => _('Arithmetic'),
+			'name' => _('Custom multiplier')
+		],
+		ZBX_PREPROC_DELTA_VALUE => [
+			'group' => _x('Change', 'noun'),
+			'name' => _('Simple change')
+		],
+		ZBX_PREPROC_DELTA_SPEED => [
+			'group' => _x('Change', 'noun'),
+			'name' => _('Change per second')
+		],
+		ZBX_PREPROC_BOOL2DEC => [
+			'group' => _('Numeral systems'),
+			'name' => _('Boolean to decimal')
+		],
+		ZBX_PREPROC_OCT2DEC => [
+			'group' => _('Numeral systems'),
+			'name' => _('Octal to decimal')
+		],
+		ZBX_PREPROC_HEX2DEC => [
+			'group' => _('Numeral systems'),
+			'name' => _('Hexadecimal to decimal')
+		],
+		ZBX_PREPROC_SCRIPT => [
+			'group' => _('Custom scripts'),
+			'name' => _('JavaScript')
+		],
+		ZBX_PREPROC_VALIDATE_RANGE => [
+			'group' => _('Validation'),
+			'name' => _('In range')
+		],
+		ZBX_PREPROC_VALIDATE_REGEX => [
+			'group' => _('Validation'),
+			'name' => _('Matches regular expression')
+		],
+		ZBX_PREPROC_VALIDATE_NOT_REGEX => [
+			'group' => _('Validation'),
+			'name' => _('Does not match regular expression')
+		],
+		ZBX_PREPROC_ERROR_FIELD_JSON => [
+			'group' => _('Validation'),
+			'name' => _('Check for error in JSON')
+		],
+		ZBX_PREPROC_ERROR_FIELD_XML => [
+			'group' => _('Validation'),
+			'name' => _('Check for error in XML')
+		],
+		ZBX_PREPROC_ERROR_FIELD_REGEX => [
+			'group' => _('Validation'),
+			'name' => _('Check for error using regular expression')
+		],
+		ZBX_PREPROC_THROTTLE_VALUE => [
+			'group' => _('Throttling'),
+			'name' => _('Discard unchanged')
+		],
+		ZBX_PREPROC_THROTTLE_TIMED_VALUE => [
+			'group' => _('Throttling'),
+			'name' => _('Discard unchanged with heartbeat')
+		],
+		ZBX_PREPROC_PROMETHEUS_PATTERN => [
+			'group' => _('Prometheus'),
+			'name' => _('Prometheus pattern')
+		],
+		ZBX_PREPROC_PROMETHEUS_TO_JSON => [
+			'group' => _('Prometheus'),
+			'name' => _('Prometheus to JSON')
 		]
 	];
 
+	$filtered_types = [];
+
+	foreach ($types as $_type => $data) {
+		if (in_array($_type, $supported_types)) {
+			$filtered_types[$data['group']][$_type] = $data['name'];
+		}
+	}
+
+	$groups = [];
+
+	foreach ($filtered_types as $label => $types) {
+		$groups[] = [
+			'label' => $label,
+			'types' => $types
+		];
+	}
+
 	if ($type !== null) {
 		foreach ($groups as $group) {
 			if (array_key_exists($type, $group['types'])) {
@@ -1871,11 +1957,8 @@ function expandItemNamesWithMasterItems($items, $data_source) {
 function checkNowAllowedTypes() {
 	return [
 		ITEM_TYPE_ZABBIX,
-		ITEM_TYPE_SNMPV1,
 		ITEM_TYPE_SIMPLE,
-		ITEM_TYPE_SNMPV2C,
 		ITEM_TYPE_INTERNAL,
-		ITEM_TYPE_SNMPV3,
 		ITEM_TYPE_AGGREGATE,
 		ITEM_TYPE_EXTERNAL,
 		ITEM_TYPE_DB_MONITOR,
@@ -1884,6 +1967,64 @@ function checkNowAllowedTypes() {
 		ITEM_TYPE_TELNET,
 		ITEM_TYPE_CALCULATED,
 		ITEM_TYPE_JMX,
-		ITEM_TYPE_HTTPAGENT
+		ITEM_TYPE_HTTPAGENT,
+		ITEM_TYPE_SNMP
 	];
 }
+
+/**
+ * Validates update interval for items, item prototypes and low-level discovery rules and their overrides.
+ *
+ * @param CUpdateIntervalParser $parser [IN]      Parser used for delay validation.
+ * @param string                $value  [IN]      Update interval to parse and validate.
+ * @param string                $field_name [IN]  Frontend or API field name in the error
+ * @param string                $error  [OUT]     Returned error string if delay validation fails.
+ *
+ * @return bool
+ */
+function validateDelay(CUpdateIntervalParser $parser, $field_name, $value, &$error) {
+	if ($parser->parse($value) != CParser::PARSE_SUCCESS) {
+		$error = _s('Incorrect value for field "%1$s": %2$s.', $field_name, _('invalid delay'));
+
+		return false;
+	}
+
+	$delay = $parser->getDelay();
+
+	if ($delay[0] !== '{') {
+		$delay_sec = timeUnitToSeconds($delay);
+		$intervals = $parser->getIntervals();
+		$flexible_intervals = $parser->getIntervals(ITEM_DELAY_FLEXIBLE);
+		$has_scheduling_intervals = (bool) $parser->getIntervals(ITEM_DELAY_SCHEDULING);
+		$has_macros = false;
+
+		foreach ($intervals as $interval) {
+			if (strpos($interval['interval'], '{') !== false) {
+				$has_macros = true;
+				break;
+			}
+		}
+
+		// If delay is 0, there must be at least one either flexible or scheduling interval.
+		if ($delay_sec == 0 && !$intervals) {
+			$error = _('Item will not be refreshed. Specified update interval requires having at least one either flexible or scheduling interval.');
+
+			return false;
+		}
+		elseif ($delay_sec < 0 || $delay_sec > SEC_PER_DAY) {
+			$error = _('Item will not be refreshed. Update interval should be between 1s and 1d. Also Scheduled/Flexible intervals can be used.');
+
+			return false;
+		}
+
+		// If there are scheduling intervals or intervals with macros, skip the next check calculation.
+		if (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
+				&& calculateItemNextCheck(0, $delay_sec, $flexible_intervals, time()) == ZBX_JAN_2038) {
+			$error = _('Item will not be refreshed. Please enter a correct update interval.');
+
+			return false;
+		}
+	}
+
+	return true;
+}
