@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,12 +19,6 @@
 **/
 
 
-function get_maintenance_by_maintenanceid($maintenanceid) {
-	return DBfetch(DBselect(
-		'SELECT m.* FROM maintenances m WHERE m.maintenanceid='.zbx_dbstr($maintenanceid)
-	));
-}
-
 function timeperiod_type2str($timeperiod_type) {
 	switch ($timeperiod_type) {
 		case TIMEPERIOD_TYPE_ONETIME:
@@ -39,7 +33,7 @@ function timeperiod_type2str($timeperiod_type) {
 	return _('Unknown');
 }
 
-function shedule2str($timeperiod) {
+function schedule2str($timeperiod) {
 	$timeperiod['hour'] = floor($timeperiod['start_time'] / SEC_PER_HOUR);
 	$timeperiod['minute'] = floor(($timeperiod['start_time'] - ($timeperiod['hour'] * SEC_PER_HOUR)) / SEC_PER_MIN);
 	if ($timeperiod['hour'] < 10) {
