@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ function select_config() {
 
 function setHostGroupInternal($groupid, $internal) {
 	return DBexecute(
-		'UPDATE groups'.
+		'UPDATE hstgrp'.
 		' SET internal='.zbx_dbstr($internal).
 		' WHERE '.dbConditionInt('groupid', [$groupid])
 	);
@@ -72,12 +72,6 @@ function update_config($config) {
 	}
 
 	$fields = [
-		'event_expire' => [
-			'min' => SEC_PER_DAY,
-			'max' => 25 * SEC_PER_YEAR,
-			'allow_zero' => false,
-			'message' => _('Invalid event expiry time: %1$s.')
-		],
 		'hk_events_trigger' => [
 			'min' => SEC_PER_DAY,
 			'max' => 25 * SEC_PER_YEAR,
@@ -100,7 +94,7 @@ function update_config($config) {
 			'min' => SEC_PER_DAY,
 			'max' => 25 * SEC_PER_YEAR,
 			'allow_zero' => false,
-			'message' => _('Invalid auto-registration data storage period: %1$s.')
+			'message' => _('Invalid autoregistration data storage period: %1$s.')
 		],
 		'hk_services' => [
 			'min' => SEC_PER_DAY,
@@ -149,9 +143,20 @@ function update_config($config) {
 			'max' => SEC_PER_DAY,
 			'allow_zero' => false,
 			'message' => _('Invalid refresh of unsupported items: %1$s')
-		]
+		],
+		'compress_older' => [
+			'min' => SEC_PER_DAY * 7,
+			'max' => 25 * SEC_PER_YEAR,
+			'allow_zero' => false,
+			'message' => _s('Invalid parameter "%1$s": %2$s.', _('Compress records older than'), '%1$s')
+		],
 	];
 
+	if (array_key_exists('compression_status', $config) && $config['compression_status'] === 0) {
+		unset($fields['compress_older']);
+		unset($config['compress_older']);
+	}
+
 	foreach ($fields as $field => $args) {
 		if (array_key_exists($field, $config)
 				&& !validateTimeUnit($config[$field], $args['min'], $args['max'], $args['allow_zero'], $error)) {
@@ -179,7 +184,7 @@ function update_config($config) {
 			}
 
 			if (isset($names[$config[$varName]])) {
-				error(_s('Duplicate severity name "%s".', $config[$varName]));
+				error(_s('Duplicate severity name "%1$s".', $config[$varName]));
 				return false;
 			}
 			else {
@@ -224,7 +229,7 @@ function update_config($config) {
 			);
 		}
 		if (array_key_exists('hk_events_autoreg', $config)) {
-			$msg[] = _s('Auto-registration event and alert data storage period "%1$s".',
+			$msg[] = _s('Autoregistration event and alert data storage period "%1$s".',
 				$config['hk_events_autoreg']
 			);
 		}
@@ -249,21 +254,6 @@ function update_config($config) {
 		if (array_key_exists('default_theme', $config)) {
 			$msg[] = _s('Default theme "%1$s".', $config['default_theme']);
 		}
-		if (array_key_exists('event_ack_enable', $config)) {
-			$msg[] = _s('Event acknowledges "%1$s".', $config['event_ack_enable']);
-		}
-		if (array_key_exists('event_expire', $config)) {
-			$msg[] = _s('Show events not older than "%1$s".', $config['event_expire']);
-		}
-		if (array_key_exists('event_show_max', $config)) {
-			$msg[] = _s('Show events max "%1$s".', $config['event_show_max']);
-		}
-		if (array_key_exists('dropdown_first_entry', $config)) {
-			$msg[] = _s('Dropdown first entry "%1$s".', $config['dropdown_first_entry']);
-		}
-		if (array_key_exists('dropdown_first_remember', $config)) {
-			$msg[] = _s('Dropdown remember selected "%1$s".', $config['dropdown_first_remember']);
-		}
 		if (array_key_exists('max_in_table', $config)) {
 			$msg[] = _s('Max count of elements to show inside table cell "%1$s".', $config['max_in_table']);
 		}
