@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 include dirname(__FILE__).'/js/conf.import.js.php';
 
 $rulesTable = (new CTable())
@@ -44,6 +48,7 @@ $user_type = CWebUser::getType();
 
 if ($user_type == USER_TYPE_SUPER_ADMIN) {
 	$titles['images'] = _('Images');
+	$titles['mediaTypes'] = _('Media types');
 	$titles['valueMaps'] = _('Value mappings');
 }
 
@@ -62,12 +67,12 @@ foreach ($titles as $key => $title) {
 		}
 
 		if ($key === 'images') {
-			$cbExist->onClick('updateWarning(this, '.CJs::encodeJson(_('Images for all maps will be updated!')).')');
+			$cbExist->onClick('updateWarning(this, '.json_encode(_('Images for all maps will be updated!')).')');
 		}
 
 		if ($key === 'valueMaps') {
 			$cbExist->onClick(
-				'updateWarning(this, '.CJs::encodeJson(_('Value mappings for value maps will be updated!')).')'
+				'updateWarning(this, '.json_encode(_('Value mappings for value maps will be updated!')).')'
 			);
 		}
 	}
@@ -91,6 +96,12 @@ foreach ($titles as $key => $title) {
 				&& $user_type != USER_TYPE_ZABBIX_ADMIN) {
 			$cbDeleted->setAttribute('disabled', 'disabled');
 		}
+
+		if ($key === 'templateLinkage') {
+			$cbDeleted->onClick('updateWarning(this, '.json_encode(
+				_('Template and host properties that are inherited through template linkage will be unlinked and cleared.')
+			).')');
+		}
 	}
 
 	$rulesTable->addRow([
@@ -103,7 +114,12 @@ foreach ($titles as $key => $title) {
 
 // form list
 $form_list = (new CFormList())
-	->addRow(_('Import file'), (new CFile('import_file'))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH))
+	->addRow((new CLabel(_('Import file'), 'import_file'))->setAsteriskMark(),
+		(new CFile('import_file'))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
+			->setAttribute('autofocus', 'autofocus')
+	)
 	->addRow(_('Rules'), new CDiv($rulesTable));
 
 // tab
@@ -116,8 +132,12 @@ $tab_view->setFooter(makeFormFooter(
 ));
 
 $form = (new CForm('post', null, 'multipart/form-data'))
-	->addVar('backurl', $data['backurl'])
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+	->addVar('rules_preset', $data['rules_preset'])
 	->addItem($tab_view);
 
 // widget
-return (new CWidget())->addItem($form);
+(new CWidget())
+	->setTitle(_('Import'))
+	->addItem($form)
+	->show();
