@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $widget = new CWidget();
 
 if ($data['parent_discoveryid'] === null) {
@@ -69,7 +73,10 @@ if ($is_templated || $discovered_graph) {
 
 if ($discovered_graph) {
 	$graphFormList->addRow(_('Discovered by'), new CLink($data['discoveryRule']['name'],
-		'graphs.php?parent_discoveryid='.$data['discoveryRule']['itemid']
+		(new CUrl('graphs.php'))
+			->setArgument('form', 'update')
+			->setArgument('parent_discoveryid', $data['discoveryRule']['itemid'])
+			->setArgument('graphid', $data['graphDiscovery']['parent_graphid'])
 	));
 }
 
@@ -92,7 +99,9 @@ $graphFormList
 			->setAriaRequired()
 	)
 	->addRow((new CLabel(_('Graph type'), 'graphtype')),
-		(new CComboBox('graphtype', $this->data['graphtype'], 'submit()', graphType()))->setEnabled(!$readonly)
+		(new CComboBox('graphtype', $this->data['graphtype'], 'jQuery(\'form[name="graphForm"]\').submit()',
+			graphType())
+		)->setEnabled(!$readonly)
 	)
 	->addRow(_('Show legend'),
 		(new CCheckBox('show_legend'))
@@ -160,7 +169,7 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 	if ($this->data['ymin_type'] == GRAPH_YAXIS_TYPE_FIXED) {
 		$yaxisMinData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 		$yaxisMinData[] = (new CTextBox('yaxismin', $this->data['yaxismin'], $readonly))
-			->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
+			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH);
 	}
 	elseif ($this->data['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
 		$graphForm->addVar('yaxismin', $this->data['yaxismin']);
@@ -185,7 +194,7 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 		$yaxisMinData[] = (new CButton('yaxis_min', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('return PopUp("popup.generic",jQuery.extend('.
-				CJs::encodeJson([
+				json_encode([
 					'srctbl' => 'items',
 					'srcfld1' => 'itemid',
 					'srcfld2' => 'name',
@@ -206,7 +215,7 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 			$yaxisMinData[] = (new CButton('yaxis_min_prototype', _('Select prototype')))
 				->addClass(ZBX_STYLE_BTN_GREY)
 				->onClick('return PopUp("popup.generic",'.
-					CJs::encodeJson([
+					json_encode([
 						'srctbl' => 'item_prototypes',
 						'srcfld1' => 'itemid',
 						'srcfld2' => 'name',
@@ -242,7 +251,7 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 	if ($this->data['ymax_type'] == GRAPH_YAXIS_TYPE_FIXED) {
 		$yaxisMaxData[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 		$yaxisMaxData[] = (new CTextBox('yaxismax', $this->data['yaxismax'], $readonly))
-			->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
+			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH);
 	}
 	elseif ($this->data['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
 		$graphForm->addVar('yaxismax', $this->data['yaxismax']);
@@ -267,7 +276,7 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 		$yaxisMaxData[] = (new CButton('yaxis_max', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('return PopUp("popup.generic",jQuery.extend('.
-				CJs::encodeJson([
+				json_encode([
 					'srctbl' => 'items',
 					'srcfld1' => 'itemid',
 					'srcfld2' => 'name',
@@ -288,7 +297,7 @@ if ($this->data['graphtype'] == GRAPH_TYPE_NORMAL || $this->data['graphtype'] ==
 			$yaxisMaxData[] = (new CButton('yaxis_max_prototype', _('Select prototype')))
 				->addClass(ZBX_STYLE_BTN_GREY)
 				->onClick('return PopUp("popup.generic",'.
-					CJs::encodeJson([
+					json_encode([
 						'srctbl' => 'item_prototypes',
 						'srcfld1' => 'itemid',
 						'srcfld2' => 'name',
@@ -361,8 +370,7 @@ $popup_options_add = [
 if ($data['normal_only']) {
 	$popup_options_add['normal_only'] = '1';
 }
-if ($data['groupid'] && $data['hostid']) {
-	$popup_options_add['groupid'] = $data['groupid'];
+if ($data['hostid']) {
 	$popup_options_add['hostid'] = $data['hostid'];
 }
 
@@ -391,13 +399,13 @@ $items_table->addRow(
 				new CHorList([
 					(new CButton('add_item', _('Add')))
 						->onClick('return PopUp("popup.generic",jQuery.extend('.
-							CJs::encodeJson($popup_options_add).',getOnlyHostParam()), null, this);'
+							json_encode($popup_options_add).',getOnlyHostParam()), null, this);'
 						)
 						->addClass(ZBX_STYLE_BTN_LINK),
 					$data['parent_discoveryid']
 						? (new CButton('add_protoitem', _('Add prototype')))
 							->onClick('return PopUp("popup.generic",'.
-								CJs::encodeJson($popup_options_add_prototype).', null, this);'
+								json_encode($popup_options_add_prototype).', null, this);'
 							)
 							->addClass(ZBX_STYLE_BTN_LINK)
 						: null
@@ -417,8 +425,12 @@ foreach ($this->data['items'] as $n => $item) {
 		$item['yaxisside'] = 0;
 	}
 
-	insert_js('loadItem('.$n.', '.CJs::encodeJson($item['gitemid']).', '.$this->data['graphid'].', '.$item['itemid'].', '.
-		CJs::encodeJson($name).', '.$item['type'].', '.$item['calc_fnc'].', '.$item['drawtype'].', '.
+	if (!array_key_exists('gitemid', $item)) {
+		$item['gitemid'] = '';
+	}
+
+	insert_js('loadItem('.$n.', '.json_encode($item['gitemid']).', '.$item['itemid'].', '.
+		json_encode($name).', '.$item['type'].', '.$item['calc_fnc'].', '.$item['drawtype'].', '.
 		$item['yaxisside'].', \''.$item['color'].'\', '.$item['flags'].');',
 		true
 	);
@@ -429,6 +441,14 @@ $graphFormList->addRow(
 	(new CDiv($items_table))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 );
 
+if ($data['parent_discoveryid']) {
+	$graphFormList->addRow(_('Discover'),
+		(new CCheckBox('discover', ZBX_PROTOTYPE_DISCOVER))
+			->setChecked($data['discover'] == ZBX_PROTOTYPE_DISCOVER)
+			->setUncheckedValue(ZBX_PROTOTYPE_NO_DISCOVER)
+	);
+}
+
 // Append tabs to form.
 $graphTab = (new CTabView())
 	->setSelected(0)
@@ -485,4 +505,4 @@ $graphForm->addItem($graphTab);
 // Append form to widget.
 $widget->addItem($graphForm);
 
-return $widget;
+$widget->show();
