@@ -1,6 +1,32 @@
 <?php
+/*
+** Zabbix
+** Copyright (C) 2001-2020 Zabbix SIA
+**
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+**/
+
+
+/**
+ * @var CView $this
+ */
 
 include dirname(__FILE__).'/common.item.edit.js.php';
+include dirname(__FILE__).'/item.preprocessing.js.php';
+include dirname(__FILE__).'/editabletable.js.php';
+include dirname(__FILE__).'/itemtest.js.php';
 
 $this->data['valueTypeVisibility'] = [];
 zbx_subarray_push($this->data['valueTypeVisibility'], ITEM_VALUE_TYPE_UINT64, 'units');
@@ -45,6 +71,7 @@ zbx_subarray_push($this->data['valueTypeVisibility'], ITEM_VALUE_TYPE_UINT64, 'r
 					&& type != <?= ITEM_TYPE_DB_MONITOR ?>
 					&& type != <?= ITEM_TYPE_SNMPTRAP ?>
 					&& type != <?= ITEM_TYPE_JMX ?>
+					&& type != <?= ITEM_TYPE_IPMI ?>
 			)
 
 			if ((type == <?= ITEM_TYPE_SSH ?> || type == <?= ITEM_TYPE_TELNET ?>)) {
