@@ -1,4 +1,28 @@
 <?php
+/*
+** Zabbix
+** Copyright (C) 2001-2020 Zabbix SIA
+**
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+**/
+
+
+/**
+ * @var CView $this
+ */
+
 $shape_border_types = [
 	SYSMAP_SHAPE_BORDER_TYPE_NONE		=> _('None'),
 	SYSMAP_SHAPE_BORDER_TYPE_SOLID		=> '———',
@@ -8,7 +32,7 @@ $shape_border_types = [
 
 $horizontal_align_types = [
 	SYSMAP_SHAPE_LABEL_HALIGN_LEFT		=> _('Left'),
-	SYSMAP_SHAPE_LABEL_HALIGN_CENTER	=> _('Center'),
+	SYSMAP_SHAPE_LABEL_HALIGN_CENTER	=> _('Centre'),
 	SYSMAP_SHAPE_LABEL_HALIGN_RIGHT		=> _('Right')
 ];
 
@@ -54,7 +78,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapElementFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Map element'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('formDragHandler')
 			->toString()
 	?>
@@ -154,7 +177,6 @@ function getFontComboBox($name) {
 					], 'triggerListRow')
 					->addRow((new CLabel(_('New triggers'), 'elementNameTriggers_ms')),
 						(new CDiv([
-							new CVar('elementExpressionTrigger', ''),
 							(new CMultiSelect([
 								'name' => 'elementNameTriggers',
 								'object_name' => 'triggers',
@@ -191,7 +213,7 @@ function getFontComboBox($name) {
 						(new CButton(null, _('Select')))
 							->addClass(ZBX_STYLE_BTN_GREY)
 							->onClick('return PopUp("popup.generic",jQuery.extend('.
-								CJs::encodeJson([
+								json_encode([
 									'srctbl' => 'sysmaps',
 									'srcfld1' => 'sysmapid',
 									'srcfld2' => 'name',
@@ -248,7 +270,7 @@ function getFontComboBox($name) {
 						_('Y'),
 						(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 						(new CTextBox('y'))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
-					], 'areaSizeRow')
+					])
 					->addRow(_('URLs'),
 						(new CDiv([
 							(new CTable())
@@ -287,7 +309,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapShapeFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Map shape'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('shapeDragHandler')
 			->toString().
 		(new CForm())
@@ -377,14 +398,18 @@ function getFontComboBox($name) {
 								->setAttribute('data-value', _('X'))
 								->setAttribute('data-value-2', _('X1')),
 							(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-							(new CTextBox('x'))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH),
+							(new CTextBox('x'))
+								->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+								->setId('shapeX'),
 							(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 							(new CSpan())
 								->addClass('switchable-content')
 								->setAttribute('data-value', _('Y'))
 								->setAttribute('data-value-2', _('Y1')),
 							(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-							(new CTextBox('y'))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+							(new CTextBox('y'))
+								->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+								->setId('shapeY')
 						]))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 					)
 					->addRow((new CSpan())
@@ -399,7 +424,7 @@ function getFontComboBox($name) {
 							(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 							(new CTextBox('width'))
 								->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
-								->setId('areaSizeWidth'),
+								->setId('shapeAreaSizeWidth'),
 							(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 							(new CSpan())
 								->addClass('switchable-content')
@@ -408,7 +433,7 @@ function getFontComboBox($name) {
 							(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 							(new CTextBox('height'))
 								->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
-								->setId('areaSizeHeight')
+								->setId('shapeAreaSizeHeight')
 						]))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 					)
 					->addItem([
@@ -436,7 +461,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapMassShapeFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Mass update shapes'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('massShapeDragHandler')
 			->toString().
 		(new CForm())
@@ -557,7 +581,6 @@ function getFontComboBox($name) {
 <script type="text/x-jquery-tmpl" id="mapMassFormTpl">
 	<?= (new CDiv(new CTag('h4', true, _('Mass update elements'))))
 			->addClass(ZBX_STYLE_DASHBRD_WIDGET_HEAD)
-			->addClass(ZBX_STYLE_CURSOR_MOVE)
 			->setId('massDragHandler')
 			->toString()
 	?>
@@ -714,7 +737,7 @@ function getFontComboBox($name) {
 							(new CButton(null, _('Add')))
 								->addClass(ZBX_STYLE_BTN_LINK)
 								->onClick('return PopUp("popup.generic",'.
-									CJs::encodeJson([
+									json_encode([
 										'srctbl' => 'triggers',
 										'srcfld1' => 'triggerid',
 										'reference' => 'linktrigger',
@@ -833,7 +856,6 @@ function getFontComboBox($name) {
 				(new CVar('element_id[#{triggerid}]', '#{triggerid}')),
 				(new CVar('element_name[#{triggerid}]', '#{name}')),
 				(new CVar('element_priority[#{triggerid}]', '#{priority}')),
-				(new CVar('element_expression[#{triggerid}]', '#{expression}')),
 				(new CButton(null, _('Remove')))
 					->addClass(ZBX_STYLE_BTN_LINK)
 					->addStyle('margin: 0 5px;')
