@@ -19,15 +19,19 @@
 **/
 
 
-$web_layout_mode = CView::getLayoutMode();
+/**
+ * @var CView $this
+ */
+
+$web_layout_mode = CViewHelper::loadLayoutMode();
 
 $historyWidget = (new CWidget())->setWebLayoutMode($web_layout_mode);
 
 $header = [
 	'left' => _n('%1$s item', '%1$s items', count($data['items'])),
 	'right' => (new CForm('get'))
-		->addVar('itemids', getRequest('itemids'))
-		->addVar('page', 1)
+		->cleanItems()
+		->addVar('itemids', $data['itemids'])
 ];
 $header_row = [];
 
@@ -58,10 +62,10 @@ else {
 }
 
 if (hasRequest('filter_task')) {
-	$header['right']->addVar('filter_task', getRequest('filter_task'));
+	$header['right']->addVar('filter_task', $data['filter_task']);
 }
 if (hasRequest('filter')) {
-	$header['right']->addVar('filter', getRequest('filter'));
+	$header['right']->addVar('filter', $data['filter']);
 }
 if (hasRequest('mark_color')) {
 	$header['right']->addVar('mark_color', getRequest('mark_color'));
@@ -84,7 +88,9 @@ $action_list = (new CList())
 	->addItem([
 		new CLabel(_('View as')),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CComboBox('action', $data['action'], 'submit()', $actions))->setEnabled((bool) $data['items']),
+		(new CComboBox('action', $data['action'], 'submit()', $actions))
+			->setEnabled((bool) $data['items'])
+			->removeId()
 	]);
 
 if ($data['action'] !== HISTORY_GRAPH && $data['action'] !== HISTORY_BATCH_GRAPH) {
@@ -99,7 +105,7 @@ if ($data['action'] == HISTORY_GRAPH && count($data['items']) == 1) {
 	]));
 }
 
-$action_list->addItem([(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN), get_icon('fullscreen')]);
+$action_list->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode]));
 
 $header['right']->addItem($action_list);
 
@@ -148,25 +154,25 @@ if ($data['action'] == HISTORY_LATEST || $data['action'] == HISTORY_VALUES) {
 					]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 				)
 				->addRow(_('Value'),
-					(new CTextBox('filter', getRequest('filter', '')))->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+					(new CTextBox('filter', getRequest('filter', '')))
+						->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+						->removeId()
 				);
 
-			$filterTask = getRequest('filter_task', 0);
-
-			$tasks = [new CComboBox('filter_task', $filterTask, 'submit()', [
+			$tasks = [(new CComboBox('filter_task', $data['filter_task'], 'submit()', [
 				FILTER_TASK_SHOW => _('Show selected'),
 				FILTER_TASK_HIDE => _('Hide selected'),
 				FILTER_TASK_MARK => _('Mark selected'),
 				FILTER_TASK_INVERT_MARK => _('Mark others')
-			])];
+			]))->removeId()];
 
-			if (str_in_array($filterTask, [FILTER_TASK_MARK, FILTER_TASK_INVERT_MARK])) {
+			if (str_in_array($data['filter_task'], [FILTER_TASK_MARK, FILTER_TASK_INVERT_MARK])) {
 				$tasks[] = ' ';
-				$tasks[] = new CComboBox('mark_color', getRequest('mark_color', 0), null, [
+				$tasks[] = (new CComboBox('mark_color', getRequest('mark_color', 0), null, [
 					MARK_COLOR_RED => _('as Red'),
 					MARK_COLOR_GREEN => _('as Green'),
 					MARK_COLOR_BLUE => _('as Blue')
-				]);
+				]))->removeId();
 			}
 
 			$filterColumn1->addRow(_('Selected'), $tasks);
@@ -181,12 +187,17 @@ if ($data['itemids']) {
 		'resourcetype' => SCREEN_RESOURCE_HISTORY,
 		'action' => $data['action'],
 		'itemids' => $data['itemids'],
+		'pageFile' => (new CUrl('history.php'))
+			->setArgument('action', $data['action'])
+			->setArgument('itemids', $data['itemids'])
+			->getUrl(),
 		'profileIdx' => $data['profileIdx'],
 		'profileIdx2' => $data['profileIdx2'],
 		'from' => $data['from'],
 		'to' => $data['to'],
-		'filter' => getRequest('filter'),
-		'filter_task' => getRequest('filter_task'),
+		'page' => $data['page'],
+		'filter' => $data['filter'],
+		'filter_task' => $data['filter_task'],
 		'mark_color' => getRequest('mark_color'),
 		'plaintext' => $data['plaintext'],
 		'graphtype' => $data['graphtype']
@@ -223,6 +234,7 @@ else {
 			->hideFilterButtons()
 			->addVar('action', $data['action'])
 			->addVar('itemids', $data['itemids']);
+
 		$filter_tab = [
 			(new CFormList())->addRow(_('Graph type'),
 				(new CRadioButtonList('graphtype', (int) $data['graphtype']))
@@ -242,7 +254,6 @@ else {
 		$filter_form->addFilterTab(_('Filter'), $filter_tab);
 	}
 
-
 	if ($data['itemids']) {
 		if ($data['action'] !== HISTORY_LATEST) {
 			$historyWidget->addItem($filter_form);
@@ -271,4 +282,4 @@ else {
 	}
 }
 
-return $historyWidget;
+$historyWidget->show();
