@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,17 +19,30 @@
 **/
 
 
-$web_layout_mode = CView::getLayoutMode();
+/**
+ * @var CView $this
+ */
+
+$web_layout_mode = CViewHelper::loadLayoutMode();
 
 $widget = (new CWidget())->setWebLayoutMode($web_layout_mode);
 
-if (in_array($web_layout_mode, [ZBX_LAYOUT_NORMAL, ZBX_LAYOUT_FULLSCREEN])) {
+if ($web_layout_mode == ZBX_LAYOUT_NORMAL) {
 	$widget
 		->setTitle(_('Slide shows'))
+		->setTitleSubmenu([
+			'main_section' => [
+				'items' => [
+					'screens.php' => _('Screens'),
+					'slides.php' => _('Slide shows')
+				]
+			]
+		])
 		->addItem((new CList())
 			->setAttribute('role', 'navigation')
 			->setAttribute('aria-label', _x('Hierarchy', 'screen reader'))
 			->addClass(ZBX_STYLE_OBJECT_GROUP)
+			->addClass(ZBX_STYLE_FILTER_BREADCRUMB)
 			->addItem([
 				(new CSpan())->addItem(new CLink(_('All slide shows'), 'slideconf.php')),
 				'/',
@@ -44,14 +57,6 @@ if (in_array($web_layout_mode, [ZBX_LAYOUT_NORMAL, ZBX_LAYOUT_FULLSCREEN])) {
 		);
 }
 
-$controls = (new CList())
-	->addItem(
-		new CComboBox('config', 'slides.php', 'redirect(this.options[this.selectedIndex].value);', [
-			'screens.php' => _('Screens'),
-			'slides.php' => _('Slide shows')
-		])
-	);
-
 $favourite_icon = get_icon('favourite', [
 	'fav' => 'web.favorite.screenids',
 	'elname' => 'slideshowid',
@@ -60,28 +65,50 @@ $favourite_icon = get_icon('favourite', [
 
 $refresh_icon = get_icon('screenconf');
 
-if ($this->data['screen']) {
-	$refresh_icon->setMenuPopup(CMenuPopupHelper::getRefresh(
-		WIDGET_SLIDESHOW,
-		'x'.$this->data['refreshMultiplier'],
-		true,
-		[
-			'elementid' => $this->data['elementId']
-		]
-	));
-}
+$refresh_icon->setMenuPopup(CMenuPopupHelper::getRefresh(WIDGET_SLIDESHOW, 'x'.$this->data['refreshMultiplier'],
+	true, ['elementid' => $this->data['elementId']]
+));
 
-if (isset($this->data['isDynamicItems'])) {
-	$controls->addItem([
-		new CLabel(_('Group'), 'groupid'),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		$this->data['pageFilter']->getGroupsCB()
-	]);
-	$controls->addItem([
-		new CLabel(_('Host'), 'hostid'),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		$this->data['pageFilter']->getHostsCB()
-	]);
+$controls = null;
+
+if ($data['has_dynamic_widgets']) {
+	$controls = (new CList())
+		->addItem(new CLabel(_('Host'), 'hostid'))
+		->addItem(
+			(new CMultiSelect([
+				'name' => 'dynamic_hostid',
+				'object_name' => 'hosts',
+				'data' => $data['host'],
+				'multiple' => false,
+				'popup' => [
+					'parameters' => [
+						'srctbl' => 'hosts',
+						'srcfld1' => 'hostid',
+						'dstfld1' => 'dynamic_hostid',
+						'monitored_hosts' => 1
+					]
+				]
+			]))->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
+		);
+
+	zbx_add_post_js(
+		'jQuery("#dynamic_hostid").on("change", function() {'.
+			'var hosts = jQuery(this).multiSelect("getData"),'.
+				'url = new Curl("slides.php", false);'.
+
+			// Make URL.
+			'url.setArgument("elementid", '.$data['screen']['slideshowid'].');'.
+			'if (hosts.length) {'.
+				'url.setArgument("hostid", hosts[0].id);'.
+			'}'.
+			'else {'.
+				'url.setArgument("reset", "reset");'.
+			'}'.
+
+			// Push URL change.
+			'return redirect(url.getUrl(), "get", "", false, false);'.
+		'});'
+	);
 }
 
 $widget->setControls((new CList([
@@ -97,24 +124,24 @@ $widget->setControls((new CList([
 		)
 		->addItem($favourite_icon)
 		->addItem($refresh_icon)
-		->addItem(get_icon('fullscreen'))
+		->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode]))
 	))
 		->setAttribute('aria-label', _('Content controls'))
 ])));
 
-if (in_array($web_layout_mode, [ZBX_LAYOUT_NORMAL, ZBX_LAYOUT_FULLSCREEN])) {
-	$widget->addItem((new CFilter())
-		->setProfile($data['timeline']['profileIdx'], $data['timeline']['profileIdx2'])
-		->setActiveTab($data['active_tab'])
-		->addTimeSelector($data['timeline']['from'], $data['timeline']['to'])
+$widget
+	->addItem(
+		(new CFilter(new CUrl()))
+			->setProfile($data['timeline']['profileIdx'], $data['timeline']['profileIdx2'])
+			->setActiveTab($data['active_tab'])
+			->addTimeSelector($data['timeline']['from'], $data['timeline']['to'],
+				$web_layout_mode != ZBX_LAYOUT_KIOSKMODE)
+	)
+	->addItem(
+		(new CDiv((new CDiv())->addStyle('position: relative;margin-top: 20px;')->addClass('is-loading')))
+			->setId(WIDGET_SLIDESHOW)
 	);
-}
-
-$widget->addItem(
-	(new CDiv((new CDiv())->addClass('preloader')))
-		->setId(WIDGET_SLIDESHOW)
-);
 
 require_once dirname(__FILE__).'/js/monitoring.slides.js.php';
 
-return $widget;
+$widget->show();
