@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,27 +18,29 @@
 **/
 
 
-var CTree = Class.create();
+var CTree = function(name, nodes) {
+	this.name = name;
+	this.nodes = nodes;
+
+	if ((tree_init = cookie.read(name)) != null) {
+		var nodes = tree_init.split(this.cookie_delimiter);
+
+		for (var i = 0, size = nodes.length; i < size; i++) {
+			this.onStartSetStatus(nodes[i]);
+		}
+
+		this.onStartOpen(nodes);
+	}
+};
 
 CTree.prototype = {
-
 	name: null,
 	nodes: [],
-
-	initialize: function(name, nodes) {
-		this.name = name;
-		this.nodes = nodes;
-
-		if ((tree_init = cookie.read(name)) != null) {
-			var nodes = tree_init.split(',');
-
-			for (var i = 0, size = nodes.length; i < size; i++) {
-				this.onStartSetStatus(nodes[i]);
-			}
-
-			this.onStartOpen(nodes);
-		}
-	},
+	/*
+	 * Encoding values separated by a comma (%2C), makes the cookie very large in size. A dot, however, remains a dot.
+	 * Must be synced with class.cookie.js for consistency.
+	 */
+	cookie_delimiter: '.',
 
 	getNodeStatus: function(id) {
 		return (empty(this.nodes[id]) || this.nodes[id].status == 'close') ? 'close' : 'open';
@@ -54,7 +56,7 @@ CTree.prototype = {
 				var node = this.nodes[id];
 
 				if (!empty(node.status) && node.status == 'open') {
-					cookieData += id + ',';
+					cookieData += id + this.cookie_delimiter;
 				}
 			}
 
