@@ -18,17 +18,16 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+
 require_once dirname(__FILE__).'/include/func.inc.php';
 require_once dirname(__FILE__).'/include/defines.inc.php';
-require_once dirname(__FILE__).'/include/classes/json/CJson.php';
 require_once dirname(__FILE__).'/include/classes/user/CWebUser.php';
 require_once dirname(__FILE__).'/include/classes/core/CHttpRequest.php';
 
 $requestType = getRequest('type', PAGE_TYPE_JSON);
 if ($requestType == PAGE_TYPE_JSON) {
 	$http_request = new CHttpRequest();
-	$json = new CJson();
-	$data = $json->decode($http_request->body(), true);
+	$data = json_decode($http_request->body(), true);
 }
 else {
 	$data = $_REQUEST;
@@ -102,7 +101,7 @@ switch ($data['method']) {
 		$result = [];
 
 		$triggers = API::Trigger()->get([
-			'output' => ['triggerid', 'expression', 'priority'],
+			'output' => ['triggerid', 'priority'],
 			'triggerids' => $data['triggerids'],
 			'limit' => $config['search_limit']
 		]);
@@ -121,9 +120,9 @@ switch ($data['method']) {
 
 	/**
 	 * Create multi select data.
-	 * Supported objects: "applications", "hosts", "hostGroup", "templates", "triggers"
+	 * Supported objects: "applications", "hosts", "hostGroup", "templates", "triggers", "application_prototypes"
 	 *
-	 * @param string $data['objectName']
+	 * @param string $data['object_name']
 	 * @param string $data['search']
 	 * @param int    $data['limit']
 	 *
@@ -132,23 +131,32 @@ switch ($data['method']) {
 	case 'multiselect.get':
 		$config = select_config();
 
-		switch ($data['objectName']) {
+		switch ($data['object_name']) {
 			case 'hostGroup':
 				$options = [
-					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'output' => ['groupid', 'name'],
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'real_hosts' => array_key_exists('real_hosts', $data) ? $data['real_hosts'] : null,
+					'with_items' => array_key_exists('with_items', $data) ? $data['with_items'] : null,
+					'with_httptests' => array_key_exists('with_httptests', $data) ? $data['with_httptests'] : null,
+					'with_hosts_and_templates' => array_key_exists('with_hosts_and_templates', $data)
+						? $data['with_hosts_and_templates']
+						: null,
+					'with_monitored_triggers' => array_key_exists('with_monitored_triggers', $data)
+						? $data['with_monitored_triggers']
+						: null,
+					'with_triggers' => array_key_exists('with_triggers', $data) ? $data['with_triggers'] : null,
+					'templated_hosts' => array_key_exists('templated_hosts', $data) ? $data['templated_hosts'] : null,
+					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null,
-					'real_hosts' => array_key_exists('real_hosts', $data) ? $data['real_hosts'] : null
+					'preservekeys' => true
 				];
 				$hostGroups = API::HostGroup()->get($options);
 
 				if ($hostGroups) {
 					if (array_key_exists('enrich_parent_groups', $data)) {
-						$hostGroups = enrichParentGroups($hostGroups, [
-							'real_hosts' => null
-						] + $options);
+						$hostGroups = enrichParentGroups($hostGroups);
 					}
 
 					CArrayHelper::sort($hostGroups, [
@@ -163,15 +171,37 @@ switch ($data['method']) {
 				}
 				break;
 
+			case 'host_templates':
 			case 'hosts':
-				$hosts = API::Host()->get([
-					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
+				$options = [
 					'output' => ['hostid', 'name'],
 					'templated_hosts' => array_key_exists('templated_hosts', $data) ? $data['templated_hosts'] : null,
+					'with_items' => array_key_exists('with_items', $data) ? $data['with_items'] : null,
+					'with_httptests' => array_key_exists('with_httptests', $data) ? $data['with_httptests'] : null,
+					'with_triggers' => array_key_exists('with_triggers', $data) ? $data['with_triggers'] : null,
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
 					'limit' => $config['search_limit']
-				]);
+				];
 
+				if ($data['object_name'] === 'host_templates') {
+					$options['templated_hosts'] = true;
+				}
+
+				if (array_key_exists('with_monitored_triggers', $data)) {
+					$options += [
+						'with_monitored_triggers' => true
+					];
+				}
+
+				if (array_key_exists('with_monitored_items', $data)) {
+					$options += [
+						'with_monitored_items' => true,
+						'monitored_hosts' => true
+					];
+				}
+
+				$hosts = API::Host()->get($options);
 
 				if ($hosts) {
 					CArrayHelper::sort($hosts, [
@@ -187,37 +217,80 @@ switch ($data['method']) {
 				break;
 
 			case 'items':
-				$items = API::Item()->get([
+			case 'item_prototypes':
+				$options = [
 					'output' => ['itemid', 'hostid', 'name', 'key_'],
 					'selectHosts' => ['name'],
 					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
 					'templated' => array_key_exists('real_hosts', $data) ? false : null,
-					'webitems' => array_key_exists('webitems', $data) ? $data['webitems'] : null,
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
 					'limit' => $config['search_limit']
-				]);
+				];
 
-				if ($items) {
-					$items = CMacrosResolverHelper::resolveItemNames($items);
-					CArrayHelper::sort($items, [
-						['field' => 'name_expanded', 'order' => ZBX_SORT_UP]
-					]);
+				if ($data['object_name'] === 'item_prototypes') {
+					$records = API::ItemPrototype()->get($options);
+				}
+				else {
+					if (array_key_exists('webitems', $data)) {
+						$options['webitems'] = $data['webitems'];
+					}
+
+					$records = API::Item()->get($options);
+				}
+
+				if ($records) {
+					$records = CMacrosResolverHelper::resolveItemNames($records);
+					CArrayHelper::sort($records, ['name_expanded']);
 
 					if (array_key_exists('limit', $data)) {
-						$items = array_slice($items, 0, $data['limit']);
+						$records = array_slice($records, 0, $data['limit']);
 					}
 
-					foreach ($items as $item) {
+					foreach ($records as $record) {
 						$result[] = [
-							'id' => $item['itemid'],
-							'name' => $item['name_expanded'],
-							'prefix' => $item['hosts'][0]['name'].NAME_DELIMITER
+							'id' => $record['itemid'],
+							'name' => $record['name_expanded'],
+							'prefix' => $record['hosts'][0]['name'].NAME_DELIMITER
 						];
 					}
 				}
 				break;
 
+			case 'graphs':
+			case 'graph_prototypes':
+				$options = [
+					'output' => ['graphid', 'name'],
+					'selectHosts' => ['name'],
+					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
+					'templated' => array_key_exists('real_hosts', $data) ? false : null,
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'limit' => $config['search_limit']
+				];
+
+				if ($data['object_name'] === 'graph_prototypes') {
+					$records = API::GraphPrototype()->get($options);
+				}
+				else {
+					$records = API::Graph()->get($options);
+				}
+
+				CArrayHelper::sort($records, ['name']);
+
+				if (array_key_exists('limit', $data)) {
+					$records = array_slice($records, 0, $data['limit']);
+				}
+
+				foreach ($records as $record) {
+					$result[] = [
+						'id' => $record['graphid'],
+						'name' => $record['name'],
+						'prefix' => $record['hosts'][0]['name'].NAME_DELIMITER
+					];
+				}
+				break;
+
 			case 'templates':
 				$templates = API::Template()->get([
 					'editable' => isset($data['editable']) ? $data['editable'] : false,
@@ -259,8 +332,8 @@ switch ($data['method']) {
 
 			case 'applications':
 				$applications = API::Application()->get([
-					'hostids' => zbx_toArray($data['hostid']),
 					'output' => ['applicationid', 'name'],
+					'hostids' => zbx_toArray($data['hostid']),
 					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
 					'limit' => $config['search_limit']
 				]);
@@ -278,6 +351,37 @@ switch ($data['method']) {
 				}
 				break;
 
+			case 'application_prototypes':
+				$discovery_rules = API::DiscoveryRule()->get([
+					'output' => [],
+					'selectApplicationPrototypes' => ['application_prototypeid', 'name'],
+					'itemids' => [$data['parent_discoveryid']],
+					'limitSelects' => $config['search_limit']
+				]);
+
+				if ($discovery_rules) {
+					$discovery_rule = $discovery_rules[0];
+
+					if ($discovery_rule['applicationPrototypes']) {
+						foreach ($discovery_rule['applicationPrototypes'] as $application_prototype) {
+							if (array_key_exists('search', $data)
+									&& stripos($application_prototype['name'], $data['search']) !== false) {
+								$result[] = [
+									'id' => $application_prototype['application_prototypeid'],
+									'name' => $application_prototype['name']
+								];
+							}
+						}
+
+						CArrayHelper::sort($result, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
+
+						if (array_key_exists('limit', $data)) {
+							$result = array_slice($result, 0, $data['limit']);
+						}
+					}
+				}
+				break;
+
 			case 'triggers':
 				$host_fields = ['name'];
 				if (array_key_exists('real_hosts', $data) && $data['real_hosts']) {
@@ -408,13 +512,88 @@ switch ($data['method']) {
 		}
 		break;
 
+	case 'patternselect.get':
+		$config = select_config();
+		$search = (array_key_exists('search', $data) && $data['search'] !== '') ? $data['search'] : null;
+		$wildcard_enabled = (strpos($search, '*') !== false);
+		$result = [];
+
+		switch ($data['object_name']) {
+			case 'hosts':
+				$options = [
+					'output' => ['name'],
+					'search' => ['name' => $search.($wildcard_enabled ? '*' : '')],
+					'searchWildcardsEnabled' => $wildcard_enabled,
+					'preservekeys' => true,
+					'limit' => $config['search_limit']
+				];
+
+				$db_result = API::Host()->get($options);
+				break;
+
+			case 'items':
+				$options = [
+					'output' => ['name'],
+					'search' => ['name' => $search.($wildcard_enabled ? '*' : '')],
+					'searchWildcardsEnabled' => $wildcard_enabled,
+					'filter' => [
+						'value_type' => [ITEM_VALUE_TYPE_UINT64, ITEM_VALUE_TYPE_FLOAT],
+						'flags' => ZBX_FLAG_DISCOVERY_NORMAL
+					],
+					'templated' => array_key_exists('real_hosts', $data) ? false : null,
+					'webitems' => array_key_exists('webitems', $data) ? $data['webitems'] : null,
+					'limit' => $config['search_limit']
+				];
+
+				$db_result = API::Item()->get($options);
+				break;
+
+			case 'graphs':
+				$options = [
+					'output' => ['name'],
+					'search' => ['name' => $search.($wildcard_enabled ? '*' : '')],
+					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
+					'templated' => array_key_exists('real_hosts', $data) ? false : null,
+					'searchWildcardsEnabled' => $wildcard_enabled,
+					'limit' => $config['search_limit']
+				];
+
+				$db_result = API::Graph()->get($options);
+				break;
+		}
+
+		$result[] = [
+			'name' => $search,
+			'id' => $search
+		];
+
+		if ($db_result) {
+			$db_result = array_flip(zbx_objectValues($db_result, 'name'));
+
+			if (array_key_exists($search, $db_result)) {
+				unset($db_result[$search]);
+			}
+
+			if (array_key_exists('limit', $data)) {
+				$db_result = array_slice($db_result, 0, $data['limit']);
+			}
+
+			foreach ($db_result as $name => $id) {
+				$result[] = [
+					'name' => $name,
+					'id' => $name
+				];
+			}
+		}
+		break;
+
 	default:
 		fatal_error('Wrong RPC call to JS RPC!');
 }
 
 if ($requestType == PAGE_TYPE_JSON) {
 	if (isset($data['id'])) {
-		echo $json->encode([
+		echo json_encode([
 			'jsonrpc' => '2.0',
 			'result' => $result,
 			'id' => $data['id']
@@ -422,9 +601,7 @@ if ($requestType == PAGE_TYPE_JSON) {
 	}
 }
 elseif ($requestType == PAGE_TYPE_TEXT_RETURN_JSON) {
-	$json = new CJson();
-
-	echo $json->encode([
+	echo json_encode([
 		'jsonrpc' => '2.0',
 		'result' => $result
 	]);
